function InitDeployment()
  for faction in game.factions do
    faction.luaData.cityDeployment = {}
  end
end


-- Clear city deployment limits
function ResetDeployLimits()
  for faction in game.factions do
    for construction in faction.constructions do
      if construction.isPort then
        construction.city.hex.originalFaction.luaData.cityDeployment[construction.hex.id] = 0
      else
        construction.hex.originalFaction.luaData.cityDeployment[construction.hex.id] = 0
      end
    end
  end
end


function GetValidDeploymentHexes(unit, faction, potential)
  local validHexes = {}
  local cityHexes = {}
  local filledhexes = {}

  for construction in faction.constructions do
    if construction.pp > 0 or construction.isPort then
      cityHexes = game.map:GetHexesInRange(construction.hex, 1)
      for i,hex in pairs(cityHexes) do
        if IsValidDeploymentHex(hex, unit, faction, potential) ~= -1 then
          if filledhexes[hex.id] ~= 1 then
            table.insert(validHexes, hex)
            filledhexes[hex.id] = 1
          end
        end
      end
    end
  end

  if faction.luaData.colonial == 1 and GetFactionTransportmoves(faction) > 0 then
    for x = 0, (game.map.hexWidth - 1) do
      local hex = GetHex(x, game.map.hexHeight-1)
      if hex.terrain.type == Terrain.WATER then
        table.insert(validHexes, hex)
      end
    end
  end

  return validHexes
end

-- Highlight valid deployment hexes
function OLDIsValidDeploymentHex(hex, unit, faction, potential) --- old
  local i, adjacent, limit, validHex
  local valid = -1

  -- Is it a colonial unit and hex
  if faction.luaData.colonial == 1 and GetFactionTransportmoves(faction) > 0 then
    if hex.y == game.map.hexHeight - 1 and hex.terrain.type == Terrain.WATER then
      return hex
    end
  end

  -- Rail units only on rail hexes
  if unit ~= nil and unit.prototype.lua.chassis == "rail" and not hex.hasRailroad then
    return -1
  end

  -- Check for adjacent construction
  local adjacentHexes = {}
  adjacentHexes = game.map:GetHexesInRange(hex, 1)
  for i,adjacent in pairs(adjacentHexes) do
    if adjacent.unit ~= nil and adjacent.unit.alive and adjacent.unit.type ~= Unit.NAVAL and adjacent.unit.faction.alliance.id ~= faction.alliance.id then
      return -1
    end
    if adjacent.construction ~= nil then

      -- If its a port, find adjacent city's original owner
      local originalFaction, city
      if adjacent.construction.isPort then
        city = adjacent.construction.city
      else
        city = adjacent.construction
      end

      if city.hex.faction.id == faction.id and city.hex.originalFaction.id == faction.id then
        if IsConstructionValidDeploy(hex, adjacent.construction, unit, faction, potential) then
          valid = adjacent
        end
      end
    end
  end

  return valid
end

-- Highlight valid deployment hexes
function IsValidDeploymentHex(hex, unit, faction, potential)
  local i, adjacent, limit, validHex
  local valid = -1

  -- Is it a colonial unit and hex
  if faction.luaData.colonial == 1 and GetFactionTransportmoves(faction) > 0 then
    if hex.y == game.map.hexHeight - 1 and hex.terrain.type == Terrain.WATER then
      return hex
    end
  end

  -- Rail units only on rail hexes
  if unit ~= nil and unit.prototype.lua.chassis == "rail" and not hex.hasRailroad then
    return -1
  end

  -- Check for adjacent construction
  local adjacentHexes = {}
  adjacentHexes = game.map:GetHexesInRange(hex, 1)
  for i,adjacent in pairs(adjacentHexes) do
    if adjacent.unit ~= nil 
		and adjacent.unit.alive 
		and adjacent.unit.type ~= Unit.NAVAL 
		and adjacent.unit.faction.alliance.id ~= faction.alliance.id then
			return -1
    end
    if adjacent.construction ~= nil then

      -- If its a port, find adjacent city's original owner
      local originalFaction, city
      if adjacent.construction.isPort then
        city = adjacent.construction.city
      else
        city = adjacent.construction
      end

      if city.hex.faction ~= nil then
			if city.hex.faction.id == faction.id 
				and city.hex.originalFaction.id == faction.id then
					if IsConstructionValidDeploy(hex, adjacent.construction, unit, faction, potential) then
						valid = adjacent
					end
			end
	  else
		return -1
	  end
    end
  end

  return valid
end


function GetConstructionDeployLimit(construction)
  if construction.type == Construction.TYPE_CITY then
    return 2
  end
  if construction.type == Construction.TYPE_FORTRESS then
    return 1
  end
  if construction.type == Construction.TYPE_PORT then
    return 1
  end
  if construction.type == Construction.TYPE_CAPITAL then
    return 3
  end
  return 0
end

function GetCurrentDeploymentCount(construction)
  return construction.hex.originalFaction.luaData.cityDeployment[construction.hex.id]
end

--[[
function GetTheoreticalDeploymentConstruction(unit,deployHex,limits)
  -- Used by the AI to work out the constraints on deployment.
  if limits == nil then
    limits = unit.faction.luaData.cityDeployment
  end
  local adjacentHexes = {}
  adjacentHexes = game.map:GetHexesInRange(hex, 1)
  for i,adjacent in pairs(adjacentHexes) do
   if adjacent.construction ~= nil then

      -- We're not concerned with ports here
      local originalFaction = adjacent.originalFaction.id

      if adjacent.faction.id == faction.id and originalFaction == faction.id then
        local deployedCount = 0
        deployedCount = deployedCount + GetCurrentDeploymentCount(adjacent.construction)
        if limits[adjacent] ~= nil then
          deployedCount = deployedCount + limits[adjacent]
        end
        if deployedCount <= GetConstructionDeployLimit(adjacent.construction) then
          return adjacent
        end
      end
    end
end--]]

function IsConstructionValidDeploy(hex, construction, unit, faction, potential)

  -- Check if city is originally owned by this faction & produces PP
  if not construction.isPort then
    if construction.hex.originalFaction.id ~= faction.id or construction.pp == 0 then
      return false
    end
  else
    if construction.city.hex.originalFaction.id ~= faction.id or construction.city.pp == 0 then
      return false
    end
  end

  -- Define how many units can be deployed for the construction type
  local limit = GetConstructionDeployLimit(construction)

  -- Check if deploy limit is exceeded
  if faction.luaData.cityDeployment[construction.hex.id] == nil then
    faction.luaData.cityDeployment[construction.hex.id] = 1
  end
  if faction.luaData.cityDeployment[construction.hex.id] >= limit or (unit~= nil and faction.luaData.colonial == 1 and construction.hex.y >= 54 and (faction.luaData.transportmovesCurrent < 1 or unit.prototype.lua.colonial ~= true)) then
    return false
  end

  -- If hex is not occupied or we want potential deploy hexes.
  -- If unit is nil, assume land unit (not rail)
  if hex.unit == nil or (potential == true and hex.unit.mp > 0) then
    if unit ~= nil and (unit.prototype.lua.chassis == "water" or unit.prototype.lua.chassis == "sub") then
      if construction.type == Construction.TYPE_PORT and hex.terrain.type == Terrain.WATER then
        return true
      end
    else
      if construction.type ~= Construction.TYPE_PORT and hex.terrain.type ~= Terrain.WATER and hex.faction.alliance.id == faction.alliance.id then
        if unit ~= nil and unit.prototype.lua.chassis == "rail" and hex.faction.id == faction.id  then
          if hex.hasRailroad then
            return true
          end
        else
          return true
        end
      end
    end
  end
  return false
end

-- Place unit & increment deployment limit of city
function LimitedDeployUnit(faction, slot, hex)
--LogUi("LIMITDEPLOYSTART")
  local i = 0
  local unit
  for slotunit in faction.production_que do
    if slot == i then
      unit = slotunit
      break
    end
    i = i + 1
  end
  LogUi("LIMITDEPLOY CHECKHEX1")
  local city = IsValidDeploymentHex(hex, unit, faction)
  LogUi("LIMITDEPLOY CHECKHEX2")

  if city ~= -1 then
      print("attempt deploy")

    GetPlayer():DeployUnit(faction, slot, hex)
    local succes = hex.unit ~= nil;

    if succes == true then

      -- Lower city deployment limit
      if city.construction ~= nil then
        faction.luaData.cityDeployment[city.id] = faction.luaData.cityDeployment[city.id] + 1
		 if faction.luaData.colonial == 1 and city.y >= 54 then
			        UseTransportmove(faction)
		end
		
      else
        print("deployed on hex not adjacent to city: " .. city.x .. "," .. city.y)
		      LogUi("deployed on hex not adjacent to city: " .. city.x .. "," .. city.y)

      end

      -- Colonial?
      if hex.unit.prototype.name == "transport" then
        UseTransportmove(faction)
        for cargo in hex.unit.cargo.units do
          ApplySlotState(cargo, slot)
          cargo.efficiency = math.floor(cargo.efficiency * 7 / 10)
          hex.unit.efficiency = cargo.efficiency
          break
        end
      else
        ApplySlotState(hex.unit, slot)
      end

      -- Enable hotkeys
      if faction.alliance.id == playerAlliance.id then
        ui.hotkeysActive = 1
      end

      -- Events
	--  LogUi("EVENTTRIGGER")
      TriggerDeployEvents(hex.unit, hex)
	--  LogUi("EVENTTRIGGER2")	
      return succes
    end
  end
 -- LogUi("LIMITDEPLOYEND")
  return false
end


function SetColonialFaction(factionId)
  faction = game:GetFactionById(factionId)
  faction.luaData.colonial = 1
end

function CanUnitDisband(unit)

  -- Only during owner's turn
  if unit.faction.alliance.id ~= game.currentAlliance.id then
    return false
  end

  if unit.alive and unit.mp > 0 and unit.supply > 0 then
    if unit.type == Unit.NAVAL then
      -- Naval units can only be repaired in port
      if unit.hex.construction ~= nil and unit.hex.construction.city.hex.faction.alliance.id == unit.alliance.id then
        return true
      end
    else
      return true
    end
  end
  return false

end

function DisbandUnit(unit)

  TriggerDisbandEvents(unit)

  -- Check if it is in transport
  local carrier
  if unit.cargo ~= nil then
    carrier = unit
    for cargo in unit.cargo.units do
      unit = cargo
      break
    end
  end

  local PPrefund, MPrefund = GetDisbandRefund(unit)
  unit.faction:ConsumeProductionPoints(-PPrefund)
  unit.faction:ConsumeManpower(-MPrefund)
  if unit.commander ~= nil then
    DetachCommanderFromUnit(unit, unit.commander, true)
  end

  if unit.alliance.id == playerAlliance.id then
    GetPlayer():DisbandUnit(unit)
  else -- for AI
    unit:Kill()
  end

  -- If it was in transport
  if carrier ~= nil then
    if unit.alliance.id == playerAlliance.id then
      GetPlayer():DisbandUnit(carrier)
    else
      carrier:Kill()
    end
  end
end

function GetDisbandRefund(unit)
  local PPrefund
  local MPrefund
  if unit.type  ~= Unit.NAVAL then
   
	---PP REFUND----
		if 	unit.hex ~= nil
			and unit.hex.originalFaction.alliance.id == unit.faction.alliance.id then -- if unit is on original own alliance hex
			if unit.hex.originalFaction.id == unit.faction.id then   
				PPrefund = math.floor(unit.prototype.costPP * unit.hp * 35 / 10000) --- if unit is on own homeland
			else
				PPrefund = math.floor(unit.prototype.costPP * unit.hp * 20 / 10000) -- only on allied nations homeland
			end
		else
			PPrefund = 0
		end
	
		--- MP REFUND ----
		if unit.hex ~= nil
			and unit.hex.originalFaction.alliance.id == unit.faction.alliance.id then
			if unit.hex.originalFaction.id == unit.faction.id then
				MPrefund = math.floor(unit.prototype.costMP * unit.hp * 100 / 10000) --- on homeland
			else 
				MPrefund = math.floor(unit.prototype.costMP * unit.hp * 50 / 10000)  --- on allied nations homeland
			end
		else
			MPrefund = 0
		end
  else
		PPrefund = math.floor(unit.prototype.costPP * unit.hp * 25 / 10000)  --- disband navy PP
		MPrefund = math.floor(unit.prototype.costMP * unit.hp * 100 / 10000) ---- disband navy MP	
  end
  return PPrefund, MPrefund
end





function GetPortCity(hex)
  if hex.construction ~= nil and hex.construction.type == Construction.TYPE_PORT then
    local originalFaction
    adjacentCities = game.map:GetHexesInRange(hex, 1)
    for i,city in pairs(adjacentCities) do
      if city.construction ~= nil and city.construction.type ~= Construction.TYPE_PORT then
        return city
      end
    end
  end
end

function SpawnUnit(unitId, faction, x, y, facing, hitpoints)
  local succes = false

  -- Unit terain type
  local unitType = "land"
  for i, v in pairs(data.units) do
    if data.units[i].id == unitId then
      unitType = data.units[i].type
      break
    end
  end

  -- Check if hex OR adjacent hex is unoccupied
  local hex = GetHex(x, y)
  if hex.unit == nil then
    succes = true
  else
    for i = 0, 5 do
      hex = GetAdjacent(x, y, i)
      if hex ~= nil then
        if hex.unit == nil then
          if unitType == "naval" and hex.terrain.type == Terrain.WATER then
            succes = true
            break
          end
          if unitType ~= "naval" and hex.terrain.type ~= Terrain.WATER then
            succes = true
            break
          end
        end
      end
    end
  end

if succes then
    unit = faction:SpawnUnit(unitId, hex.x, hex.y, facing)
 if unit ~= nil then
   ---   
	 if unit.luaData.experience == nil then
		unit.luaData.experience = 0
	end
	if faction.luaData.collaps ~= nil then
		unit.luaData.experience =  (13 - faction.luaData.collaps) *10
			if unit.luaData.experience == nil then
				unit.luaData.experience = 0
			elseif unit.luaData.experience <0 then
			
				unit.luaData.experience = 0
			end
	end
		if unit.luaData.experience <0 then
			unit.luaData.experience = 0
		end
		if unitType == "naval" then
			unit.luaData.endurance = 10
		end
		if game.date.year == 1914 
			and unit.prototype.name == "battlecruiser" 
			and (unit.faction.id == 2 or unit.faction.id == 3 or unit.faction.id == 5) then
			
			unit.luaData.experience = math.random(150,250)
		end
		
   ---
	
	  if hitpoints < 100 then
        if unit:Damage(100 - hitpoints) then
          return true
        else
          return false
        end
      end
    end
  end
end


function PurchaseUnit(faction, proto, ignoreCost)
local nozep = 0
local nobomb = 0
local nosub = 0
local notank = 0
local notrain = 0
local nofighter = 0

local zepcounter = 0
local bombcounter = 0
local subcounter = 0
local tankcounter = 0
local traincounter  = 0
local fightercounter = 0

local bzepcounter = 0
local bbombcounter = 0
local bsubcounter = 0
local btankcounter = 0	
local btraincounter = 0
local bfightercounter = 0
	  
local noinf = 0
local britain = game:GetFactionById(1)
local germany = game:GetFactionById(2)
local usa = game:GetFactionById(10)

	

  -- Check if sufficient resources are available
--------------------------------
  
  if faction.alliance.id == player.alliance.id  then
	 -- and (faction.id == 10 or faction.luaData.unitLevel["zeppelin"] ~= nil) --- US cannot build zeps
--[[	  and faction.luaData.unitLevel["bomber"] ~= nil 
	  and faction.luaData.unitLevel["armouredtrain"] ~= nil 
	  and faction.luaData.unitLevel["fighter"] ~= nil 
	  
	  and (faction.id == 10 
				or faction.id == 1
				or faction.id == 0
				or faction.id == 8
				or faction.luaData.unitLevel["submarine"] ~= nil) --- US cannot buld subs
	  and faction.luaData.unitLevel["armour"] ~= nil then ]]        -- and game.type = Game.TYPE_SINGLE then
	 
--	 for faction in player.alliance.factions do
		
		LogUi("OKAY!")
		-------------------ON MAP--------------------
		for unit in faction.units do
			if unit.hex ~= nil and unit.prototype.name == "zeppelin" then
				zepcounter = zepcounter + 1
			end
			if unit.hex ~= nil and unit.prototype.name == "bomber" then
				bombcounter = bombcounter + 1
			end
			
			if unit.hex ~= nil and unit.prototype.name == "submarine" then
				subcounter = subcounter + 1
			end
			if unit.hex ~= nil and unit.prototype.name == "armouredtrain" then
				traincounter = traincounter + 1
			end
			if unit.hex ~= nil and unit.prototype.name == "armour" then
				tankcounter = tankcounter + 1
			end
			if 	unit.cargo  ~= nil then
				for cargo in unit.cargo.units do 
					if cargo.prototype.name == "armour" then
						tankcounter = tankcounter + 1
					end	
				end	
			end	
			if unit.hex ~= nil and unit.prototype.name == "fighter" then
				fightercounter = fightercounter + 1
			end
			
		end
		--------------PROD QUEUE--------------------
		for slotunit in faction.production_que do
			
			if slotunit.prototype.name == "zeppelin" then
				bzepcounter = bzepcounter + 1
			end	
			if slotunit.prototype.name == "bomber" then
				bbombcounter = bbombcounter + 1
			end
			
			if slotunit.prototype.name == "submarine" then
				bsubcounter = bsubcounter + 1
			end
			if slotunit.prototype.name == "armouredtrain" then
				btraincounter = btraincounter + 1
			end
			
			if slotunit.prototype.name == "armour" then
				btankcounter = btankcounter + 1
			--	LogUi("TANK NUMBER: " .. btankcounter)
			end
			if slotunit.prototype.name == "fighter" then
				bfightercounter = bfightercounter + 1
			end
			
			
		end
		--LogUi("ZEPS on queue:" .. bzepcounter)
		--LogUi("BOMBAS on queue:" .. bbombcounter)
		--LogUi("SUBS on queue:" .. bsubcounter)
		
--	end
		zepcounter = zepcounter +bzepcounter
		bombcounter = bombcounter + bbombcounter
		subcounter = subcounter + bsubcounter
		tankcounter = tankcounter + btankcounter
		traincounter = traincounter + btraincounter
		fightercounter = fightercounter + bfightercounter
		

		if 	tankcounter >= 1 
			and tankcounter >=  faction.luaData.unitLevel["armour"]   then
			notank = notank +1	 		
		end
		if 	traincounter  >= 1  
			and traincounter >= faction.luaData.unitLevel["armouredtrain"]   then
			notrain = notrain +1	 		
		end
		if 	fightercounter  >= 1  
			and fightercounter >= faction.luaData.unitLevel["fighter"]   then
			nofighter = nofighter +1	 		
		end
		if 	zepcounter  >= 1 
			and zepcounter >= faction.luaData.unitLevel["zeppelin"]   then
			nozep = nozep +1
			--LogUi("GERMAN ZEPTECHS: " .. faction.luaData.unitLevel["zeppelin"])
		end
		if 	bombcounter >= 1 
			and bombcounter >=  faction.luaData.unitLevel["bomber"]   then
			nobomb = nobomb +1	     
			--LogUi("GERMAN BOMBTECHS: " .. germany.luaData.unitLevel["bomber"])
		end
				
		if faction.id == 2 
			and	subcounter >= faction.luaData.unitLevel["submarine"] * 2 then
		--	LogUi("GERMAN SUBTECHS: " .. faction.luaData.unitLevel["submarine"])
				nosub = nosub +1		
		end
		if faction.id == 3 
			and	subcounter >= faction.luaData.unitLevel["submarine"]  then
		--	LogUi("GERMAN SUBTECHS: " .. faction.luaData.unitLevel["submarine"])
				nosub = nosub +1		
		end
		
	  end
		if britain.alliance.id == player.alliance.id 
			and GetEvent("PalsBattalions") == 0 then
		 	         -- PopupMessage("nobuildinfantry")
			noinf = 1
		end
	  
	  
-------------------------------------	  
	if (ignoreCost == true 
			or (faction.pp >= proto.costPP and faction.mp >= proto.costMP))  
		and (faction.id ~= 1 
				or ((faction.id == 1 and proto.id ~= 1) 
				or (faction.id == 1 and proto.id == 1 and noinf ==0))) 
		and (proto.id  ~= 15 
				and proto.id ~= 14 
				and proto.id ~= 13
				and proto.id ~= 8 
				and proto.id ~= 2 
				and proto.id ~= 5) 
		
			or ((proto.id == 15 and nozep == 0) 
			or (proto.id== 14 and nobomb == 0)
			or (proto.id == 8 and nosub == 0)
			or (proto.id == 5 and notrain == 0)
			or (proto.id == 2 and notank == 0)
			or (proto.id == 13 and nofighter == 0)) then
		
		
		if faction:BuyUnit(proto.id)  
			and ((proto.id  ~= 15 
					and proto.id ~= 14 
					and proto.id ~= 13
					and proto.id ~= 8
					and proto.id ~= 5	
					and proto.id ~= 2) 
				or 
					((proto.id == 15 and nozep == 0) 
						or (proto.id== 14 and nobomb == 0)
						or (proto.id == 13 and nofighter == 0)
						or (proto.id == 8 and nosub == 0)
						or (proto.id == 5 and notrain == 0)
						or (proto.id == 2 and notank == 0)))     then
		
							AddSlotState(faction)
							return true			
		else
						
		end
else
		
		if faction.id == 1 
			and noinf == 1 
			and proto.id == 1 then
			PopupMessage("nobuildinfantry")
		
		
		elseif nozep ~= 0 and proto.id == 15 then
	          PopupMessage("nobuildzepz")
		
		elseif nobomb ~= 0 and proto.id == 14 then
			PopupMessage("nobuildbombaz")
			
		elseif nosub ~= 0 and proto.id == 8 then
			PopupMessage("nobuildsubz")	
			
		elseif notank ~= 0 and proto.id == 2 then
			PopupMessage("nobuildtankz")
		
		elseif notrain ~= 0 and proto.id == 5  then
			PopupMessage("nobuildtrainz")		

		elseif nofighter ~= 0 and proto.id == 13  then
			PopupMessage("nobuildfighters")					
		
		end	
end
		
end



function CancelUnit(faction, unit, slot)
  -- Refund unfinished % of PP
  local refundPP = unit.prototype.costPP * (unit.turns) / unit.prototype.turns
  -- Make sure refund is always >= disband refund
  if refundPP < unit.prototype.costPP / 2 then
    refundPP = unit.prototype.costPP / 2
  end
  faction:ConsumeProductionPoints(-refundPP)

  -- Refund all MP
  faction:ConsumeManpower(-unit.prototype.costMP)

  faction:CancellUnitProduction(slot)

  ClearSlot(faction, slot)
end



function AddSlotState(faction)

  local slot = 0
  if faction.luaData.slots == nil then
    faction.luaData.slots = {}
  else
    for _, _ in pairs(faction.luaData.slots) do
      slot = slot + 1
    end
  end

  faction.luaData.slots[slot] = {}
  faction.luaData.slots[slot].quality = GetMPQuality(faction)
  faction.luaData.slots[slot].techs = utils.deepcopy(faction.luaData.factionTechs)
 
end

function ApplySlotState(unit, slot)
  if unit.faction.luaData.slots[slot] ~= nil then
    unit.luaData.quality = unit.faction.luaData.slots[slot].quality
    unit.efficiency = unit.luaData.quality
    UpdateUnit(unit, unit.faction.luaData.slots[slot].techs)
    ClearSlot(unit.faction, slot)
  end
end

function ClearSlot(faction, slot)
  while faction.luaData.slots[slot] ~= nil do
    faction.luaData.slots[slot] = faction.luaData.slots[slot+1]
    slot = slot + 1
  end
end


function CanPurchaseUnit(faction, proto, availablePP, availableMP)

  -- Check if it's a type that is not available for purchase
  if proto.lua.noPurchase then
    return false
  end

  -- Check if the unit has been unlocked
  local unitLevel = faction.luaData.unitLevel[proto.name]
  if unitLevel == 0 then
    return false
  end

  -- Check if faction has enough resources
  if availablePP == nil then
    availablePP = faction.pp
  end
  if availableMP == nil then
    availableMP = faction.mp
  end
  if proto.costPP > availablePP or proto.costMP > availableMP then
    return false
  end

  return true
end
