function InitResources()
  for faction in game.factions do
    faction.luaData.entrenchMax = 1

    faction.luaData.startingMP = faction.mp
    faction.luaData.qualityMP = 3

    -- Count capitals
    faction.luaData.capitals = 0
    for construction in faction.constructions do
      if construction.type == Construction.TYPE_CAPITAL then
        faction.luaData.capitals = faction.luaData.capitals + 1
      end
    end
    if (faction.luaData.capitals > 0 or IsNewFaction(faction) == true) then
      faction.luaData.productionLost = 0
    else
      faction.luaData.productionLost = 1
    end

    faction.luaData.warEffort = 100

    -- Set contruction hitpoints based on PP output
    SetConstructionHP(faction)
    CheckManpowerLevel(faction)
  end

end


	
 function DoMPIncome(faction)
      local growthrate = 2 -- 0.002  -- Growthrate 5% of MPmax per year => ca 0.2 % per turn (when ca 25 turns = 1 year)
      local MPIncome = math.ceil(growthrate * data.factions[faction.id+1].manpowerMax / 1000)  -- small factions get +1 MP per turn due to rounding up
    
	if game.date.year == 1914 then
		faction:ConsumeManpower(-MPIncome)
    elseif game.date.year >= 1915 then
		faction:ConsumeManpower(-MPIncome *2)
	end
	
	end

function ManageFinances(alliance)

  if alliance.id ~= 0 then

    for faction in alliance.factions do

      -- Check if Manpower reserve drops below a certain %
	  DoMPIncome(faction)   ---                                                    <--Our new function !!!
      CheckManpowerLevel(faction)

      PayFactionIncome(faction)

      -- Extra income for AI on highest difficulty level
      if game.type == Game.TYPE_SINGLE 
		and alliance.id ~= playerAlliance.id then
        local percentage = 50

        -- start with lower bonuses first 5 turns
        if game.turn <= 5 then
          percentage = percentage - percentage / game.turn
        end
        if game:GetAlliancePlayer(alliance).difficulty >= 1 
			and player.alliance.id == 1 
			and game.type == Game.TYPE_SINGLE then
			GiveExtraIncome(faction, percentage)
		elseif game:GetAlliancePlayer(alliance).difficulty >= 1 then
			GiveExtraIncome(faction, -percentage* 0.25)  -- *0.65
        end
      end

      SpendFactionResearchUpkeep(faction)

      SpendFactionUpkeep(faction)

      CheckDebtMoraleEffect(faction)

      ProduceAmmunition(faction)

      RepairConstructions(faction)

    end

  end

end

function PrintResources()
  local allianceIncome = {}
  allianceIncome[1] = 0
  allianceIncome[2] = 0
  for faction in game.factions do
    LogUi("")
    LogUi("NATION: " .. faction.name)
    LogUi("**********************")
    LogUi("Income Cities: " .. GetIncomePP(faction))
    local convoyIncome = 0
    if faction.luaData.convoys ~= nil then
    for _, convoy in pairs(faction.luaData.convoys) do
      convoyIncome = convoyIncome + convoy.cargo / convoy.interval
    end
    end
    LogUi("Income Convoys: " .. convoyIncome)
    LogUi("Total Income: " .. GetIncomePP(faction) + convoyIncome)
    if faction.id == 0 or faction.id == 1 or faction.id == 4 or faction.id == 7 or faction.id == 8 or faction.id == 18 then
      allianceIncome[1] = allianceIncome[1] + GetIncomePP(faction) + convoyIncome
    end
    if faction.id == 2 or faction.id == 3 or faction.id == 5 or faction.id == 19 then
      allianceIncome[2] = allianceIncome[2] + GetIncomePP(faction) + convoyIncome
    end
  end
    LogUi("")
    LogUi("ENTENTE ALLIES")
    LogUi("**********************")
    LogUi("Income : " .. allianceIncome[1])
    LogUi("")
    LogUi("CENTRAL POWERS")
    LogUi("**********************")
    LogUi("Income : " .. allianceIncome[2])

end

-- Get total upkeep cost of faction's current units + units in queue
function GetUpkeep(faction, pureArmyCost)
  local upkeep = 0
  
  for unit in faction.units do
    if unit.alive and unit.prototype.upkeepPP > 0 and not (unit.type == Unit.NAVAL and unit.hex.construction ~= nil) then

	  local temp = 0
      if unit.hp < 80 then
        temp = math.floor(unit.prototype.upkeepPP * unit.hp / 100)
        if temp == 0 then
          temp = 1
        end
      else
        temp = unit.prototype.upkeepPP
      end
      upkeep = upkeep + temp
    end
  end
  for unit in faction.production_que do
    upkeep = upkeep + unit.prototype.upkeepPP
  end
  

  -- Free 10 PP
  if not pureArmyCost then
    if upkeep > gameplay.FreeUpkeep then
      upkeep = upkeep - gameplay.FreeUpkeep
    else
      upkeep = 0
    end
  end
  return upkeep
end




function GetIncomePP(faction, ignoreWarEffort)
  local income = 0
  if ignoreWarEffort then
    income = math.ceil(GetFactionProduction(faction) * GetMPOutput(faction) / 100)
  else
    income = math.ceil(GetFactionProduction(faction) * GetMPOutput(faction) / 100 * faction.luaData.warEffort / 100)
  end

  return income
end

function GetFactionProduction(faction)
  local income = 0
  for construction in faction.constructions do
    income = income + GetConstructionProduction(construction)
  end
  return income
end

function OLDGetConstructionProduction(construction) --- old version
  if construction == nil then
    return 0
  end
  if construction.type ~= Construction.TYPE_PORT then
    if construction.hex.faction.id == construction.hex.originalFaction.id then
      return math.floor(construction.efficency / 10)
    else
      return math.floor(construction.efficency / 20)
      --test 0 income for captured cities
      --return 0
    end
  end
  return 0
end

function GetConstructionProduction(construction)
  if construction == nil then
    return 0
  end
  if construction.type ~= Construction.TYPE_PORT then
    if construction.hex.faction ~= nil
		and construction.hex.originalFaction ~= nil then
		if construction.hex.faction.id == construction.hex.originalFaction.id then
			return math.floor(construction.efficency / 10)
		else
			return math.floor(construction.efficency / 20)

		end
	end
  end
  return 0
end


function PayFactionIncome(faction)
  --print(faction.name .." ".. GetIncomePP(faction))
  faction:ConsumeProductionPoints(-GetIncomePP(faction))--GetExtraAIIncome(faction))
end

function SpendFactionUpkeep(faction)
  faction:ConsumeProductionPoints(GetUpkeep(faction))
end

function SpendFactionResearchUpkeep(faction)
  faction:ConsumeProductionPoints(GetResearchUpkeep(faction))
end

function RefundUpkeepDamagedUnits(faction)
  local refund = 0
  for unit in faction.units do
    if unit.alive and unit.hp < 80 then
      local temp = unit.prototype.upkeepPP - math.floor(unit.prototype.upkeepPP * unit.hp / 100)
      if temp == unit.prototype.upkeepPP then
        temp = unit.prototype.upkeepPP - 1
      end
      refund = refund + temp
    end
  end
  faction:ConsumeProductionPoints(-refund)
end


function GetMPOutput(faction)
  local minimum = 50
  return minimum + math.min(minimum, math.ceil(faction.mp * minimum / data.factions[faction.id+1].manpowerMax))
end


---- lower is orig
function CheckManpowerLevel(faction)
local malus = 0

  if faction.luaData.qualityMP == 3 then
    if GetMPOutput(faction) < 90 then
      faction.luaData.qualityMP = 2
      if playerAlliance ~= nil and faction.alliance.id == playerAlliance.id and game.turn > 1 then
        PopupManpowerDropOnTurnReport(faction)
      end
    end
  end
  if faction.luaData.qualityMP == 2 then
    if GetMPOutput(faction) < 85 then
      faction.luaData.qualityMP = 1
      if playerAlliance ~= nil and faction.alliance.id == playerAlliance.id and game.turn > 1 then
        PopupManpowerDropOnTurnReport(faction)
      end
    end
  end
  if faction.luaData.qualityMP == 1 then
		
		if faction.luaData.collaps ~= nil then
			if faction.luaData.collaps >0 then
				malus = faction.luaData.collaps 
			end
		end	
    if GetMPOutput(faction) < 70 + malus  then
      faction.luaData.qualityMP = 0
      if playerAlliance ~= nil and faction.alliance.id == playerAlliance.id and game.turn > 1 then
        PopupManpowerDropOnTurnReport(faction)
      end
    end
  end
end







function PopupManpowerDropOnTurnReport(faction)
  faction.luaData.showManpowerWarning = 1
end

function SetFactionCollaps(id, collaps)
  local faction = game:GetFactionById(id)
  if faction.luaData.collaps == nil then
    faction.luaData.collaps = 0
    
    --faction.luaData.id = faction.id
  end
end

function GetFactionCollaps(faction)
   
     local faction = game:GetFactionById(id)

   if faction.luaData.collaps == nil then
    faction.luaData.collaps = 0
    
    --faction.luaData.id = faction.id
  end
  
  
  return(faction.luaData.collaps)
end



function GetMPQuality(faction)
  if faction.luaData.qualityMP == 0 then
    return 60
  elseif faction.luaData.qualityMP == 1 then
    return 80
  
  elseif faction.luaData.qualityMP == 2 then
	return 90
  
  else
    return 100
  end
end



function GetFactionCollaps(faction)
  --  local faction = game:GetFactionById(faction.id)

  local value = faction.luaData.collaps
  
  if faction.luaData.collaps == nil then
  value = 0
  end
  
  return value
end

function CheckDebtMoraleEffect(faction)
  local dept = -faction.pp
  if dept > 0 then
    -- Hack to avoid AI paying dept in morale
    if faction.alliance.id == playerAlliance.id or game.type == Game.TYPE_PBEM then
      DeptMoraleEffect(faction, -dept / 10)
      faction:ConsumeProductionPoints(-dept)
    elseif dept > 25 then
      faction:ConsumeProductionPoints(-dept+10)
    end
    if faction.alliance.id == playerAlliance.id then
      AddToReportQueue(playerAlliance, "national_deficit", "../flags/flag_" .. faction.name .. "_16")
    end
  end
end

-- A construction max hp = pp production * 10
function SetConstructionHP(faction)
  for construction in faction.constructions do
    construction:SetEfficency(construction.pp * 10)
  end
end

-- A construction repairs X hp per turn
function RepairConstructions(faction)
  for construction in faction.constructions do
    -- If construction is damaged
    if construction.efficency < construction.pp * 10 then
      -- Repair hp
      local repairPoints = 2
      if construction.efficency + repairPoints < construction.pp * 10 then
        construction:SetEfficency(construction.efficency + repairPoints)
      else
        construction:SetEfficency(construction.pp * 10)
      end
    end
  end
end

function CanFactionProduce(faction)
  if (faction.luaData.capitals > 0 or IsNewFaction(faction) == true) then
    return true
  else
    return false
  end
end


function GiveExtraIncome(faction, percentage)
  if GetIncomePP(faction) > GetUpkeep(faction) then
    local bonus = (math.max(0,GetIncomePP(faction) - GetUpkeep(faction))) * percentage / 100
    faction:ConsumeProductionPoints(-bonus)
    print("AI PP bonus: " .. bonus)
  end
  if game.turn == 1 then
    GiveOneOffBonuses(faction,faction.alliance)
  end
end

function GiveOneOffBonuses(faction,alliance)
  if game.type == Game.TYPE_SINGLE 
	and alliance.id ~= playerAlliance.id 
	and game:GetAlliancePlayer(alliance).difficulty >= 1 then
    -- add bonuses to mp & morale
    if faction.luaData.startingMP == faction.mp then -- extra step for multiple declarations of war
      faction:ConsumeManpower(-faction.mp*0.1)
      --ChangeFactionMorale(faction,faction.morale*0.15)
      Log("Giving bonuses to: "..faction.name)
    end
  end
end

function GetFactionEconomicStrength(faction)
  local value = 0

  -- Cities
  for construction in faction.constructions do
    value = value + GetConstructionProduction(construction)
  end

  

  -- Convoys
  for _, convoy in pairs(faction.luaData.convoys) do
    value = value + convoy.cargo / convoy.interval
  end

  return value
end

function GetWarEffort(factionId)
  local faction = game:GetFactionById(factionId)
  return faction.luaData.warEffort
end

function SetWarEffort(factionId, amount)
  local faction = game:GetFactionById(factionId)
  faction.luaData.warEffort = amount
  if faction.alliance.id == playerAlliance.id and game ~= nil and game.turn > 1 then
    AddToReportQueue(playerAlliance, "war_effort_" .. amount, "../flags/flag_" .. faction.name .. "_16")
  end
end
