function GetValidTargets(unit, considerAmmo, from, useFOW) --useFOW is an optional parameter

  if from == nil then
    from = unit.hex
  end

  if considerAmmo and GetUnitAmmoRequirement(unit) > unit.faction.am and unit.type ~= Unit.NAVAL then
    return {}
  end

  local hexes = game.map:GetHexesInRange(from, unit.stats.range)
  local validHexes = {}
  for i,hex in pairs(hexes) do
    -- Ugly hack for armoured train, should probably be for all rail units with range <= 1
    if unit.prototype.name ~= "armouredtrain" or hex.hasRailroad then
      if IsValidTarget(unit, hex, considerAmmo, useFOW) then
        table.insert(validHexes, hex)
      end
    end
  end
  return validHexes
end

function GetUnitAmmoRequirement(unit)
  return math.ceil(unit.stats.ammunition * unit.hp / 100)
end

function IsValidTarget(unit, hex, considerAmmo, useFOW, target, attackerNotMoved)
  if target == nil then
    target = hex.unit
  end

  if considerAmmo and GetUnitAmmoRequirement(unit) > unit.faction.am then
    if unit.type ~= Unit.NAVAL or (target ~= nil and target.type ~= Unit.NAVAL) then
      return false
    end
  end

  if attackerNotMoved == nil then
    attackerNotMoved = (unit.mp > 0)
  end

  -- If hex is visible
  
  
  if unit.faction.alliance:IsVisible(hex) or useFOW == false then
    -- And there's a unit
    if target ~= nil and target.faction.alliance.id ~= unit.faction.alliance.id and target.faction.alliance.id ~= 0 then
      if target.type ~= Unit.NAVAL then
        if unit.prototype.lua.stats["groundattack"] ~= nil then
          return true
        end
        -- Transport ship: Can attack ground if cargo is ground unit
        if unit.cargo ~= nil then
          for cargo in unit.cargo.units do
            if cargo.prototype.lua.class == "ground" then
              return true
            end
            break
          end
        end
      else
        if unit.prototype.lua.stats["navalattack"] ~= nil then
			  if target.prototype.name == "submarine" then
				-- Ship that just revealed sub can atack it
				if unit.luaData.subRevealedUnit == target.id and unit.luaData.subRevealedTurn == game.turn then
				  return true
				end
				-- Can only attack sub with ship that has not yet moved
				if (attackerNotMoved and unit.faction.alliance:IsVisible(target)) or (HasUnitTech(unit, "passive_hydrophone" and unit.prototype.name == "lightcruiser")) then
				  return true
				end
			  else
				return true
			  end
        elseif unit.prototype.lua.stats["subattack"] ~= nil then
				if target.prototype.name == "submarine" then
				-- Can only attack sub with ship that has not yet moved
				if (attackerNotMoved and unit.faction.alliance:IsVisible(target)) or (HasUnitTech(unit, "passive_hydrophone" and unit.prototype.name == "lightcruiser"))  then
				  return true
				end
			  end
        end
      end
    end
  end
  return false
end

function GetValidStrategicTargets(unit, from)

  if from == nil then
    from = unit.hex
  end

  local targets = game.map:GetHexesInRange(from, unit.stats.range)
  local moves = unit:GetPotentialMoves(false) -- game.map:GetHexesInRange(from, unit.stats.mp)
  if # moves > 0 then
    local filteredMoves = {}
    for i, hex in pairs(moves) do
      if unit.faction.alliance:IsVisible(hex) then
        table.insert(filteredMoves, hex)
      end
    end
    moves = filteredMoves
  end
  local hexes = core.setSymmetricDifference(targets, moves, function(hex1, hex2) return hex1.id == hex2.id end)
  local validHexes = {}
  for i,hex in pairs(hexes) do
    -- If there is a construction
    if hexes[i].construction ~= nil and hexes[i].construction.type ~= Construction.TYPE_PORT and hexes[i].construction.efficency > 0 then
      -- and its owned by enemy
      if hexes[i].faction.alliance.id ~= unit.faction.alliance.id and hexes[i].faction.alliance.id ~= 0 then
        -- And there's no unit there (otherwise its already marked as target)
        if hexes[i].unit == nil or unit.faction.alliance:IsVisible(hexes[i]) == false then
          table.insert(validHexes, hexes[i])
        end
      end
    end
  end
  return validHexes

end
