-- Auto capture enemy regions that no longer have a city or unit inside
function FillGaps(alliance, faction)

  gameplay.filled = {}

  local hexes = {}

  for hex in alliance.border do
    if faction == nil or hex.faction.id == faction.id then
      table.insert(hexes, hex)
    end
  end

  for _, hex in pairs(hexes) do
    if gameplay.filled[hex.id] == nil then
      FillHex(hex)
    end
  end

end

function FillHex(startingHex)

  local closed = {}
  local count = {}

  local hexesToCheck = {}
  table.insert(hexesToCheck, startingHex)
  closed[startingHex.id] = true

  local fill = 0
  for _, hex in pairs(hexesToCheck) do
    if hex.construction ~= nil or hex.unit ~= nil then
      fill = 1
      break
    else
      local adjacentHexes = game.map:GetHexesInRange(hex, 1)
      for _, adjacent in pairs(adjacentHexes) do
        if closed[adjacent.id] == nil then
          if adjacent.faction ~= nil then
            if adjacent.faction.alliance.id == startingHex.faction.alliance.id  then
              table.insert(hexesToCheck, adjacent)
              closed[adjacent.id] = true
            else
              if count[adjacent.faction.id] == nil then
                count[adjacent.faction.id] = 1
              else
                count[adjacent.faction.id] = count[adjacent.faction.id] + 1
              end
              closed[adjacent.id] = true
            end
          end
        end
      end
    end
  end

  local maxCounter = 0
  local capFaction
  if fill == 0 then
    for factionId, counter in pairs(count) do
      local faction = game:GetFactionById(factionId)
      if counter > maxCounter and faction.alliance.id ~= 0 then
        counter = maxCounter
        capFaction = faction
      end
    end
  end

  for _, hex in pairs(hexesToCheck) do
    gameplay.filled[hex.id] = fill

    if fill == 0 and capFaction ~= nil then
      --DisplayProductionValue(hex, 1)
      --Log("hex " .. hex.x .. "," .. hex.y)
      if hex.originalFaction.alliance.id ~= capFaction.alliance.id then
        hex:SetFaction(capFaction)
      else
        hex:SetFaction(hex.originalFaction)
      end
    end
  end

end

-- Faction gives captured territory back after surrender
-- OR passes it to ally that has the most units present in the hex region
function TerritoryReclaim(faction)

  gameplay.filled = {}

  for hex in faction.hexes do
    if hex.originalFaction.id ~= faction.id and gameplay.filled[hex.id] == nil then
      ReclaimFill(hex, faction)
    end
  end

  FillGaps(faction.alliance, faction)
end

function ReclaimFill(startingHex, surrenderFaction)

  local closed = {}
  local count = {}

  local hexesToCheck = {}
  table.insert(hexesToCheck, startingHex)
  closed[startingHex.id] = true

  local fill = 0
  for _, hex in pairs(hexesToCheck) do
    if hex.unit ~= nil then
      fill = 1
      if count[hex.unit.faction.id] == nil then
        count[hex.unit.faction.id] = 1
      else
        count[hex.unit.faction.id] = count[hex.unit.faction.id] + 1
      end
    end

    local adjacentHexes = game.map:GetHexesInRange(hex, 1)
    for _, adjacent in pairs(adjacentHexes) do
      if closed[adjacent.id] == nil then
        if adjacent.faction ~= nil and adjacent.faction.id == surrenderFaction.id then
          if adjacent.faction.alliance.id == startingHex.faction.alliance.id  then
            table.insert(hexesToCheck, adjacent)
            closed[adjacent.id] = true
          end
        end
      end
    end
  end

  local maxCounter = 0
  local capFaction
  if fill == 1 then
    for factionId, counter in pairs(count) do
      local faction = game:GetFactionById(factionId)
      if counter > maxCounter and faction.alliance.id ~= 0 and faction.id ~= surrenderFaction.id then
        counter = maxCounter
        capFaction = faction
      end
    end
  end

  for _, hex in pairs(hexesToCheck) do
    gameplay.filled[hex.id] = fill

    if fill == 1 and capFaction ~= nil then
      hex:SetFaction(capFaction)
    else
      hex:SetFaction(hex.originalFaction)
    end
  end
end
