MAX_FLAGS_PER_PANEL = 26

class 'DiplomacyPanel' (LuaPanel)

function DiplomacyPanel:__init(alliance)
  LuaPanel.__init(self, diplomacy)


  self.tabImage = self:AddWidget({
    type = "image",
    position = {66, 315},
    image = "infotab1"
  })
  self.closeButton = self:AddWidget({
    name = "close_label",
    type = "button",
    image = "button_default",
    position = {195, 492},
    text = "close",
    sfx = "click",
    on_click = function(button)
      button.parent:Hide()
      ui.hotkeysActive = 1
    end
  })
  self:AddWidget({
    name = "stats_label",
    type = "label",
    position = {128, 330},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "statistics",
    sfx = "click",
    on_click = function(button)
      button.parent.statisticsPanel:Show()
      --button.parent.relationPanel:Show()
      button.parent:UpdateRelationStats(ui.selectedRelation)
      button.parent.informationPanel:Hide()
      button.parent.tabImage:SetImage("graphics/hud", "infotab1.png")
      --self.closeButton:Hide()
    end
  })
  self:AddWidget({
    name = "info_label",
    type = "label",
    position = {258, 330},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "information",
    sfx = "click",
    on_click = function(button)
      button.parent.statisticsPanel:Hide()
      --button.parent.relationPanel:Hide()
      button.parent.informationPanel:Show()
      button.parent.informationPanel.content:SetLocalText(ui.selectedRelation.name .. "_info")
      button.parent.tabImage:SetImage("graphics/hud", "infotab2.png")
      --self.closeButton:Show()
    end
  })

  self.informationPanel = self:AddWidget({
    type = "panel",
  })
  self.informationPanel.content = self.informationPanel:AddWidget({
    type = "memo",
    position = {78, 357},
    size = {220, 120},
    font = "times12_font",
    color   = {224, 220, 218},
    border = 0,
    OnScroll = function(this, delta)
      if delta == 1 then
        if self.informationPanel.content.startingLine > 0 then
          self.informationPanel.content.startingLine = self.informationPanel.content.startingLine - 1
          self.informationPanel.pagebar:SetPosition(235+66, 63+315 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
        end
      else
        if self.informationPanel.content.startingLine < self.informationPanel.content.numLines - 8 then
          self.informationPanel.content.startingLine = self.informationPanel.content.startingLine + 1
          self.informationPanel.pagebar:SetPosition(235+66, 63+315 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
        end
      end
    end
  })
  self.informationPanel:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {230+66, 43+315},
    sfx = "click",
    on_click = function(button)
      if self.informationPanel.content.startingLine > 0 then
        self.informationPanel.content.startingLine = self.informationPanel.content.startingLine - 1
        self.informationPanel.pagebar:SetPosition(235+66, 63+315 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
      end
    end
  })
  self.informationPanel:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {230+66, 148+315},
    sfx = "click",
    on_click = function(button)
      if self.informationPanel.content.startingLine < self.informationPanel.content.numLines - 8 then
        self.informationPanel.content.startingLine = self.informationPanel.content.startingLine + 1
        self.informationPanel.pagebar:SetPosition(235+66, 63+315 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
      end
    end
  })
  self.informationPanel.pagebar = self.informationPanel:AddWidget({
    type = "image",
    image = "pagebar_dark",
    position = {235+66, 63+315},
  })
  self.informationPanel:Hide()

  self.alliance = alliance

  -- Create sub panel types
  self.statisticsPanel = self:AdoptWidget(RelationStatsPanel())

  -- Create alliance flag buttons
  self.flags = {}
  local slot = 1
  for faction in playerAlliance.factions do
    -- only for factions with capitals
    if faction.luaData.capitals > 0 then
      self:FactionDiplomacyFlag(faction, slot)
      slot = slot + 1
    end
  end


  self:AddWidget({
    name = "relation_flag",
    type = "image",
    position = {76, 130},
  })
  self:AddWidget({
    name = "relation_name",
    type = "text",
    font = "times16_font",
    position = {195, 289},
    color   = {67, 56, 47},
    align = "center"
  })

  self.allianceName = {}
  self.allianceColour = {}
  for i = 0, 2 do
    self.allianceName[i] = self:AddWidget({
      type = "text",
      font = "hud14_font",
      position = {250, 30},
      align = "left",
      color   = {67, 56, 47},
      text = "alliance" .. i .. "_title"
    })
    self.allianceColour[i] = self:AddWidget({
      type = "image",
      position = {250, 30},
      image = "alliance_colour_" .. i
    })
  end

  self.relationsRow = {}
  for i = 0, 5 do
    self.relationsRow[i] = self:AddWidget({
      type = "image",
      position = {250, 30},
      image = "relations_row"
    })
  end

  j = 0
  self.relations = {}
  -- Create relation flags
  for faction in game.factions do
    if IsFactionVisible(faction) then

      --  Select first nation by default
      if j == 0 then
        ui.selectedRelation = faction
      end

      self.relations[faction.id] = self:AdoptWidget(DiplomacyRelationFlag(faction, j))

      j = j + 1

      -- Dont exceed max flags
      if j == MAX_FLAGS_PER_PANEL then
        break
      end

    end
  end

end

function DiplomacyPanel:OnShow()
  self:Refresh()
  self:UpdateRelationStats(ui.selectedRelation)
  self:RefreshRelations()
  ui.hotkeysActive = 0
end

function DiplomacyPanel:FactionDiplomacyFlag(faction, slot)

  --wow what a bad hack...
  if slot == nil then
    slot = 0
    for faction2 in playerAlliance.factions do
      if faction2.luaData.capitals > 0 then
        slot = slot + 1
      end
    end
  end


  if self.flags[faction.id] == nil then

    local factionName = faction.name

    self.flags[faction.id] = self:AddWidget({
      name = "flag" .. faction.id .. "_button",
      type = "button",
      image = "../flags/flag_" .. factionName .. "_32_button",
      position = {19, 16 + (slot - 1) * 30},
      sfx = "click",
      tooltip = factionName,
      on_click = function(button)
        ui.diplomacyFaction = faction
        SelectPanelFaction(faction)
        self:Refresh()
      end
    })

    -- Auto select first flag
    if slot == 1 then
      self:GetText("pp_value"):SetNumber(GetIncomePP(faction) - GetUpkeep(faction) - GetResearchUpkeep(faction))
      self:GetText("mp_value"):SetNumber(faction.mp)
      self:GetText("am_value"):SetNumber(GetIncomeAM(faction))
      self:GetText("faction_name"):SetLocalText(faction.name .. "_title")
      --self:GetText("alliance_name"):SetLocalText(faction.alliance.name)
      self:GetImage("faction_flag"):SetImage("graphics/flags","flag_" .. factionName .. "_128.png")

      ui.diplomacyFaction = faction
    end

  else
    self.flags[faction.id]:SetPosition(19, 16 + (slot - 1) * 30)
    self.flags[faction.id]:Show()
  end

end

function DiplomacyPanel:RemoveDiplomacyFlag(faction)
  -- If this faction is currently selected, select another
  if ui.diplomacyFaction.id == faction.id then
    for otherFaction in playerAlliance.factions do
      if CanFactionProduce(otherFaction) then
        ui.diplomacyFaction = otherFaction
        --SelectPanelFaction(otherFaction)
        break
      end
    end
  end

  -- Remove flag button
  self:GetButton("flag" .. faction.id .. "_button"):Hide()

  -- Shift other flags
  local slot = 1
  for otherFaction in playerAlliance.factions do
    if CanFactionProduce(otherFaction) then
      self.flags[otherFaction.id]:SetPosition(19, 16 + (slot - 1) * 30)
      slot = slot + 1
    end
  end
end

function DiplomacyPanel:RefreshRelations()

--~ print("refresh")
  local counter = {}
  counter[0] = 0
  counter[1] = 0
  counter[2] = 0

  local totalRows = {}
  totalRows[ui.playerAlliance.id] = math.floor((playerAlliance.numFactions - 1) / 6) + 1
--~   totalRows[ui.neutralAlliance.id] = math.floor((ui.neutralAlliance.numFactions - 1) / 6)
  totalRows[ui.enemyAlliance.id] = math.floor((ui.enemyAlliance.numFactions - 1) / 6) + 1
  totalRows[ui.neutralAlliance.id] = 6 - totalRows[playerAlliance.id] - totalRows[ui.enemyAlliance.id]

--~   if totalRows[playerAlliance.id] + totalRows[ui.neutralAlliance.id] + totalRows[enemyAlliance.id] < 6 then
--~     totalRows[ui.neutralAlliance.id] = totalRows[ui.neutralAlliance.id] + 1
--~   end
--~ print(playerAlliance.numFactions)
--~ print(ui.enemyAlliance.numFactions)
--~ print(totalRows[playerAlliance.id])
--~ print(totalRows[ui.enemyAlliance.id])
--~ print(totalRows[ui.neutralAlliance.id])
  for faction in game.factions do
    if IsFactionVisible(faction) then
      local flag = self.relations[faction.id]
      if faction.alliance.id == playerAlliance.id then
        flag:SetPosition(361 + (counter[playerAlliance.id] % 6) * 81, 58 + math.floor(counter[playerAlliance.id] / 6) * 66)
        counter[playerAlliance.id] = counter[playerAlliance.id] + 1
      elseif faction.alliance.id == 0 then
        flag:SetPosition(361 + (counter[0] % 6) * 81, 58 + 30 + totalRows[playerAlliance.id] * 66 + math.floor(counter[0] / 6) * 66)
        counter[0] = counter[0] + 1
      else
        flag:SetPosition(361 + (counter[ui.enemyAlliance.id] % 6) * 81, 58 + 30 + 30 + totalRows[ui.neutralAlliance.id] * 66 + totalRows[playerAlliance.id] * 66 + math.floor(counter[ui.enemyAlliance.id] / 6) * 66)
        counter[ui.enemyAlliance.id] = counter[ui.enemyAlliance.id] + 1
      end
    end
  end
  --for alliance in game.alliances do
    self.allianceName[playerAlliance.id]:SetPosition(387, 38)
    self.allianceColour[playerAlliance.id]:SetPosition(360, 38)
    self.allianceName[ui.neutralAlliance.id]:SetPosition(387, 38 + 30 + totalRows[playerAlliance.id] * 66)
    self.allianceColour[ui.neutralAlliance.id]:SetPosition(360, 38 + 30 + totalRows[playerAlliance.id] * 66)
    self.allianceName[ui.enemyAlliance.id]:SetPosition(387, 38 + 30 + 30 + totalRows[ui.neutralAlliance.id] * 66 + totalRows[playerAlliance.id] * 66)
    self.allianceColour[ui.enemyAlliance.id]:SetPosition(360, 38 + 30 + 30 + totalRows[ui.neutralAlliance.id] * 66 + totalRows[playerAlliance.id] * 66)
  --end

  local rowcount = 0
  for i = 0, totalRows[playerAlliance.id] - 1 do
    self.relationsRow[rowcount]:SetPosition(357, 54 + i * 66)
    rowcount = rowcount + 1
  end
  for i = rowcount, rowcount + totalRows[ui.neutralAlliance.id] - 1 do
    self.relationsRow[rowcount]:SetPosition(357, 84 + i * 66)
    rowcount = rowcount + 1
  end
  for i = rowcount, rowcount + totalRows[ui.enemyAlliance.id] - 1 do
    self.relationsRow[rowcount]:SetPosition(357, 114 + i * 66)
    rowcount = rowcount + 1
  end
end

function DiplomacyPanel:UpdateRelationStats(faction)

  self.statisticsPanel:Refresh(faction)

end



function DiplomacyPanel:Refresh()

  self:GetText("pp_value"):SetNumber(ui.diplomacyFaction.pp)
  self:GetText("mp_value"):SetNumber(ui.diplomacyFaction.mp)
  self:GetText("am_value"):SetNumber(ui.diplomacyFaction.am)


--~         self:GetText("pp_value"):SetNumber(GetIncomePP(faction) - GetUpkeep(faction))
--~         self:GetText("mp_value"):SetNumber(faction.mp)
--~         self:GetText("am_value"):SetNumber(GetIncomeAM(faction))
        self:GetText("faction_name"):SetLocalText(ui.diplomacyFaction.name .. "_title")
        --self:GetText("alliance_name"):SetLocalText(faction.alliance.name)
        self:GetImage("faction_flag"):SetImage("graphics/flags","flag_" .. ui.diplomacyFaction.name .. "_128.png")

        --self:UpdateRelationStats(ui.selectedRelation)

--~   local increment
--~   for alliance in game.alliances do
--~     for faction in alliance.factions do
--~       increment = CalcAlignmentIncrement(faction)
--~       if increment >= 0 then
--~         increment = "+" .. increment
--~       end

--~       local relationFlag = self:GetPanel("relationFlagButton" .. faction.id)

--~     end
--~   end

end


-----------------------------------------------------------------------


class 'DiplomacyRelationFlag' (LuaPanel)

function DiplomacyRelationFlag:__init(faction)
  LuaPanel.__init(self)

  self:AddWidget({
    name = "flag" .. faction.id .. "_button",
    type = "button",
    image = "../flags/flag_" .. faction.name .. "_64_button",
    position = {0, 0},
    sfx = "click",
    tooltip = faction.name,
    on_click = function(button)
      self.parent:UpdateRelationStats(faction)
    end
  })

end


-----------------------------------------------------------------------

class 'RelationStatsPanel' (LuaPanel)

function RelationStatsPanel:__init()
  LuaPanel.__init(self)

  self:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {80, 367},
    align = "left",
    color = {241, 237, 233},
    text = "relation_status"
  })
  self.relationStatus = self:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {80, 411},
    align = "left",
    color = {248, 215, 53},
  })

  self:AddWidget({
    type = "image",
    image = "progress_black",
    position = {90, 386},
  })

  self.alignmentBar = self:AddWidget({
    type = "image",
    image = "progress_green",
    position = {90, 386},
  })


  self.allyPanel = self:AdoptWidget(RelationAllyPanel())
  self.enemyPanel = self:AdoptWidget(RelationEnemyPanel())
  self.neutralPanel = self:AdoptWidget(RelationNeutralPanel())
  self.currentPanel = self.allyPanel

end

function RelationStatsPanel:OnShow()
end

function RelationStatsPanel:Refresh(faction)

  ui.selectedRelation = faction

  self.parent:GetText("relation_name"):SetLocalText(faction.name .. "_title")
  self.parent:GetImage("relation_flag"):SetImage("graphics/flags","flag_" .. faction.name .. "_32_dis.png")
  self.parent.informationPanel.content:SetLocalText(ui.selectedRelation.name .. "_info")
  self.parent.informationPanel.content.maxLines = 8
  self.parent.informationPanel.pageCount = math.max(8, self.parent.informationPanel.content.numLines)
  self.parent.informationPanel.pagebar:SetSource(0, 0, 6, 80 * 8 / self.parent.informationPanel.pageCount)

  local percent
  if playerAlliance.id == 1 then
    percent = faction.luaData.alignment
  else
    percent = 100 - faction.luaData.alignment
  end

  self.currentPanel:Hide()
  if faction.id ~= ui.diplomacyFaction.id then
    if percent >= 99 then
      self.currentPanel = self.allyPanel
    elseif percent == 0 then
      self.currentPanel = self.enemyPanel
    else
      self.currentPanel = self.neutralPanel
    end
    self.currentPanel:Show()
  end

  if ui.selectedRelation.alliance.id == playerAlliance.id then
    self.relationStatus:SetLocalText("status_ally")
    self.relationStatus:SetColour(192,204,111,255)
  end

  -- Alignment bar & status update
  if percent < 30 then
    self.alignmentBar:SetImage("graphics/hud", "progress_red.png")
    if percent == 0 then
      self.relationStatus:SetLocalText("status_enemy")
      self.relationStatus:SetColour(255,48,48,255)
    else
      self.relationStatus:SetLocalText("status_resentful")
      self.relationStatus:SetColour(255,145,48,255)
    end
  elseif percent > 70 then
    self.alignmentBar:SetImage("graphics/hud", "progress_green.png")
    if percent == 100 then
      self.relationStatus:SetLocalText("status_ally")
      self.relationStatus:SetColour(174,237,40,255)
    else
      self.relationStatus:SetLocalText("status_sympathetic")
      self.relationStatus:SetColour(233,255,48,255)
    end
  else
    self.alignmentBar:SetImage("graphics/hud", "progress_yellow.png")
    self.relationStatus:SetLocalText("status_neutral")
    self.relationStatus:SetColour(248, 215, 53,255)
  end
  self.alignmentBar:SetSource(0, 0, 1+202*percent/100, 8)

end


-----------------------------------------------------------------------

--~ function GetTotalDiplomaticBudget(alliance)
--~   local budget = 0
--~   local neutral = game:GetAllianceById(0)
--~   for faction in neutral.factions do
--~     --for relation = 0, 23 do
--~       budget = budget + (alliance.luaData.diploBudget[faction.id])
--~     --end
--~   end
--~   return budget
--~ end

diplomacy =
{
  name = "diplomacy_panel",
  type = "panel",
  position = {(engine.window.width)/2-437, (engine.window.height-200)/2-274},
  image = "diplomacy_panel",
  visible = false,

  tooltip =
  {
    rect  = {10, 10, 0, 0},
    color = (255 255 255),
    bgcolor = (10 10 10),
    font  = "hud_font"
  },

  controls =
  {
    {
      name = "pp_title",
      type = "image",
      position = {204, 39},
      image = "resource_pp_white",
      tooltip = "resourcePP"
    },
    {
      name = "pp_value",
      type = "text",
      font = "hud14_font",
      position = {258, 48},
      align = "center",
      color   = {241, 237, 233},
      tooltip = "currentPP"
    },
    {
      name = "mp_title",
      type = "image",
      position = {204, 64},
      image = "resource_mp_white",
      tooltip = "resourceMP"
    },
    {
      name = "mp_value",
      type = "text",
      font = "hud14_font",
      position = {258, 73},
      align = "center",
      color   = {241, 237, 233},
      tooltip = "currentMP"
    },
    {
      name = "am_title",
      type = "image",
      position = {204, 89},
      image = "resource_am_white",
      tooltip = "resourceAM"
    },
    {
      name = "am_value",
      type = "text",
      font = "hud14_font",
      position = {258, 98},
      align = "center",
      color   = {241, 237, 233},
      tooltip = "currentAM"
    },
    {
      name = "pp_income",
      type = "text",
      font = "hud12_font",
      position = {298, 48},
      align = "center",
      color   = {241, 237, 233},
      tooltip = "incrementPP"
    },
--
	{
       name = "mp_income",
       type = "text",
       font = "hud12_font",
       position = {298, 73},
       align = "center",
       color   = {241, 237, 233},
       tooltip = "incrementMP"
     },
    {
      name = "am_income",
      type = "text",
      font = "hud12_font",
      position = {298, 98},
      align = "center",
      color   = {241, 237, 233},
      tooltip = "incrementAM"
    },
    {
      name = "faction_name",
      type = "text",
      font = "hud14_font",
      position = {258, 24},
      align = "center",
      color   = {241, 237, 233},
    },
    {
      name = "faction_flag",
      type = "image",
      position = {65, 16},
    },
  }
}


-----------------------------------------------------------------------

class 'RelationNeutralPanel' (LuaPanel)

function RelationNeutralPanel:__init()
  LuaPanel.__init(self)

  self:AddWidget({
    name = "dow_button",
    type = "button",
    image = "button_default",
    position = {195, 402},
    text = "declare_war",
    sfx = "click",
    on_click = function(button)
      function DeclareWarConfirmed()
        DeclareWar(playerAlliance, ui.selectedRelation)
        self.parent.parent:UpdateRelationStats(ui.selectedRelation)
        self.parent.parent:RefreshRelations()
        ui.mainPanel.diplomacyTab:Refresh()
      end
      PopupDialog("declare_war_confirm", DeclareWarConfirmed)
    end
  })

--~   self:AddWidget({
--~     name = "force_button",
--~     type = "button",
--~     image = "button_default",
--~     position = {195, 432},
--~     text = "force_join",
--~     sfx = "click",
--~     on_click = function(button)
--~       function ForceJoinConfirmed()
--~         ForceJoin(playerAlliance, ui.selectedRelation)
--~         self.parent.parent:UpdateRelationStats(ui.selectedRelation)
--~       end
--~       PopupDialog("force_join_confirm", ForceJoinConfirmed)
--~     end
--~   })

  self:Hide()

end

function RelationNeutralPanel:OnHide()
  --self.parent.relationPanel:Hide()
end


-----------------------------------------------------------------------


class 'RelationAllyPanel' (LuaPanel)

function RelationAllyPanel:__init()
  LuaPanel.__init(self)


--~   self:AddWidget({
--~     type = "text",
--~     font = "hud12_font",
--~     position = {80, 447},
--~     align = "left",
--~     color = {241, 237, 233},
--~     text = "Resources Transfer"
--~   })
--~   self:AddWidget({
--~     type = "image",
--~     position = {80, 467},
--~     image = "resource_pp_white",
--~     tooltip = "costPP"
--~   })
--~   self.transferPP = self:AddWidget({
--~     type = "edit",
--~     text = "0",
--~     size = {width=40, height=20},
--~     position = {110, 465},
--~   })
--~   self:AddWidget({
--~     type = "image",
--~     position = {200, 467},
--~     image = "resource_am_white",
--~   })
--~   self.transferAM = self:AddWidget({
--~     type = "edit",
--~     text = "0",
--~     size = {width=40, height=20},
--~     position = {230, 465},
--~   })

--~   self:AddWidget({
--~     name = "donate_button",
--~     type = "button",
--~     image = "button_default",
--~     position = {77, 492},
--~     text = "donate",
--~     sfx = "click",
--~     on_click = function(button)
--~       TransferResource(ui.diplomacyFaction, ui.selectedRelation, tonumber(self.transferPP.text), "PP")
--~       TransferResource(ui.diplomacyFaction, ui.selectedRelation, tonumber(self.transferAM.text), "AM")
--~       ui.resourcesPanel:Refresh()
--~     end
--~   })

--~   self:AddWidget({
--~     name = "kick_button",
--~     type = "button",
--~     image = "button_default",
--~     position = {195, 402},
--~     text = "Bugger Off",
--~     sfx = "click",
--~     on_click = function(button)
--~       DiscardFaction(ui.selectedRelation)
--~     end
--~   })

--~   self:AddWidget({
--~     name = "kick_button",
--~     type = "button",
--~     image = "button_default",
--~     position = {195, 432},
--~     text = "Surrender",
--~     sfx = "click",
--~     on_click = function(button)
--~       ui.selectedRelation.morale = 0
--~     end
--~   })

  self:Hide()

end

function RelationAllyPanel:OnHide()
  --self.parent.relationPanel:Hide()
end


-----------------------------------------------------------------------


class 'RelationEnemyPanel' (LuaPanel)

function RelationEnemyPanel:__init()
  LuaPanel.__init(self)

--~   self.acceptSurrenderButton = self:AddWidget({
--~     name = "accept_button",
--~     type = "button",
--~     image = "button_default",
--~     position = {77, 492},
--~     text = "accept",
--~     sfx = "click",
--~     on_click = function(button)
--~     end
--~   })

--~   self:AddWidget({
--~     name = "kick_button",
--~     type = "button",
--~     image = "button_default",
--~     position = {195, 402},
--~     text = "Bugger Off",
--~     sfx = "click",
--~     on_click = function(button)
--~       DiscardFaction(ui.selectedRelation)
--~     end
--~   })

--~   self:AddWidget({
--~     name = "kick_button",
--~     type = "button",
--~     image = "button_default",
--~     position = {195, 432},
--~     text = "Surrender",
--~     sfx = "click",
--~     on_click = function(button)
--~       ui.selectedRelation.morale = 0
--~     end
--~   })

  self:Hide()

end

function RelationEnemyPanel:OnShow()
--~   if ui.selectedRelation.morale <= 0 then
--~     self.acceptSurrenderButton:Show()
--~   else
--~     self.acceptSurrenderButton:Hide()
--~   end
end

function RelationEnemyPanel:OnHide()
  --self.parent.relationPanel:Hide()
end
