class 'ResourcesPanel' (LuaPanel)

function ResourcesPanel:__init(alliance)
  LuaPanel.__init(self, resources_panel)

  self.alliance = alliance

  self.slots = {}
  local slot = 1
  for faction in self.alliance.factions do
  --if faction.alliance.id == sai.alliance.id or faction.alliance.id == sai.enemyAlliance.id then
    self:AddFactionResourceInfo(faction, slot)
    slot = slot + 1
  --end
  end

end

function ResourcesPanel:AddFactionResourceInfo(faction, slot)

  if self.slots[faction.id] == nil then
    self.slots[faction.id] = self:AdoptWidget(ResourceInfoPanel(faction, slot))
  else
    self.slots[faction.id].slot = slot
    self.slots[faction.id]:SetPosition(0, 16 + (slot -1) * 14)
    self.slots[faction.id]:Refresh()
  end

  self:Refresh()
end


function ResourcesPanel:RemoveFaction(faction)

  local slot = 0
  for factionId, slot in pairs(self.slots) do
    if factionId ~= faction.id and slot.slot > self.slots[faction.id].slot then
      slot.slot = slot.slot - 1
      self.slots[factionId]:SetPosition(0, 16 + (slot.slot -1) * 14)
    end
  end

  self.slots[faction.id].slot = -1

  self:Refresh()
end


function ResourcesPanel:Refresh()

  for _, slot in pairs(self.slots) do
    slot:Refresh()
  end

end

resources_panel =
{
  name = "resources_panel",
  type = "panel",
  position = {12, 30},

  controls =
  {
    {
      name = "resourcePP_image",
      type = "image",
      image = "resource_pp",
      position = {24, 2}, -- +1
      tooltip = "resourcePP",
    },
    {
      name = "resourceMP_image",
      type = "image",
      image = "resource_mp",
      position = {59, 2}, -- +2
      tooltip = "resourceMP",
    },
    {
      name = "resourceAM_image",
      type = "image",
      image = "resource_am",
      position = {93, 2}, -- +2
      tooltip = "resourceAM",
    },
    {
      name = "resourceRC_image",
      type = "image",
      image = "resource_rail",
      position = {125, 2}, -- +2
      tooltip = "resourceRC",
    },
    {
      name = "resourceTC_image", -- transportmove
      type = "image",
      image = "resource_tc",
      position = {159, 2}, -- + 4
      tooltip = "resourceTC",
    },
    {
      name = "resourceNM_image",
      type = "image",
      image = "resource_nm",
      position = {194, 2}, -- +5
      tooltip = "resourceNM",
    },
  },
}


----------------------------------------------------------------


class 'ResourceInfoPanel' (LuaPanel)

function ResourceInfoPanel:__init(faction, slot)
  LuaPanel.__init(self)

  self.slot = slot
  self.faction = faction

  self:SetPosition(0, 16 + (slot - 1) * 14)

  self:AddWidget({
    type = "image",
    image = "../flags/flag_" .. self.faction.name .. "_16",
    position = {1, 0},
    tooltip = self.faction.name
  })
  self.pp = self:AddWidget({
    type = "text",
    font = "verdana12_font",
    position = {36, 5},
    color   = {42, 34, 22},
    shadow = {offset = {0, 0}},
    align = "center",
    text = "-",
    tooltip = "resourcePP",
  })
  self.mp = self:AddWidget({
    type = "text",
    font = "verdana12_font",
    position = {70, 5}, -- +2
    color   = {42, 34, 22},
    shadow = {offset = {0, 0}},
    align = "center",
    text = "-",
    tooltip = "resourceMP",
  })
  self.am = self:AddWidget({
    type = "text",
    font = "verdana12_font",
    position = {104, 5}, -- +2
    color   = {42, 34, 22},
    shadow = {offset = {0, 0}},
    align = "center",
    text = "-",
    tooltip = "resourceAM",
  })
  self.rm = self:AddWidget({
    type = "text",
    font = "verdana12_font",
    position = {137, 5},  -- +2
    color   = {42, 34, 22},
    shadow = {offset = {0, 0}},
    align = "center",
    text = "-",
    tooltip = "resourceRC",
  })
  self.tm = self:AddWidget({ -- Transportmoves
    type = "text",
    font = "verdana12_font",
    position = {171, 5}, -- + 4
    color   = {42, 34, 22},
    shadow = {offset = {0, 0}},
    align = "center",
    text = "-",
    tooltip = "resourceTC",
  })
  self.nm = self:AddWidget({
    type = "text",
    font = "verdana12_font",
    position = {207, 5}, -- +5
    color   = {42, 34, 22},
    shadow = {offset = {0, 0}},
    align = "center",
    text = "-",
    tooltip = "resourceNM",
  })

end

function ResourceInfoPanel:Refresh()
  if self.slot > 10 or self.slot == -1 then
    self:Hide()
  else
    self:Show()

    self.pp:SetNumber(self.faction.pp)
    self.mp:SetNumber(self.faction.mp)
    self.am:SetNumber(self.faction.am)

    -- Rail moves
    local rm = GetFactionRailmoves(self.faction)
    self.rm:SetNumber(rm)
    if rm == 0 then
      self.rm:SetColour(137,31,3,255)
    else
      self.rm:SetColour(42, 34, 22, 255)
    end

    -- Transport moves
    local tm = GetFactionTransportmoves(self.faction)
    self.tm:SetNumber(tm)
    if tm == 0 then
      self.tm:SetColour(137,31,3,255)
    else
      self.tm:SetColour(42, 34, 22, 255)
    end

    -- MP colour
    if self.faction.luaData.qualityMP == 2 then
      self.mp:SetColour(255,228,0,255)
    elseif self.faction.luaData.qualityMP == 1 then
      self.mp:SetColour(201,85,0,255)
    elseif self.faction.luaData.qualityMP == 0 then
      self.mp:SetColour(137,31,3,255)
    end


    -- National morale colour
    if self.faction.luaData.moraleLevel < 40 then
      self.nm:SetColour(137,31,3,255)
    elseif self.faction.luaData.moraleLevel > 60 then
      self.nm:SetColour(70,104,10,255)
    else
      self.nm:SetColour(64,54,12,255)
    end
    --self.nm:SetText(self.faction.luaData.moraleLevel .. "%")
    self.nm:SetText(GetMoralePercent(self.faction) .. "%")
    --self.nm:SetText("" .. self.faction.morale)
  end
end
