class 'ChatPanel' (LuaPanel)

function ChatPanel:__init(alliance)
  LuaPanel.__init(self)

  self:SetPosition((engine.window.width)/2-437, (engine.window.height-200)/2-274)
  self:SetBackgroundImage("chat_panel")

  self.endTurn = 0

  self.scrollPos = 0

  self.playerName = self:AddWidget({
    type = "text",
    position = {98, 79},
    align = "center",
    font = "hud16_font",
    color = {247, 245, 241},
    text = pbem.playerName
  })

  self.playerAvatar = self:AddWidget({
    type = "image",
    position = {32, 100},
    image = "../commanders/commander_" .. settings.avatar,
  })

  self:AddWidget({
    type = "image",
    position = {40, 210},
    image = "alliance_colour_" .. playerAlliance.id,
    tooltip = "player_" .. playerAlliance.id
  })

  self.opponentName = self:AddWidget({
    type = "text",
    position = {777, 79},
    align = "center",
    font = "hud16_font",
    color = {247, 245, 241},
    text = settings.opponentName
  })

  self.opponentAvatar = self:AddWidget({
    type = "image",
    position = {710, 100},
    image = "../commanders/commander_none",
  })

  self:AddWidget({
    type = "image",
    position = {718, 210},
    image = "alliance_colour_" .. (3 - playerAlliance.id),
    tooltip = "player_" .. (3 - playerAlliance.id),
  })

  self:AddWidget({
    type = "text",
    position = {71, 28},
    align = "center",
    font = "hud14_font",
    color = {247, 245, 241},
    text = "chat_title"
  })

  self:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {658, 108},
    sfx = "click",
    Clicked = function()
      if self.scrollPos > 0 then
        self.scrollPos = self.scrollPos - 1
        self:Refresh(self.scrollPos)
      end
    end
  })
  self:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {658, 415},
    sfx = "click",
    Clicked = function()
      if not self.lastPrinted then
        self.scrollPos = self.scrollPos + 1
        self:Refresh(self.scrollPos)
      end
    end
  })

  self.edit = self:AddWidget({
    type = "edit",
    name = "nameEdit",
    position = {194, 458},
    size = {width=483, height=57},
    EnterPressed = function(edit)
      if edit.text ~= "" then
        game.chat:AddMessage(playerAlliance.id, edit.text)

        self.listLength = self.listLength + 1
        self.lengths[self.listLength] = GetTextHeight("times12_font", edit.text, self.memoWidth)

        --self:AddChatMessage(playerAlliance.id, edit.text)
        self:JumpToLast()

        edit:Clear()
      end
    end
  })

  self.turnButton = self:AddWidget({
    name = "endTurnButton",
    type = "button",
    image = "button_default",
    position = {725, 490},
    text = "end_turn",
    sfx = "end_turn",
    tooltip = "end_turn_button",
    on_click = function(button)
      if not ui.MPturnEnded then
        if self.edit.text ~= "" then
          local function ConfirmEndTurn()
            PreEndTurn()
            button.parent:Hide()
          end
          PopupDialog("chat_confirm", ConfirmEndTurn)
        else
          PreEndTurn()
          button.parent:Hide()
        end
      end
    end
  })

  self.replayButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {725, 490},
    text = "start_replay",
    sfx = "click",
    tooltip = "replay_start_text",
    on_click = function(button)
	
      game:StartReplay()
      ui.replayStartPanel:Hide()
      button.parent.turnButton:Show()
      button.parent.surrenderButton:Show()
      button.parent:Hide()
      button:Hide()
    end
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {35, 490},
    font = "hud14_font",
    text = "close",
    sfx = "click",
    on_click = function(button)
      button.parent:Hide()
      ui.hotkeysActive = 1
    end
  })
  self.surrenderButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {725, 450},
    font = "hud14_font",
    text = "surrender",
    sfx = "click",
    on_click = function(button)
      local function ConfirmSurrender()
        -- Surrender game
        GetPlayer():Surrender()

        -- Block UI
        ui.MPturnEnded = true
        ui.mainPanel.blockingPanel:Popup("pbem")

        -- Upload turn
        GetPlayer():UploadTurn(function(error) UploadFailed(error) end)

        -- Hide chat panel
        button.parent:Hide()
      end
      PopupDialog("surrender_confirm", ConfirmSurrender)
    end
  })

  self.content = self:AddWidget({
    type = "panel",
    position = {197, 111},
  })

  -- count chat messages
  self.memoWidth = 410
  local whiteSpace = 15

  local maxHeight = 325
  self.lengths = {}

  self.listLength = 0
  for message in game.chat.messages do
    self.listLength = self.listLength + 1
    self.lengths[self.listLength] = GetTextHeight("times12_font", message.message, self.memoWidth)
  end
  if self.listLength > 0 then
    self:JumpToLast()
  end

--~   if game.turn == 1 and startingAlliance == playerAlliance.id then
--~     self.replayButton:Hide()
--~   else
--~     self.turnButton:Hide()
--~     self.surrenderButton:Hide()
--~   end
  self.replayButton:Hide()

  self:Hide()

end

function ChatPanel:OnShow()
  ui.hotkeysActive = 0
end

function ChatPanel:Refresh(scrollPos)

  self.content:RemoveWidgets()

  self.scrollPos = scrollPos

  self.content.lines = 0

  local counter = 0
  self.lastPrinted = true
  for message in game.chat.messages do
    if counter >= self.scrollPos then
      if not self:AddChatMessage(message.sender, message.message) then
        -- break when max height is exceeded
        self.lastPrinted = false
        print("last")
        break
      end
    end
    counter = counter + 1
  end

  local opponentAlliance = game:GetAllianceById(3 - playerAlliance.id)
  if opponentAlliance.luaData.avatar ~= nil then
    self.opponentAvatar:SetImage("graphics/commanders", "commander_" .. opponentAlliance.luaData.avatar .. ".png")
  end

end

function ChatPanel:AddChatMessage(allianceId, message)

  --self.listLength = self.listLength + 1

  --memoWidth = 410
  local whiteSpace = 15

  local maxHeight = 325

  local textColor
  if allianceId == 1 then
    textColor = {70, 104, 10}
  else
    textColor = {137, 31, 3}
  end

  local margin = 0
  if allianceId == playerAlliance.id then
    margin = 0
  else
    margin = 35
  end

  local memoHeight = GetTextHeight("times12_font", message, self.memoWidth)

  --self.lengths[self.listLength] = memoHeight

  -- Dont add message if it's too long to fit
  if self.content.lines + memoHeight > maxHeight then
    self.lastPrinted = false
    return false
  end

  self.content:AddWidget({
    type = "memo",
    position = {margin, self.content.lines},
    size = {self.memoWidth, memoHeight},
    font = "times12_font",
    color   = textColor,
    text = message,
    border = 0
  })

  self.content.lines = self.content.lines + whiteSpace + memoHeight

  return true
end

function ChatPanel:JumpToLast()
  local totalLength = 0
  for j = 1, self.listLength do
    local i = self.listLength - j + 1
    totalLength = totalLength + self.lengths[i] + 15
    if totalLength - 15 > 325 then
      break
    else
      self.scrollPos = i - 1
      print(self.scrollPos)
      print(totalLength)
    end
  end
  self:Refresh(self.scrollPos)
end
