class 'CursorModePanel' (LuaPanel)

function CursorModePanel:__init()
  LuaPanel.__init(self)

  self:SetBackgroundImage("toptab")

  self:SetPosition(engine.window.width/2-323, 0)

  self.title = self:AddWidget({
    type = "text",
    font = "hud14_font",
    position = {323, 8},
    align = "center",
    color   = {10, 5, 0},
    text = "mode_repair",
  })

  self:Hide()

end

function CursorModePanel:OnShow()
  GetPlayer():DeselectUnit()

  if ui.UIMode == REPAIR then
    SetCursorImage("cursor_repair")
    self.title:SetLocalText("mode_repair")
  end
  if ui.UIMode == UPGRADE then
    SetCursorImage("cursor_upgrade")
    self.title:SetLocalText("mode_upgrade")
  end
  if ui.UIMode == DISBAND then
    SetCursorImage("cursor_disband")
    self.title:SetLocalText("mode_disband")
  end
  if ui.UIMode == DEPLOY then
    SetCursorImage("cursor_deploy")
    self.title:SetLocalText("mode_deploy")
  end
  if ui.UIMode == ATTACH then
    SetCursorImage("cursor_deploy")
    self.title:SetLocalText("mode_attach")
  end
end

function CursorModePanel:OnHide()

  --SetCursorImage("cursor_default")
end


---------------------------------------------------------------

class 'CursorModeTooltip' (LuaPanel)

function CursorModeTooltip:__init()
  LuaPanel.__init(self)

  self:SetBackgroundImage("tooltip")
  self:SetMouseInputEnabled(false)

  self.hitpoints_text = self:AddWidget(
  {
    type = "text",
    font = "hud_font",
    position = {47, 12},
    align = "center",
    color   = {0, 0, 0},
    text = "10"
  })
  self.hitpoints_text:SetMouseInputEnabled(false)

  self.resource_1_text = self:AddWidget(
  {
    type = "text",
    font = "hud_font",
    position = {47, 32},
    align = "center",
    color   = {0, 0, 0},
    text = "10"
  })
  self.resource_1_text:SetMouseInputEnabled(false)

  self.resource_2_text = self:AddWidget(
  {
    type = "text",
    font = "hud_font",
    position = {47, 52},
    align = "center",
    color   = {0, 0, 0},
    text = "10"
  })
  self.resource_2_text:SetMouseInputEnabled(false)

  self:Hide()

end


function CursorModeTooltip:Display(hex)

  local x, y = HexWindowPosition(hex)
  local height = 62

  if ui.UIMode == REPAIR then
    SetCursorAnimation("cursor_repair")
    if self.lastUIMode ~= ui.UIMode then
      self.lastUIMode = ui.UIMode
      self:SetBackgroundImage("tooltip")
      self.resource_1_text:Show()
      self.resource_2_text:Show()
    end
  end
  if ui.UIMode == UPGRADE then
    SetCursorAnimation("cursor_upgrade")
    if self.lastUIMode ~= ui.UIMode then
      self.lastUIMode = ui.UIMode
      self:SetBackgroundImage("tooltip")
      self.resource_1_text:Show()
      self.resource_2_text:Show()
    end
  end
  if ui.UIMode == DISBAND then
    SetCursorAnimation("cursor_disband")
    if self.lastUIMode ~= ui.UIMode then
      self.lastUIMode = ui.UIMode
      self:SetBackgroundImage("tooltip")
      self.resource_1_text:Show()
      self.resource_2_text:Show()
    end
  end
  if ui.UIMode == DEFAULT then
    height = 24
    --SetCursorAnimation("cursor_disband")
    if self.lastUIMode ~= ui.UIMode then
      self.lastUIMode = ui.UIMode
      self:SetBackgroundImage("tooltip_ammo")
      self.resource_1_text:Hide()
      self.resource_2_text:Hide()
    end
  end

  self:SetPosition(x+6, y-height+7)

  self:Show()

end


function CursorModeTooltip:OnHide()

  if ui.UIMode == REPAIR then
    SetCursorImage("cursor_repair")
  end
  if ui.UIMode == UPGRADE then
    SetCursorImage("cursor_upgrade")
  end
  if ui.UIMode == DISBAND then
    SetCursorImage("cursor_disband")
  end

end

function DisplayRepairZone()
  ui.SelectNothing()
  local hexArray = {}
  local hexArrayYellow = {}
  for faction in playerAlliance.factions do
    for unit in faction.units do
      if CanRepairUnit(unit) == true then
        local costPP, costMP = GetRepairCost(unit)
        if costPP <= faction.pp and costMP <= faction.mp then
          table.insert(hexArray, unit.hex)
        else
          table.insert(hexArrayYellow, unit.hex)
        end
      end
    end
  end
  GetScene():ClearActions()
  GetScene():AddActionImages(hexArray, "unit_marker_white")
  GetScene():AddActionImages(hexArrayYellow, "unit_marker_yellow")
end

function DisplayAttachZone(commander)
  local hexArray = {}
  for faction in playerAlliance.factions do
    for unit in faction.units do
      if CommanderCompatible(commander, unit) then
        table.insert(hexArray, unit.hex)
      end
    end
  end
  GetScene():ClearActions()
  GetScene():AddActionImages(hexArray, "unit_marker_white")
end

function DisplayUpgradeZone()
  local hexArray = {}
  local hexArrayYellow = {}
  for faction in playerAlliance.factions do
    for unit in faction.units do
      local check = UnitCanUpgrade(unit)
      if check > 0 then
        if check <= faction.pp then
          table.insert(hexArray, unit.hex)
        else
          table.insert(hexArrayYellow, unit.hex)
        end
      end
    end
  end
  GetScene():ClearActions()
  GetScene():AddActionImages(hexArray, "unit_marker_white")
  GetScene():AddActionImages(hexArrayYellow, "unit_marker_yellow")
end

function DisplayDisbandZone()
  local hexArray = {}
  for faction in playerAlliance.factions do
    for unit in faction.units do
      if CanUnitDisband(unit) then
        table.insert(hexArray, unit.hex)
      end
    end
  end
  GetScene():ClearActions()
  GetScene():AddActionImages(hexArray, "unit_marker_white")
end


---------------------------------------------------------------


function HexWindowPosition(hex)
  local mapPosX, mapPosY = PixelPosition(hex)
  return mapPosX - GetPlayer().camera.x, mapPosY - GetPlayer().camera.y
end

function SetCursorImage(file)
  GetPlayer().scene.cursor:SetImage(file)
end

function SetCursorAnimation(file)
  GetPlayer().scene.cursor:SetAnimation(file)
end

function ResetCursor()
  GetPlayer().scene.cursor:Reset()
end

function ResetCursorMode()
  ui.cursormode:Hide()
  if ui.cursorModeTooltipPanel ~= nil then
    ui.cursorModeTooltipPanel:Hide()
  end
  GetScene():ClearActions()
  ResetCursor()

  if ui.UIMode == DEPLOY or ui.UIMode == ATTACH then
    if ui.selectedQueueSlot ~= nil then
      ui.selectedQueueSlot:SetBackgroundImage("queue")
    end
  end

  ui.UIMode = DEFAULT
end
