class 'DiplomacyTab' (LuaPanel)

function DiplomacyTab:__init(alliance)
  LuaPanel.__init(self)

  self:SetBackgroundImage("tabs_diplomacy")

  self.alliance = alliance
  -- Diplomacy tab

  -- Create queue elements
  self.queueElement = {}
  for slot = 0, 11 do
    self.queueElement[slot] = self:AdoptWidget(DiplomacyQueueElement(slot))
  end

end

function DiplomacyTab:OnShow()
  self:Refresh()
  hud:GetPanel("diplomacy_panel"):Show()
end

function DiplomacyTab:OnHide()
  hud:GetPanel("diplomacy_panel"):Hide()
end

function DiplomacyTab:Refresh()
  local slot = 0
  local factionsTable = {}
  for faction in ui.neutralAlliance.factions do
    if self:GetStraightAlignment(faction) < 6.8 then
      --factionsTable[#factionsTable] = faction
      --  print(#factionsTable)
      table.insert(factionsTable, faction)
    end
  end

  factionsTable = self:SortFactionsByAlignment(factionsTable)

  for i, faction in pairs(factionsTable) do
    self.queueElement[slot].flag:SetImage("graphics/flags", "flag_" .. faction.name .. "_64.png")
    local toward = 0
    if faction.luaData.alignment < 50 then
      toward = 2
    end
    if faction.luaData.alignment > 50 then
      toward = 1
    end
    self.queueElement[slot].alliance:SetImage("graphics/hud", "alliance_colour_" .. toward .. ".png")
    if faction.luaData.alignment_display > 0 then
      local turns = GetFactionJoinTurns(faction)
      if turns <= 10 then
        self.queueElement[slot].turns:SetText(turns .. "-" .. turns + faction.luaData.alignment_display)
      else
        self.queueElement[slot].turns:SetText((math.floor(turns/5) * 5)  .. "+")
      end
    else
      self.queueElement[slot].turns:SetNumber(GetFactionJoinTurns(faction))
    end
    if CheckFactionDOWBalance(faction) then
      self.queueElement[slot].turns:SetColour(67, 56, 47,255)
      self.queueElement[slot].turnIcon:SetImage("graphics/hud", "turn_icon.png")
    else
      self.queueElement[slot].turns:SetColour(255,48,48,255)
      self.queueElement[slot].turnIcon:SetImage("graphics/hud", "turn_icon_red.png")
    end
    self.queueElement[slot].flag.tooltip = faction.name
    self.queueElement[slot].alliance.tooltip = "join_alliance_" .. toward
    self.queueElement[slot].faction = faction
    self.queueElement[slot]:Show()
    slot = slot + 1
  end

  -- Hide remaining slots
  for i = slot, 11 do
    self.queueElement[i]:Hide()
  end
end

function DiplomacyTab:SortFactionsByAlignment(workingTable)
  finalTable = {}
  repeat
    local smallestValue = 100
    local smallestFaction
    local smallestIndex
    for i, faction in pairs(workingTable) do
      if self:GetStraightAlignment(faction) < smallestValue then
        smallestValue = self:GetStraightAlignment(faction)
        smallestFaction = faction
        smallestIndex = i
      end
    end
    table.insert(finalTable, smallestFaction)
    table.remove(workingTable, smallestIndex)
    --print(#workingTable)
  until # workingTable == 0
  return finalTable
end

function DiplomacyTab:GetStraightAlignment(faction)
  if faction.luaData.alignment > 50 then
    return 100 - faction.luaData.alignment
  elseif faction.luaData.alignment < 50 then
    return faction.luaData.alignment
  else
    return 50
  end
end


----------------------------------------------------------------

class 'DiplomacyQueueElement' (LuaPanel)

function DiplomacyQueueElement:__init(slot)
  LuaPanel.__init(self)

  self:SetPosition(10 + (slot % 6) * 79, 32 + math.floor(slot / 6) * 77)

  self:SetBackgroundImage("queue_diplomacy")

  --self.faction =

  self.flag = self:AddWidget({
    type = "image",
    position = {7, 7},
    on_click = function(button)
      ui.diplomacyPanel:UpdateRelationStats(button.parent.faction)
    end
  })
  self.alliance = self:AddWidget({
    type = "image",
    position = {7, 55},
  })
  self.turns = self:AddWidget({
    type = "text",
    font = "hud12_font",
    align = "right",
    color = {67, 56, 47},
    position = {56, 57},
    tooltip = "turns_to_join",
  })

  self.turnIcon = self:AddWidget({
    type = "image",
    position = {60, 56},
    image = "turn_icon",
    tooltip = "turns_to_join",
  })
end
