dofile("ui/frontend/pbem_panel.lua")


class 'MultiplayerMenuScreen' (GameScreen)

function MultiplayerMenuScreen:__init()
  GameScreen.__init(self)
end

function screenAlign(widget, width, height)
  widget:SetPosition(width * engine.window.width - widget.width / 2, height * engine.window.height - widget.height / 2)
end

function MultiplayerMenuScreen:OnInit()
  self.hud:Load(hud.menu_main)

  -- Draw background & hi-res bar edges
  local bg = self.hud:GetPanel("main_panel")
  local temp = bg:GetImage("background")
  local date = GetDate()
  local date11 = false
  if (date.day == 11 and date.month == 11) then
    date11 = true
  end
  local date28 = false
  if (date.day == 28 and date.month == 7) then
    date28 = true
  end
  
  if engine.window.width / engine.window.height > 1.5 then
    if date28 then
	   temp:SetImage("graphics/hud","menu_background_wide_22.png")	
	 
	elseif date11 then	
		 temp:SetImage("graphics/hud","menu_background_wide_11.png")
	
	else
		temp:SetImage("graphics/hud","menu_background_wide.png")
	end
	
  else
	if date28 then
		temp:SetImage("graphics/hud","menu_background_22.png")
	
    elseif date11 then
		temp:SetImage("graphics/hud","menu_background_11.png")
    
	else
		temp:SetImage("graphics/hud","menu_background.png")
	end
  end
  temp:SetScaling(engine.window.width / bg.width, engine.window.height / bg.height)
  temp = bg:GetImage("menu_hiresbarleft_image")
  temp:SetPosition(0, engine.window.height-18)
  temp = bg:GetImage("menu_hiresbarright_image")
  temp:SetPosition(engine.window.width-1024, engine.window.height-18)

  -- reposition menu bar
  local temp = self.hud:CreateWidget(hud.menu_bar)
  temp:SetPosition((engine.window.width-1024) / 2, engine.window.height - 40)


  if engine:Platform() == "iPad" then
	self.loginPanel = self.hud:AdoptWidget(LoginPanelIpad(self))
  else
	self.loginPanel = self.hud:AdoptWidget(LoginPanel(self))
  end
	
  self.loginNewPanel = self.hud:AdoptWidget(LoginNewPanel(self))
  self.registerPanel = self.hud:AdoptWidget(RegisterPanel(self))
  self.serialPanel = self.hud:AdoptWidget(SerialPanel())

--~   self.pbemPanel = PbemPanel()
--~   self.pbemPanel:Hide()
--~   self.hud:AdoptWidget(self.pbemPanel)

  self.pbemPanel = self.hud:AdoptWidget(PbemPanel(parent))
  self.pbemPanel:Hide()

end

function MultiplayerMenuScreen:OnEnter()

  pbem_ready = {}

  self.hud:GetImage("menu_bar_image"):SetSource(0, 80, 1024, 40);
  
  if engine:Platform() == "iPad" then
	self.logged = 1
  end
  
  if self.logged == nil then
    self.serialPanel:Show()
    self.error = pbem:CheckSerial(function(result) self:Connect(result) end)
  elseif self.logged == 1 then
    self.loginPanel:Show()
  else
    self.pbemPanel:Show()
  end

  local language = GetUserString("lang", "english")
  if language == "english" then
    self.hud:GetImage("language_english"):SetImage("graphics/hud/flags", "english_sel.png")
    self.hud:GetImage("language_german"):SetImage("graphics/hud/flags", "german.png")
    self.hud:GetImage("language_italian"):SetImage("graphics/hud/flags", "italian.png")
    self.hud:GetImage("language_french"):SetImage("graphics/hud/flags", "french.png")
  elseif language == "german" then
    self.hud:GetImage("language_english"):SetImage("graphics/hud/flags", "english.png")
    self.hud:GetImage("language_german"):SetImage("graphics/hud/flags", "german_sel.png")
    self.hud:GetImage("language_italian"):SetImage("graphics/hud/flags", "italian.png")
    self.hud:GetImage("language_french"):SetImage("graphics/hud/flags", "french.png")
  elseif language == "italian" then
    self.hud:GetImage("language_english"):SetImage("graphics/hud/flags", "english.png")
    self.hud:GetImage("language_german"):SetImage("graphics/hud/flags", "german.png")
    self.hud:GetImage("language_italian"):SetImage("graphics/hud/flags", "italian_sel.png")
    self.hud:GetImage("language_french"):SetImage("graphics/hud/flags", "french.png")
  elseif language == "french" then
    self.hud:GetImage("language_english"):SetImage("graphics/hud/flags", "english.png")
    self.hud:GetImage("language_german"):SetImage("graphics/hud/flags", "german.png")
    self.hud:GetImage("language_italian"):SetImage("graphics/hud/flags", "italian.png")
    self.hud:GetImage("language_french"):SetImage("graphics/hud/flags", "french_sel.png")
  end

end

function MultiplayerMenuScreen:OnExit()
  --pbem:LogOut()
end

function MultiplayerMenuScreen:Connect(result)

  self.serialPanel:Hide()
--~   LogUi("connected")
  if result == 0 then
    if self:CompareVersions(engine.versionString, pbem.versionString) then
      self.logged = 1
      self.loginPanel:Show()
    else
      engine.screen.hud:AdoptWidget(ErrorPopup("pbem.pbem_200"))
    end
  elseif result == 5 or result == 301 then
    self.loginNewPanel:Show()
  elseif result == 100 then
    local yes_func = function()
      self.serialPanel:Show()
      pbem:CheckSerial(function(result) self:Connect(result) end)
    end
    local no_func = function()
      engine:SetScreen(screens.mainMenu)
    end
    engine.screen.hud:AdoptWidget(ErrorPopup("time_out_retry", yes_func, no_func))
  else
    engine.screen.hud:AdoptWidget(ErrorPopup("pbem_" .. result))
  end
end

function MultiplayerMenuScreen:ParseVersionString(str)
  local sep1 = string.find(str, "%.", 0)
  local sep2 = string.find(str, "%.", sep1 + 1)

  local ver = {}
  ver.major = tonumber(string.sub(str, 0, sep1 - 1))
  ver.minor = tonumber(string.sub(str, sep1 + 1, sep2 - 1))
  ver.revision = tonumber(string.sub(str, sep2 + 1))
  return ver;
end

function MultiplayerMenuScreen:CompareVersions(engine, server)
  local engineVer = self:ParseVersionString(engine)
  local serverVer = self:ParseVersionString(server)

  if engineVer.major >= serverVer.major and engineVer.minor >= serverVer.minor then
    return true
  end

  return false
end

function MultiplayerMenuScreen:Login(login, pass)

  pbem:LogIn(login, pass, false, function(error) self:LoginResult(error) end)

  self.loginPanel.loginButton:Hide()

end

function MultiplayerMenuScreen:LoginResult(result)

  self.loginPanel.loginButton:Hide()

  if result == 0 then
    self.loginPanel:Hide()
    self.logged = 2
    pbem:SetUserInt("CTGW_avatar", settings.avatar, function(error) self:SetAvatar(error) end)

  else
    engine.screen.hud:AdoptWidget(ErrorPopup("pbem.pbem_" .. result))

    self.loginPanel.loginButton:Show()
  end
end

function MultiplayerMenuScreen:SetAvatar(error)

  if error == 0 then
    SetUserInt("avatar", settings.avatar)
    self.pbemPanel:Show()

  else
    --engine.screen.hud:AdoptWidget(ErrorPopup("pbem.pbem_" .. error))

    self.pbemPanel:Show()

    self.loginPanel.loginButton:Show()
  end
end


function MultiplayerMenuScreen:Register(login, pass, email)

--~   function registerFunc()
--~     result = utils.makeTable(pbem.result)

--~     if tonumber(result[1]) == 0 then
--~       self.statusText:SetText("Logged in")
--~       self:RefreshMembers()
--~     else
--~       self.statusText:SetText(pbem.errorMessage)
--~       --engine.screen.hud:AdoptWidget(ErrorPopup("pbem.pbem_" .. result))
--~     end
--~   end

  pbem:RegisterUser(login, pass, email, function(error) self:RegisterResult(error) end)

  self.registerPanel.registerButton:Hide()

end

function MultiplayerMenuScreen:RegisterResult(result)
LogUi("register result")
  self.registerPanel.registerButton:Show()
LogUi("register result button shown")
  if result == 0 then
    --self.loginPanel:Show()
    LogUi("no error")
    self.registerPanel:Hide()
    LogUi("register result hidden")
    self.serialPanel:Show()
    LogUi("serial panel shown")
    self.error = pbem:CheckSerial(function(result) LogUi("check check succes") self:Connect(result) end)
    --engine:SetScreen(screens.multiplayerMenu)
  else
    --return pbem:ErrorMessage(result)
    engine.screen.hud:AdoptWidget(ErrorPopup("pbem.pbem_" .. result))
  end
end

function MultiplayerMenuScreen:RegisterExistingResult(result)

  if result == 0 then
    self.loginNewPanel:Hide()

    self.serialPanel:Show()

    self.error = pbem:CheckSerial(function(result) LogUi("check check succes") self:Connect(result) end)
  else
    engine.screen.hud:AdoptWidget(ErrorPopup("pbem.pbem_" .. result))
  end
end

--~ function MultiplayerMenuScreen:RefreshMembers()

--~   pbem:ListMembers()
--~   --members = makeTable(pbem.result)

--~   if self.lobbyPanel == nil then
--~     self.lobbyPanel = self.pbemPanel:AddWidget({type = "panel", position = {600, 300}})
--~   end

--~   self.lobbyPanel:RemoveWidgets()

--~   local i = 0
--~   for member in pbem.result do
--~     self.lobbyPanel:AddWidget({type = "text", text = member, position  = {0, i * 30}})
--~     i = i + 1
--~   end

--~ end



--------------------------------------------------------------------------------------


class 'SerialPanel' (LuaPanel)

function SerialPanel:__init(pbemScreen)
  LuaPanel.__init(self)

  self:SetBackgroundImage("login_bg")

  screenAlign(self, 0.5, 0.5)

  self:AddWidget({
    type = "image",
    position = {62, 54},
    image = "login_box",
  })

  self:AddWidget({
    type = "text",
    text = "checking_serial",
    align = "center",
    position = {132, 238},
    color = {226, 224, 223},
  })

  self.video = self:AddWidget({type = "animation"});
  self.video:SetScaling(1, 1)
  self.video:SetPosition(100, 85)

  self:Hide()

end

function SerialPanel:OnShow()
  self.video:PlayFile("ai_gearwheel");
end

--------------------------------------------------------------------------------------


class 'LoginPanel' (LuaPanel)

function LoginPanel:__init(pbemScreen)
  LuaPanel.__init(self)

  self:SetBackgroundImage("login_bg")

  settings.avatar = GetUserInt("avatar", 0)
  settings.avatar2 = "none"

  screenAlign(self, 0.5, 0.5)

  self.login = self:AddWidget({
    type = "text",
    position = {132, 39},
    align = "center",
    font = "hud16_font",
    color = {247, 245, 241},
  })

  self:AddWidget({
    type = "image",
    position = {62, 54},
    image = "login_box",
  })

  self.avatar = self:AddWidget({
    type = "image",
    position = {67, 59},
    image = "../commanders/commander_" .. settings.avatar,
    tooltip = "change_avatar",
    OnScroll = function(this, delta)
      if delta == 1 then
        settings.avatar = settings.avatar - 1
        if settings.avatar < 0 then
          settings.avatar = #data.commanders - 1
        end
        self.avatar:SetImage("graphics/commanders", "commander_" .. settings.avatar .. ".png")
      else
        settings.avatar = settings.avatar + 1
        if settings.avatar >= #data.commanders then
          settings.avatar = 0
        end
        self.avatar:SetImage("graphics/commanders", "commander_" .. settings.avatar .. ".png")
      end
    end
  })

  self:AddWidget({
    type = "text",
    text = "<",
    align = "center",
    position = {38, 120},
    color = {226, 224, 223},
    font = "title_font",
    tooltip ="previous_avatar",
    sfx = "click",
    Clicked = function()
      settings.avatar = settings.avatar - 1
      if settings.avatar < 0 then
        settings.avatar = #data.commanders - 1
      end
      --SetUserInt("avatar", settings.avatar)
      self.avatar:SetImage("graphics/commanders", "commander_" .. settings.avatar .. ".png")
    end
  })

  self:AddWidget({
    type = "text",
    text = ">",
    align = "center",
    position = {222, 120},
    color = {226, 224, 223},
    font = "title_font",
    tooltip ="next_avatar",
    sfx = "click",
    Clicked = function()
      settings.avatar = settings.avatar + 1
      if settings.avatar >= #data.commanders then
        settings.avatar = 0
      end
      --SetUserInt("avatar", settings.avatar)
      self.avatar:SetImage("graphics/commanders", "commander_" .. settings.avatar .. ".png")
    end
  })

  self:AddWidget({
    type = "text",
    text = "login_pw",
    align = "center",
    position = {132, 220-15},
    color = {226, 224, 223},
  })

  self:AddWidget({
    type = "text",
    text = "logging_in",
    align = "center",
    position = {132, 279},
    color = {226, 224, 223},
  })

  self.passEdit = self:AddWidget({
    type = "edit",
    position = {31, 235-15},
    size = {width = 200, height = 20},
    sfx = "click",
    EnterPressed = function(edit)
      pbemScreen:Login(pbem.playerName, self.passEdit.text)
    end
  })
  self.passEdit:SetEchoMode(Edit.ECHO_PASSWORD)

  if settings.rememberPW == 1 then
    -- Generate PW
    local char = {}
    local pw = ""
    for i = 1, 20 do
      char[i] = GetUserInt("character_" .. i, -1)
      if char[i] == -1 then
        char[i] = nil
        break
      else
        pw = pw .. string.char(char[i])
      end
    end
    self.passEdit:SetText(pw)
  end

  local rememberOnEffect = function()
    settings.rememberPW = 1
    SetUserInt("rememberPW", settings.rememberPW)
    if GetUserInt("tutorial_rememberPW", 0) ~= 1 and engine.screen ~= nil then
      MenuAdvicePopup("rememberPW")
      SetUserInt("tutorial_rememberPW", 1)
    end
  end
  local rememberOffEffect = function() settings.rememberPW = 0 SetUserInt("rememberPW", settings.rememberPW) end
  self:AdoptWidget(CheckBox(31, 245, "remember_pw", rememberOnEffect, rememberOffEffect, settings.rememberPW))

  self.statusText = self:AddWidget({type = "text", position = {285, 148}, text = "not_logged"})
  self.statusText:Hide()

  self.loginButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {75, 265},
    sfx = "click",
    text = "login",
    Clicked = function()
      pbemScreen:Login(pbem.playerName, self.passEdit.text)

      -- Store PW per char
      if settings.rememberPW == 1 then
        local len = string.len(self.passEdit.text)
        if len > 0 then
          for i = 1, 30 do
            if i <= len then
              SetUserInt("character_" .. i, string.byte(self.passEdit.text, i))
            else
              SetUserInt("character_" .. i, -1)
            end
          end
        end
      else
        for i = 1, 30 do
          SetUserInt("character_" .. i, -1)
        end
      end
    end
  })

  self:Hide()

end

function LoginPanel:OnShow()
  self.login:SetText(pbem.playerName)
end

class 'LoginPanelIpad' (LuaPanel)

function LoginPanelIpad:__init(pbemScreen)
  LuaPanel.__init(self)
  LogUi("using iPad login")
  self:SetBackgroundImage("login_bg")
  settings.avatar = GetUserInt("avatar", 0)
  screenAlign(self, 0.5, 0.5)

  self:AddWidget({
    type = "text",
    text = "enter_account",
    align = "center",
    position = {132, 50},
    color = {226, 224, 223},
  })

  self:AddWidget({
    type = "text",
    text = "login_name",
    align = "center",
    position = {132, 75},
    color = {226, 224, 223},
  })

  self:AddWidget({
    type = "text",
    text = "login_pw",
    align = "center",
    position = {132, 120},
    color = {226, 224, 223},
  })

  self:AddWidget({
    type = "text",
    text = "logging_in",
    align = "center",
    position = {132, 179},
    color = {226, 224, 223},
  })

  self.nameEdit = self:AddWidget({
    type = "edit",
    position = {31, 90},
    size = {width = 200, height = 20},
    sfx = "click",
    EnterPressed = function(edit)
       pbemScreen:Login(self.nameEdit.text, self.passEdit.text)
	   SetUserString("ipad_username", self.nameEdit.text)
	  self.loginButton:Hide()
    end
  })

  self.passEdit = self:AddWidget({
    type = "edit",
    position = {31, 135},
    size = {width = 200, height = 20},
    sfx = "click",
    EnterPressed = function(edit)
      pbemScreen:Login(self.nameEdit.text, self.passEdit.text)
	  SetUserString("ipad_username", self.nameEdit.text)
	  self.loginButton:Hide()
    end
  })
  self.passEdit:SetEchoMode(Edit.ECHO_PASSWORD)

  self.loginButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {75, 165},
    sfx = "click",
    text = "log_in",
    Clicked = function()
      pbemScreen:Login(self.nameEdit.text, self.passEdit.text)
	  SetUserString("ipad_username", self.nameEdit.text)
	  self.loginButton:Hide()
    end
  })

  self:AddWidget({
    type = "text",
    text = "no_account",
    align = "center",
    position = {132, 250},
    color = {226, 224, 223},
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {75, 265},
    sfx = "click",
    text = "register",
    Clicked = function()
      self:Hide()
      pbemScreen.registerPanel:Show()
    end
  })
  
  self.nameEdit:SetText(GetUserString("ipad_username", ""))

  self.pbemScreen = pbemScreen

  self:Hide()

end

--------------------------------------------------------------------------------------


class 'LoginNewPanel' (LuaPanel)

function LoginNewPanel:__init(pbemScreen)
  LuaPanel.__init(self)

  self:SetBackgroundImage("login_bg")

  screenAlign(self, 0.5, 0.5)

  self:AddWidget({
    type = "text",
    text = "enter_account",
    align = "center",
    position = {132, 50},
    color = {226, 224, 223},
  })

  self:AddWidget({
    type = "text",
    text = "login_name",
    align = "center",
    position = {132, 75},
    color = {226, 224, 223},
  })

  self:AddWidget({
    type = "text",
    text = "login_pw",
    align = "center",
    position = {132, 120},
    color = {226, 224, 223},
  })

  self:AddWidget({
    type = "text",
    text = "logging_in",
    align = "center",
    position = {132, 179},
    color = {226, 224, 223},
  })

  self.nameEdit = self:AddWidget({
    type = "edit",
    position = {31, 90},
    size = {width = 200, height = 20},
    sfx = "click",
    EnterPressed = function(edit)
      self:Login(self.nameEdit.text, self.passEdit.text)
    end
  })

  self.passEdit = self:AddWidget({
    type = "edit",
    position = {31, 135},
    size = {width = 200, height = 20},
    sfx = "click",
    EnterPressed = function(edit)
      self:Login(self.nameEdit.text, self.passEdit.text)
    end
  })
  self.passEdit:SetEchoMode(Edit.ECHO_PASSWORD)

  self.loginButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {75, 165},
    sfx = "click",
    text = "log_in",
    Clicked = function()
      self:Login(self.nameEdit.text, self.passEdit.text)
    end
  })

  self:AddWidget({
    type = "text",
    text = "no_account",
    align = "center",
    position = {132, 250},
    color = {226, 224, 223},
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {75, 265},
    sfx = "click",
    text = "register",
    Clicked = function()
      self:Hide()
      pbemScreen.registerPanel:Show()
    end
  })

  self.pbemScreen = pbemScreen

  self:Hide()

end


function LoginNewPanel:Login(login, pass)
  pbem:LogIn(login, pass, true, function(error) self.pbemScreen:RegisterExistingResult(error) end)

  self.loginButton:Hide()
end

function LoginNewPanel:LoginResult(result)

  self.loginButton:Show()

  if result == 0 then
    self:Hide()

    self.parent.serialPanel:Show()

    pbem:CheckSerial(function(result) self.parent:Connect(result) end)
  else
    --return pbem:ErrorMessage(result)
    engine.screen.hud:AdoptWidget(ErrorPopup("pbem.pbem_" .. result))
  end
end

--------------------------------------------------------------------------------------


class 'RegisterPanel' (LuaPanel)

function RegisterPanel:__init(pbemScreen)
  LuaPanel.__init(self)

  self:SetBackgroundImage("login_bg")
  screenAlign(self, 0.5, 0.5)

  self:AddWidget({
    type = "text",
    position = {132, 39},
    align = "center",
    font = "hud16_font",
    color = {247, 245, 241},
    text = "new_account"
  })

  self:AddWidget({
    type = "text",
    text = "login_name",
    position = {132, 70},
    align = "center",
    color = {226, 224, 223},
  })

  self.loginEdit = self:AddWidget({
    type = "edit",
    position = {31, 85},
    size = {width = 200, height = 20},
    sfx = "click",
  })

  self:AddWidget({
    type = "text",
    text = "login_email",
    position = {132, 120},
    align = "center",
    color = {226, 224, 223},
  })

  self.emailEdit = self:AddWidget({
    type = "edit",
    position = {31, 135},
    size = {width = 200, height = 20},
    sfx = "click",
  })

  self:AddWidget({
    type = "text",
    text = "login_pw",
    position = {132, 170},
    align = "center",
    color = {226, 224, 223},
  })

  self.passEdit = self:AddWidget({
    type = "edit",
    position = {31, 185},
    size = {width = 200, height = 20},
    sfx = "click",
  })
  self.passEdit:SetEchoMode(Edit.ECHO_PASSWORD)

  self:AddWidget({
    type = "text",
    text = "login_pw_repeat",
    align = "center",
    position = {132, 220},
    color = {226, 224, 223},
  })

  self.passEditR = self:AddWidget({
    type = "edit",
    position = {31, 235},
    size = {width = 200, height = 20},
    sfx = "click",
  })
  self.passEditR:SetEchoMode(Edit.ECHO_PASSWORD)

  self.statusText = self:AddWidget({type = "text", position = {285, 148}, text = "not_logged"})
  self.statusText:Hide()

  self.registerButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {75, 265},
    sfx = "click",
    text = "register",
    Clicked = function()
      pbemScreen:Register(self.loginEdit.text, self.passEdit.text, self.emailEdit.text)
    end
  })

  self:Hide()

end


--------------------------------------------------------------------------------------

--~ pbem:CheckSerial(function(result) self:Connect(result) end)

--~ function CallPBEMFunction(func, error_func, timeout_extra)

--~   func()

--~ end
