sPBEMGame = LoadSample("new_pbem.wav")

frontEnd = {}
frontEnd.refreshing = false

--test
frontEnd.accepting = 1

function PbemGameCreateErrorHandler(errNo)
  local yes_func = function()
    engine:CreatePbemGame(frontEnd.selectedMPGame, PbemGameCreateErrorHandler)
  end
  local no_func = function()
    engine:SetScreen(screens.multiplayerMenu)
  end
  engine.screen.hud:AdoptWidget(ErrorPopup("time_out_retry", yes_func, no_func))
end

class 'PbemPanel' (LuaPanel)

function PbemPanel:__init()
  LuaPanel.__init(self)

  self:AddWidget({type = "image", image = "pbem_panel"})

  if engine.window.height < 768 then
    self:SetPosition(78, 5)
  else
    self:SetPosition(78 + (engine.window.width-1024) / 2, 120 + (engine.window.height-768) / 2)
  end
  --self:SetPosition((engine.window.width - self.width) / 2, (engine.window.height - self.height) / 2)

--~   self.tabImages = self:AddWidget({type = "image", image = "pbem_tabs"})
--~   self.tabImages:SetPosition(9, 13)
--~   self.tabImages:SetSource(0, 0, self.tabImages.width, self.tabImages.height / 4)

  self.allGamesTab= self:AdoptWidget(AllGamesPanel())
  self.myTurnsTab= self:AdoptWidget(MyTurnsPanel())
  self.myChallengesTab = self:AdoptWidget(MyChallengesPanel())
  self.acceptChallengeTab = self:AdoptWidget(AcceptChallengePanel())
  self.gameSetup= self:AdoptWidget(PBEMGameSetupPanel())

  self.tabs = {}
  self.tabs[0] = self:AddWidget({
    type = "image",
    image = "pbem_tab_1",
    position = {10, 14},
    sfx = "click",
    Clicked = function()
      self:SetPanel(self.allGamesTab)
    end
  })
  self.tabs[1] = self:AddWidget({
    type = "image",
    image = "pbem_tab_2",
    position = {220, 14},
    sfx = "click",
    Clicked = function()
      self:SetPanel(self.myTurnsTab)
    end
  })
  self.tabs[2] = self:AddWidget({
    type = "image",
    image = "pbem_tab_3",
    position = {436, 14},
    sfx = "click",
    Clicked = function()
      self:SetPanel(self.myChallengesTab)
    end
  })
  self.tabs[3] = self:AddWidget({
    type = "image",
    image = "pbem_tab_4",
    position = {653, 14},
    sfx = "click",
    Clicked = function()
      self:SetPanel(self.acceptChallengeTab)
    end
  })

  -- Panel Tab Labels
  self:AddWidget({
    type = "label",
    text = "all_games",
    font = "menu_font",
    position = {111, 29},
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    align = "center",
    sfx = "click",
    Clicked = function()
      self:SetPanel(self.allGamesTab)
    end
  })
  self:AddWidget({
    type = "label",
    text = "my_turns",
    font = "menu_font",
    position = {328, 29},
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    align = "center",
    sfx = "click",
    Clicked = function()
      self:SetPanel(self.myTurnsTab)
    end
  })
  self:AddWidget({
    type = "label",
    text = "my_challenges",
    font = "menu_font",
    position = {546, 29},
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    align = "center",
    sfx = "click",
    Clicked = function()
      self:SetPanel(self.myChallengesTab)
    end
  })
  self:AddWidget({
    type = "label",
    text = "open_challenges",
    font = "menu_font",
    position = {763, 29},
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    align = "center",
    sfx = "click",
    Clicked = function()
      self:SetPanel(self.acceptChallengeTab)
    end
  })

  self.gameInfo = self:AdoptWidget(PbemGameInfoPanel())

  self.passwordPanel = self:AdoptWidget(PasswordPanel())

  self.acceptPanel = self:AdoptWidget(AcceptedPanel())

  self.waitPanel = self:AdoptWidget(WaitingPanel())

  self.autoRefreshOn = false

end

function PbemPanel:OnShow()
  -- Open default panel

  self:SetPanel(self.allGamesTab)

  self.allGamesTab.selectedGame = nil
  self.myTurnsTab.selectedGame = nil
  self.myChallengesTab.selectedGame = nil
  self.acceptChallengeTab.selectedGame = nil

--~   self.allGamesTab.list:GetImage("pbem_selected_game"):Hide()
--~   self.myTurnsTab.list:GetImage("pbem_selected_game"):Hide()
--~   self.myChallengesTab.list:GetImage("pbem_selected_game"):Hide()
--~   self.acceptChallengeTab.list:GetImage("pbem_selected_game"):Hide()

  --if not self.autoRefreshOn then
  --  self.autoRefreshOn = true

  --end
  
  if pbemCurrentTab.list ~= nil and not frontEnd.refreshing then
    pbemCurrentTab.list:Refresh(pbemCurrentTab.filter)
  end
end

function PbemPanel:OnHide()
  self:CancelAutoRefresh()
end

function PbemPanel:SetPanel(panel)

  if pbemCurrentTab ~= nil then
    pbemCurrentTab:Hide()

    pbemPreviousTab = pbemCurrentTab
  end

  self.gameInfo:Refresh()

  pbemCurrentTab = panel

  pbemCurrentTab:Show()

  if pbemCurrentTab.list ~= nil then
    pbemCurrentTab.list:Refresh(pbemCurrentTab.filter)
  end

--~   if self.refreshTask ~= nil then
--~     self:CancelAutoRefresh()
--~   end

--~   self:AutoRefresh()
end


function PbemPanel:HandleAcceptedGame()

  pbemCurrentTab.list:Refresh(pbemCurrentTab.filter, "accepted")

  frontEnd.accepting = 1

--~   self:SetPanel(self.allGamesTab)

--~   pbemCurrentTab.list:Refresh(pbemCurrentTab.filter)
end

function PbemPanel:AutoRefresh()
  --if self.autoRefreshOn then
    if pbemCurrentTab.list ~= nil and not frontEnd.refreshing then
      pbemCurrentTab.list:Refresh(pbemCurrentTab.filter)
    end
    self.refreshTask = engine:ScheduleTask(10000, function() self:AutoRefresh() end)
  --end
end

function PbemPanel:CancelAutoRefresh()
  if self.refreshTask ~= nil then
    engine:CancellTask(self.refreshTask)
  end
end

function PbemPanel:CancelChallengeResult(filter, result)
  if result == 0 then
    pbemCurrentTab.list:Refresh(filter, "continue")
  else
    CheckError(result)
    self.waitPanel:Hide()
  end
end

----------------------------------

class 'AllGamesPanel' (LuaPanel)

function AllGamesPanel:__init()
  LuaPanel.__init(self)
  self:Hide()

  self:SetPosition(25, 60)
  --self:SetBackgroundImage("pbem_bg")

  self:AddWidget({
    type = "text",
    text = "pbem_game_scenario",
    color = {67, 56, 47},
    position = {40, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_player",
    color = {67, 56, 47},
    position = {270, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_alliance",
    color = {67, 56, 47},
    position = {420, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_date",
    color = {67, 56, 47},
    position = {620, 15},
  })

  self.scrollPos = 0
  self.listLength = 0
  self:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {784, 15},
    Clicked = function()
      if self.scrollPos > 0 then
        self.scrollPos = self.scrollPos - 10
        --self.list:SetPosition(0, self.scrollPos*-20)
        self.list:QuickRefresh(self.filter)
        self.list:Refresh(self.filter)
      end
    end
  })
  self:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {784, 221},
    Clicked = function()
      if self.scrollPos < self.listLength - 10 then
        self.scrollPos = self.scrollPos + 10
        --self.list:SetPosition(0, self.scrollPos*-20)
        self.list:QuickRefresh(self.filter)
        self.list:Refresh(self.filter)
      end
    end
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {0, 427},
    text = "close",
    sfx = "click",
    Clicked = function()
      engine:SetScreen(screens.mainMenu)
    end
  })

--~   self:AddWidget({
--~     type = "button",
--~     image = "button_default",
--~     position = {133, 427},
--~     text = "log_out",
--~     sfx = "click",
--~     Clicked = function()
--~       pbem:LogOut(function(error) CheckError(error) end)
--~       engine:SetScreen(screens.mainMenu)
--~     end
--~   })

  self:AddWidget({
    type = "text",
    color = {247, 245, 241},
    position = {190+133, 441},
    align = "center",
    text = "refreshing",
  })

  self.refreshButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {133+133, 427},
    text = "refresh",
    sfx = "click",
    Clicked = function()
      self.list:Refresh()
    end
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {573, 427},
    text = "issue_challenge",
    tooltip = "issue_challenge_tip",
    sfx = "click",
    Clicked = function()
      self.parent:SetPanel(self.parent.gameSetup)
    end
  })

  self:AddWidget({
    name = "ButtonRemove",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "remove_result",
    sfx = "click",
    Clicked = function(button)
      self.parent.waitPanel:Show()
      pbem:MarkAsViewed(self.selectedGame,
                        function(error)
                          CheckError(error,
                                      function()
                                        self.list:Refresh(nil, "continue")
                                        button:Hide()
                                      end,
                                      function()
                                        self.parent.waitPanel:Hide()
                                        button:Show()
                                      end)
                        end)
      button:Hide()
    end
  }):Hide()

  self:AddWidget({
    name = "ButtonAccept",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "accept_challenge",
    sfx = "click",
    Clicked = function(button)
      if self.selectedType ~= 4 then
        self.parent.waitPanel:Show()
        assert(self.selectedGame ~= nil, "erroref")
        pbem:AcceptChallenge(self.selectedGame,
                             "",
                             function(error)
                               CheckError(error,
                                          function()
                                            self.parent:HandleAcceptedGame()
                                            button:Show()
                                          end,
                                          function()
                                            self.parent.waitPanel:Hide()
                                            button:Show()
                                          end)
                             end)
      else
        self.parent.passwordPanel:Popup(self.selectedGame)
      end
      button:Hide()
    end
  }):Hide()

  self:AddWidget({
    name = "ButtonPlay",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "play_challenge",
    tooltip = "play_challenge_tip",
    sfx = "click",
    Clicked = function(this)
      self.parent.autoRefreshOn = false
      if not frontEnd.refreshing then
        frontEnd.selectedMPGame = self.selectedGame
        engine:SetScreen(screens.loadingMenu)
        engine:CreatePbemGame(self.selectedGame, PbemGameCreateErrorHandler)
        self.parent:CancelAutoRefresh()
      else
        frontEnd.gameLaunched = self.selectedGame
      end
      this:Hide()
    end
  }):Hide()

  self:AddWidget({
    name = "ButtonCancel",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "cancel_challenge",
    sfx = "click",
    Clicked = function(this)
      self.parent.waitPanel:Show()
      pbem:CancelChallenge(self.selectedGame, function(result) self.parent:CancelChallengeResult(nil, result) end)
      this:Hide()
    end
  }):Hide()

  self:AddWidget({
    name = "ButtonClaim",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "claim_challenge",
    tooltip = "claim_challenge_tip",
    sfx = "click",
    Clicked = function()
      if currentItem ~= nil then
        --engine:CreatePbemGame(self.gamesList.currentItem.gameNo)
      end
    end
  }):Hide()
  
 
  

  self:AddWidget({
    name = "pagebar",
    type = "image",
    image = "pagebar",
    position = {790,36},
  })

  --local turnWarningOnEffect = function() self.warning = 1 end
  --local turnWarningOffEffect = function() self.warning = 0 end
  self.warning = 1
  --self:AdoptWidget(CheckBox(0, 400, "Play sound when game is ready to play", turnWarningOnEffect, turnWarningOffEffect, 1))

  self.list = self:AdoptWidget(GamesListPanel())

  --self.gameInfo = self:AdoptWidget(PbemGameInfoPanel())

end

function AllGamesPanel:OnShow()
  --self.parent.tabImages:SetSource(0, 0, self.parent.tabImages.width, 28)
  self.parent.tabs[0]:SetImage("graphics/hud/", "pbem_tab_1_sel.png")
  self.parent.tabs[1]:SetImage("graphics/hud/", "pbem_tab_2.png")
  self.parent.tabs[2]:SetImage("graphics/hud/", "pbem_tab_3.png")
  self.parent.tabs[3]:SetImage("graphics/hud/", "pbem_tab_4.png")

  self.list:Refresh(self.filter)
end

function AllGamesPanel:OnRefresh()
  self.list:Refresh(self.filter)
end

function AllGamesPanel:SelectGame(gameType, listID, row, gameOver)
  self.selectedGame = listID
  self.selectedType = gameType
  if gameOver then
    self:GetButton("ButtonRemove"):Show()
    self:GetButton("ButtonAccept"):Hide()
    self:GetButton("ButtonCancel"):Hide()
    self:GetButton("ButtonPlay"):Hide()
    self:GetButton("ButtonClaim"):Hide()
  elseif gameType == 0 then
    self:GetButton("ButtonRemove"):Hide()
    self:GetButton("ButtonAccept"):Hide()
    self:GetButton("ButtonCancel"):Hide()
    self:GetButton("ButtonPlay"):Show()
    self:GetButton("ButtonClaim"):Hide()
  elseif gameType == 1 then
    self:GetButton("ButtonRemove"):Hide()
    self:GetButton("ButtonAccept"):Hide()
    self:GetButton("ButtonCancel"):Show()
    self:GetButton("ButtonPlay"):Hide()
    self:GetButton("ButtonClaim"):Hide()
  elseif gameType == 2 then
    self:GetButton("ButtonRemove"):Hide()
    self:GetButton("ButtonAccept"):Hide()
    self:GetButton("ButtonCancel"):Hide()
    self:GetButton("ButtonPlay"):Hide()
    self:GetButton("ButtonClaim"):Show()
  else
    self:GetButton("ButtonRemove"):Hide()
    self:GetButton("ButtonAccept"):Show()
    self:GetButton("ButtonCancel"):Hide()
    self:GetButton("ButtonPlay"):Hide()
    self:GetButton("ButtonClaim"):Hide()
  end
end




----------------------------------

class 'MyTurnsPanel' (LuaPanel)

function MyTurnsPanel:__init()
  LuaPanel.__init(self)
  self:Hide()

  self:SetPosition(25, 60)
  --self:SetBackgroundImage("pbem_bg")

  self:AddWidget({
    type = "text",
    text = "pbem_game_scenario",
    color = {67, 56, 47},
    position = {40, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_player",
    color = {67, 56, 47},
    position = {270, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_alliance",
    color = {67, 56, 47},
    position = {420, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_date",
    color = {67, 56, 47},
    position = {620, 15},
  })

  self.scrollPos = 0
  self.listLength = 0
  self:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {784, 15},
    Clicked = function()
      if self.scrollPos > 0 then
        self.scrollPos = self.scrollPos - 10
        self.list:QuickRefresh(self.filter)
        self.list:Refresh(self.filter)
      end
    end
  })
  self:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {784, 221},
    Clicked = function()
      if self.scrollPos < self.listLength - 10 then
        self.scrollPos = self.scrollPos + 10
        self.list:QuickRefresh(self.filter)
        self.list:Refresh(self.filter)
      end
    end
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {0, 427},
    text = "close",
    sfx = "click",
    Clicked = function()
      engine:SetScreen(screens.mainMenu)
    end
  })

--~   self:AddWidget({
--~     type = "button",
--~     image = "button_default",
--~     position = {133, 427},
--~     text = "log_out",
--~     sfx = "click",
--~     Clicked = function()
--~       pbem:LogOut(function(error) CheckError(error) end)
--~       engine:SetScreen(screens.mainMenu)
--~     end
--~   })

  self:AddWidget({
    type = "text",
    color = {247, 245, 241},
    position = {190+133, 441},
    align = "center",
    text = "refreshing",
  })

  self.refreshButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {133+133, 427},
    text = "refresh",
    sfx = "click",
    Clicked = function()
      self.list:Refresh(self.filter)
    end
  })

  self:AddWidget({
    name = "ButtonRemove",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "remove_result",
    sfx = "click",
    Clicked = function(button)
      pbem:MarkAsViewed(self.selectedGame,
                        function(error)
                          CheckError(error,
                                      function()
                                        button:Show()
                                      end,
                                      function()
                                        button:Show()
                                      end)
                        end)
      button:Hide()
    end
  }):Hide()

  self:AddWidget({
    name = "ButtonPlay",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "play_challenge",
    tooltip = "play_challenge_tip",
    sfx = "click",
    Clicked = function(this)
      self.parent.autoRefreshOn = false
      if not frontEnd.refreshing then
        frontEnd.selectedMPGame = self.selectedGame
        engine:SetScreen(screens.loadingMenu)
        engine:CreatePbemGame(self.selectedGame, PbemGameCreateErrorHandler)
        self.parent:CancelAutoRefresh()
      else
        frontEnd.gameLaunched = self.selectedGame
      end
      this:Hide()
    end
  }):Hide()

  self:AddWidget({
    name = "ButtonClaim",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "claim_challenge",
    tooltip = "claim_challenge_tip",
    sfx = "click",
    Clicked = function()
      if currentItem ~= nil then
        --engine:CreatePbemGame(self.gamesList.currentItem.gameNo)
      end
    end
  }):Hide()

  self:AddWidget({
    name = "pagebar",
    type = "image",
    image = "pagebar",
    position = {790,36},
  })

  --local turnWarningOnEffect = function() self.warning = 1 end
  --local turnWarningOffEffect = function() self.warning = 0 end
  self.warning = 1
  --self:AdoptWidget(CheckBox(0, 400, "Play sound when game is ready to play", turnWarningOnEffect, turnWarningOffEffect, 1))

  self.filter = "myTurns"
  self.list = self:AdoptWidget(GamesListPanel())

  local effectFilter = function(setting)
    if setting == 1 then
      self.filter = "allMatches"
    elseif setting == 2 then
      self.filter = "myTurns"
    elseif setting == 3 then
      self.filter = "opponentTurns"
    elseif setting == 4 then
      self.filter = "results"
    end
    if self.connected then
      self.list:Refresh(self.filter)
    end
    --self.list:Refresh(self.filter)
  end
  self:AdoptWidget(RadioButton(30, 225, 4, "PBEM_filter_", effectFilter, 2, 0, 200))

  --self.gameInfo = self:AdoptWidget(PbemGameInfoPanel())

end

function MyTurnsPanel:OnShow()
  self.connected = true

  --self.parent.tabImages:SetSource(0, 28, self.parent.tabImages.width, 28)
  self.parent.tabs[0]:SetImage("graphics/hud/", "pbem_tab_1.png")
  self.parent.tabs[1]:SetImage("graphics/hud/", "pbem_tab_2_sel.png")
  self.parent.tabs[2]:SetImage("graphics/hud/", "pbem_tab_3.png")
  self.parent.tabs[3]:SetImage("graphics/hud/", "pbem_tab_4.png")

  self.list:Refresh(self.filter)
end

function MyTurnsPanel:OnRefresh()
  self.list:Refresh(self.filter)
end

function MyTurnsPanel:SelectGame(gameType, listID, row, gameOver)
  self.selectedGame = listID
  self.selectedType = gameType
  if gameOver then
    self:GetButton("ButtonRemove"):Show()
    self:GetButton("ButtonPlay"):Hide()
    self:GetButton("ButtonClaim"):Hide()
  elseif gameType == 0 then
    self:GetButton("ButtonRemove"):Hide()
    self:GetButton("ButtonPlay"):Show()
    self:GetButton("ButtonClaim"):Hide()
  elseif gameType == 2 then
    self:GetButton("ButtonRemove"):Hide()
    self:GetButton("ButtonPlay"):Hide()
    self:GetButton("ButtonClaim"):Show()
  else
    self:GetButton("ButtonRemove"):Hide()
    self:GetButton("ButtonPlay"):Hide()
    self:GetButton("ButtonClaim"):Hide()
  end
end


----------------------------------

class 'MyChallengesPanel' (LuaPanel)

function MyChallengesPanel:__init()
  LuaPanel.__init(self)
  self:Hide()

  self:SetPosition(25, 60)
  --self:SetBackgroundImage("pbem_bg")

  self:AddWidget({
    type = "text",
    text = "pbem_game_scenario",
    color = {67, 56, 47},
    position = {40, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_player",
    color = {67, 56, 47},
    position = {270, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_alliance",
    color = {67, 56, 47},
    position = {420, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_date",
    color = {67, 56, 47},
    position = {620, 15},
  })

  self.scrollPos = 0
  self.listLength = 0
  self:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {784, 15},
    Clicked = function()
      if self.scrollPos > 0 then
        self.scrollPos = self.scrollPos - 10
        self.list:QuickRefresh(self.filter)
        self.list:Refresh(self.filter)
      end
    end
  })
  self:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {784, 221},
    Clicked = function()
      if self.scrollPos < self.listLength - 10 then
        self.scrollPos = self.scrollPos + 10
        self.list:QuickRefresh(self.filter)
        self.list:Refresh(self.filter)
      end
    end
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {0, 427},
    text = "close",
    sfx = "click",
    Clicked = function()
      engine:SetScreen(screens.mainMenu)
    end
  })

--~   self:AddWidget({
--~     type = "button",
--~     image = "button_default",
--~     position = {133, 427},
--~     text = "log_out",
--~     sfx = "click",
--~     Clicked = function()
--~       pbem:LogOut(function(error) CheckError(error) end)
--~       engine:SetScreen(screens.mainMenu)
--~     end
--~   })

  self:AddWidget({
    type = "text",
    color = {247, 245, 241},
    position = {190+133, 441},
    align = "center",
    text = "refreshing",
  })

  self.refreshButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {133+133, 427},
    text = "refresh",
    sfx = "click",
    Clicked = function()
      self.list:Refresh("myChallenges")
    end
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {573, 427},
    text = "issue_challenge",
    tooltip = "issue_challenge_tip",
    sfx = "click",
    Clicked = function()
      self.parent:SetPanel(self.parent.gameSetup)
    end
  })

  self:AddWidget({
    name = "ButtonCancel",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "cancel_challenge",
    sfx = "click",
    Clicked = function(this)
      self.parent.waitPanel:Show()
      pbem:CancelChallenge(self.selectedGame, function(result) self.parent:CancelChallengeResult("myChallenges", result) end)
      this:Hide()
    end
  }):Hide()

  self:AddWidget({
    name = "pagebar",
    type = "image",
    image = "pagebar",
    position = {790,36},
  })

  self.filter = "myChallenges"
  self.list = self:AdoptWidget(GamesListPanel())

end

function MyChallengesPanel:OnShow()
  --self.parent.tabImages:SetSource(0, 56, self.parent.tabImages.width, 28)
  self.parent.tabs[0]:SetImage("graphics/hud/", "pbem_tab_1.png")
  self.parent.tabs[1]:SetImage("graphics/hud/", "pbem_tab_2.png")
  self.parent.tabs[2]:SetImage("graphics/hud/", "pbem_tab_3_sel.png")
  self.parent.tabs[3]:SetImage("graphics/hud/", "pbem_tab_4.png")

  self.list:Refresh(self.filter)
end

function MyChallengesPanel:OnRefresh()
  self.list:Refresh(self.filter)
end

function MyChallengesPanel:SelectGame(gameType, listID, row)
  self.selectedGame = listID
  self.selectedType = gameType
  if gameType == 1 then
    self:GetButton("ButtonCancel"):Show()
  else
    self:GetButton("ButtonCancel"):Hide()
  end
end



----------------------------------

class 'AcceptChallengePanel' (LuaPanel)

function AcceptChallengePanel:__init()
  LuaPanel.__init(self)

  self:SetPosition(25, 60)
  --self:SetBackgroundImage("pbem_bg")

  self:AddWidget({
    type = "text",
    text = "pbem_game_scenario",
    color = {67, 56, 47},
    position = {40, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_player",
    color = {67, 56, 47},
    position = {270, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_alliance",
    color = {67, 56, 47},
    position = {420, 15},
  })

  self:AddWidget({
    type = "text",
    text = "pbem_game_date",
    color = {67, 56, 47},
    position = {620, 15},
  })

  self.scrollPos = 0
  self.listLength = 0
  self:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {784, 15},
    Clicked = function()
      if self.scrollPos > 0 then
        self.scrollPos = self.scrollPos - 10
        self.list:QuickRefresh(self.filter)
        self.list:Refresh(self.filter)
      end
    end
  })
  self:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {784, 221},
    Clicked = function()
      if self.scrollPos < self.listLength - 10 then
        self.scrollPos = self.scrollPos + 10
        self.list:QuickRefresh(self.filter)
        self.list:Refresh(self.filter)
      end
    end
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {0, 427},
    text = "close",
    sfx = "click",
    Clicked = function()
      engine:SetScreen(screens.mainMenu)
    end
  })

--~   self:AddWidget({
--~     type = "button",
--~     image = "button_default",
--~     position = {133, 427},
--~     text = "log_out",
--~     sfx = "click",
--~     Clicked = function()
--~       pbem:LogOut(function(error) CheckError(error) end)
--~       engine:SetScreen(screens.mainMenu)
--~     end
--~   })

  self:AddWidget({
    type = "text",
    color = {247, 245, 241},
    position = {190+133, 441},
    align = "center",
    text = "refreshing",
  })

  self.refreshButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {133+133, 427},
    text = "refresh",
    sfx = "click",
    Clicked = function()
      self.list:Refresh("openChallenges")
    end
  })

  self:AddWidget({
    name = "ButtonAccept",
    type = "button",
    image = "button_default",
    position = {706, 427},
    text = "accept_challenge",
    sfx = "click",
    Clicked = function(button)
      if self.selectedType ~= 4 then
        self.parent.waitPanel:Show()
        assert(self.selectedGame ~= nil, "erroref")
        pbem:AcceptChallenge(self.selectedGame,
                             "",
                             function(error)
                               CheckError(error,
                                          function()
                                            self.parent:HandleAcceptedGame()
                                            button:Show()
                                          end,
                                          function()
                                            self.parent.waitPanel:Hide()
                                            button:Show()
                                          end)
                             end)
      else
        self.parent.passwordPanel:Popup(self.selectedGame)
      end
      button:Hide()
    end
  }):Hide()

  self:AddWidget({
    name = "pagebar",
    type = "image",
    image = "pagebar",
    position = {790,36},
  })

  self.filter = "openChallenges"
  self.list = self:AdoptWidget(GamesListPanel())

  self:Hide()

end

function AcceptChallengePanel:OnShow()
  --self.parent.tabImages:SetSource(0, 84, self.parent.tabImages.width, 28)
  self.parent.tabs[0]:SetImage("graphics/hud/", "pbem_tab_1.png")
  self.parent.tabs[1]:SetImage("graphics/hud/", "pbem_tab_2.png")
  self.parent.tabs[2]:SetImage("graphics/hud/", "pbem_tab_3.png")
  self.parent.tabs[3]:SetImage("graphics/hud/", "pbem_tab_4_sel.png")

  self.list:Refresh(self.filter)
end

function AcceptChallengePanel:OnRefresh()
  self.list:Refresh(self.filter)
end

function AcceptChallengePanel:SelectGame(gameType, listID, row)
  self.selectedGame = listID
  self.selectedType = gameType
  if gameType == 3 or gameType == 4 then
    self:GetButton("ButtonAccept"):Show()
  else
    self:GetButton("ButtonAccept"):Hide()
  end
end

-----------------------------------------------

class 'GamesListPanel' (LuaPanel)

function GamesListPanel:__init()
  LuaPanel.__init(self)

  self.numGames = 0
end

function GamesListPanel:Refresh(filter, afterAction)

  self.parent.refreshButton:Hide()

  frontEnd.refreshing = true

  -- build lists
  if filter == nil then
    pbem:ListGames(function() pbem:ListChallenges(function() self:ClearList() self:RefreshGamesList() self:RefreshChallengesList() self:PostRefresh(afterAction) end) end)

  elseif filter == "myTurns" or filter == "allMatches" or filter == "opponentTurns" or filter == "results" then
    pbem:ListGames(function()  self:ClearList() self:RefreshGamesList(filter) self:PostRefresh(afterAction) end)
  elseif filter == "myChallenges" then
    pbem:ListChallenges(function() self:ClearList() self:RefreshChallengesList("own") self:PostRefresh(afterAction) end)
  elseif filter == "openChallenges" then
    pbem:ListChallenges(function() self:ClearList() self:RefreshChallengesList("other") self:PostRefresh(afterAction) end)
  end
end


function GamesListPanel:QuickRefresh(filter)

  -- build lists
  if filter == nil then
    self:ClearList()
    self:RefreshGamesList()
    self:RefreshChallengesList()
    self:PostRefresh()
  elseif filter == "myTurns" or filter == "allMatches" or filter == "opponentTurns" or filter == "results" then
    self:ClearList()
    self:RefreshGamesList(filter)
    self:PostRefresh()
  elseif filter == "myChallenges" then
    self:ClearList()
    self:RefreshChallengesList("own")
    self:PostRefresh()
  elseif filter == "openChallenges" then
    self:ClearList()
    self:RefreshChallengesList("other")
    self:PostRefresh()
  end
end

function GamesListPanel:PostRefresh(afterAction)
  frontEnd.refreshing = false
  if frontEnd.gameLaunched ~= nil then
    frontEnd.selectedMPGame = self.selectedGame
    engine:SetScreen(screens.loadingMenu)
    engine:CreatePbemGame(frontEnd.gameLaunched, PbemGameCreateErrorHandler)
    self.parent.parent:CancelAutoRefresh()
    frontEnd.gameLaunched = nil
  end

  self.parent.refreshButton:Show()

  self.pageCount = math.floor((self.numGames+9)/10)

  self.parent.listLength = self.numGames

  self.parent:GetImage("pagebar"):SetSource(0, 0, 6, 181 / self.pageCount)
  self.parent:GetImage("pagebar"):SetPosition(790, 36 + (181* ((self.parent.scrollPos)/10) / self.pageCount) )

  if afterAction == "hosted" then
    self.parent.parent:SetPanel(pbemPreviousTab)
    self.parent.parent.waitPanel:Hide()
    --FIXME: add popup to inform player new challenge is at bottom of his list
  elseif afterAction == "accepted" then
    self.parent.parent.waitPanel:Hide()
    --self.parent.parent:HandleAcceptedGame(self.selectedGame)
  elseif afterAction == "continue" then
    self.parent.parent.waitPanel:Hide()
  end
end

function GamesListPanel:ClearList()

  self.numGames = 0

  self:RemoveWidgets()

  self:AddWidget({
    name = "pbem_selected_game",
    type = "image",
    image = "pbem_selection_highlight",
    position = {0, 0},
  }):Hide()

end

function GamesListPanel:RefreshGamesList(filter)
  local row = self.numGames
  local listID = 0
  local opponentSide, playerSide

  -- List "my turn" games
  if filter == nil or filter == "myTurns" or filter == "allMatches" then
    for gameInfo in pbem.games do
      if gameInfo.currentPlayerName == pbem.playerName then
        if gameInfo:GetPlayerName(1) == pbem.playerName then
          playerSide = 1
          opponentSide = 2
        else
          playerSide = 2
          opponentSide = 1
        end
        if not gameInfo.gameOver or not gameInfo.viewed then
          self:AddGameToList(row, 0, gameInfo.id, listID, GetCampaignName(gameInfo.gameName), gameInfo:GetPlayerName(opponentSide), playerSide, gameInfo.dateIssued, nil, gameInfo.gameOver, gameInfo.viewed)
          row = row + 1
        end
      end
      listID = listID + 1
    end
  end

  -- List "opponent turn" challenges
  if filter == nil or filter == "opponentTurns" or filter == "results" or filter == "allMatches" then
    listID = 0
    for gameInfo in pbem.games do
      if gameInfo.currentPlayerName ~= pbem.playerName then
        if gameInfo:GetPlayerName(1) == pbem.playerName then
          playerSide = 1
          opponentSide = 2
        else
          playerSide = 2
          opponentSide = 1
        end

        if not gameInfo.viewed then
          if (filter == "opponentTurns" and not gameInfo.gameOver) or (filter == "results" and gameInfo.gameOver) or (filter ~= "opponentTurns" and filter ~= "results") then
            self:AddGameToList(row, 2, gameInfo.id, listID, GetCampaignName(gameInfo.gameName), gameInfo:GetPlayerName(opponentSide), playerSide, gameInfo.dateIssued, nil, gameInfo.gameOver, gameInfo.viewed)
            row = row + 1
          end
        end
      end
      listID = listID + 1
    end
  end

  self.numGames = row
end

function GamesListPanel:RefreshChallengesList(which)
  local row = self.numGames
  local challengeID = 0
  local listID = 0
  local playerSide

  if which ~= "own" then
    -- List open challenges
    challengeID = 0
    for challenge in pbem.challenges do
      if challenge.challengerName ~= pbem.playerName then
        if challenge.challengerSide == 1 then
          playerSide = 2
        else
          playerSide = 1
        end
        if challenge.isPrivate == false then
          self:AddGameToList(row, 3, challenge.id, challengeID, GetCampaignName(challenge.gameName), challenge.challengerName, playerSide, challenge.dateIssued, challenge.avatar)
        else
          self:AddGameToList(row, 4, challenge.id, challengeID, GetCampaignName(challenge.gameName), challenge.challengerName, playerSide, challenge.dateIssued, challenge.avatar)
        end
        row = row + 1
      end
      challengeID = challengeID + 1
    end
  end

  if which ~= "other" then
    -- List own challenges
    challengeID = 0
    for challenge in pbem.challenges do
      if challenge.challengerName == pbem.playerName then
        --self.challengesList:AddItem(challenge.id .. " " .. challenge.challengerName)
        self:AddGameToList(row, 1, challenge.id, challengeID, GetCampaignName(challenge.gameName), "?", challenge.challengerSide, challenge.dateIssued)
        row = row + 1
      end
      challengeID = challengeID + 1
    end
  end

  self.numGames = row
end

function GamesListPanel:AddGameToList(row, gameType, gameID, listID, campaignName, hostName, hostSide, gameDate, avatar2, gameOver, viewed)

  if row < self.parent.scrollPos or row >= self.parent.scrollPos + 10 then
    return
  end

  local imageName, toolTipText

  if gameOver then
    imageName = "pbem_result"
    toolTipText = "pbem_game_result"
  elseif gameType == 0 then
    imageName = "pbem_you"
    toolTipText = "pbem_game_type_1"
    if pbem_ready[gameID] == nil then
      pbem_ready[gameID] = 1
      if self.parent.warning == 1 then
        PlaySample(sPBEMGame)
      end
      if frontEnd.accepting == 1 then
        -- popup message
        self.parent.parent.acceptPanel:Popup(listID, hostSide, campaignName)
      end
    end
  elseif gameType == 1 then
    imageName = "pbem_waiting"
    toolTipText = "pbem_game_type_2"
  elseif gameType == 2 then
    imageName = "pbem_opponent"
    toolTipText = "pbem_game_type_3"
    if pbem_ready[gameID] == nil then
      pbem_ready[gameID] = 1
      if frontEnd.accepting == 1 then
        -- popup message
        self.parent.parent.acceptPanel:Popup(listID, hostSide, campaignName)
      end
    end
  elseif gameType == 3 then
    imageName = "pbem_open"
    toolTipText = "pbem_game_type_4"
  else
    imageName = "pbem_private"
    toolTipText = "pbem_game_type_5"
  end

  self:AddWidget({
    type = "text",
    text = campaignName,
    color = {67, 56, 47},
    position = {40, 35 + (row-self.parent.scrollPos)*20},
  })
  local textID = self:AddWidget({
    type = "text",
    text = hostName,
    color = {67, 56, 47},
    position = {270, 35 + (row-self.parent.scrollPos)*20},
  })
  if hostSide == 1 then
    --textID:SetColour(70, 104, 10, 255)
    self:AddWidget({
      type = "text",
      text = "entente",
      color = {70, 104, 10},
      position = {420, 35 + (row-self.parent.scrollPos)*20},
    })
  else
    --textID:SetColour(137, 31, 3, 255)
    self:AddWidget({
      type = "text",
      text = "central_powers",
      color = {137, 31, 3, 255},
      position = {420, 35 + (row-self.parent.scrollPos)*20},
    })
  end
  self:AddWidget({
    type = "text",
    text = string.sub(gameDate, 1, -4),
    color = {67, 56, 47},
    position = {620, 35 + (row-self.parent.scrollPos)*20},
  })
  self:AddWidget({
    type = "image",
    image = "pbem_selection_hotspot",
    tooltip = toolTipText,
    position = {10, 32 + (row-self.parent.scrollPos)*20},
    sfx = "click",
    Clicked = function(button)
      if frontEnd.accepting ~= 1 then
        self.parent.parent.gameInfo:Refresh(hostName, campaignName)
        self.parent:SelectGame(gameType, listID, row, gameOver)
        pbem:GetUserInt(hostName, "CTGW_avatar", -1, function(value, error) self:HandleGameSelect(value, error) end)
        self:GetImage("pbem_selected_game"):Show()
        self:GetImage("pbem_selected_game"):SetPosition(10, 31 + (row-self.parent.scrollPos)*20)
      end
    end
  })
  self:AddWidget({
    type = "image",
    image = imageName,
    tooltip = toolTipText,
    position = {18, 33 + (row-self.parent.scrollPos)*20},
    sfx = "click",
    Clicked = function(button)
      if frontEnd.accepting ~= 1 then
        self.parent.parent.gameInfo:Refresh(hostName, campaignName)
        self.parent:SelectGame(gameType, listID, row, gameOver)
        pbem:GetUserInt(hostName, "CTGW_avatar", -1, function(value, error) self:HandleGameSelect(value, error) end)
        self:GetImage("pbem_selected_game"):Show()
        self:GetImage("pbem_selected_game"):SetPosition(10, 31 + (row-self.parent.scrollPos)*20)
      end
    end
  })

  if self.parent.selectedGame == listID and self.parent.selectedType == gameType then
    self:GetImage("pbem_selected_game"):Show()
    self:GetImage("pbem_selected_game"):SetPosition(10, 31 + (row-self.parent.scrollPos)*20)
  end

end

function GamesListPanel:HandleGameSelect(opponentAvatar, error)

  if error == 0 then
    if opponentAvatar ~= -1 and opponentAvatar <= # data.commanders then
      settings.avatar2 = opponentAvatar
    else
      settings.avatar2 = "none"
    end
    self.parent.parent.gameInfo:SetOpponentAvatar()
  else
    engine.screen.hud:AdoptWidget(ErrorPopup("pbem.pbem_" .. error))
  end

end


----------------------------------

hud.menu_pbemsetup =
{
  name = "pbemsetup_panel",
  type = "panel",
  position = {0, 0},
  image = "pbem_setup",

  controls =
  {

    {
      name = "selectCampaign_text",
      type = "text",
      position = {178, 80},
      font = "menu_font",
      align = "center",
      color   = {224, 220, 218},
      text = "select_campaign",
    },

    {
      name = "briefing_type",
      type = "text",
      position = {591, 80},
      font = "menu_font",
      align = "center",
      color   = {224, 220, 218},
      text = "briefing_video",
    },

    {
      name = "selectAlliance_text",
      font = "menu_font",
      position = {44, 235},
      align = "left",
      color   = {67, 56, 47},
      text = "select_alliance",
    },
    {
      name = "alliance1_label",
      type = "label",
      font = "menu_font",
      position = {63, 260},
      align = "left",
      normal   = {67, 56, 47},
      hilight  = {255, 255, 255},
      select   = {255, 255, 255},
      disabled = {50, 50, 50},
      shadow = {offset = {0, 0}},
      text = "alliance1",
      sfx = "click",
      Clicked = function(button)
        button.parent:GetImage("alliance_checkon"):SetPosition(44, 260)
        self = button.parent
        self.allianceSelection = 1
      end
    },
    {
      name = "alliance2_label",
      type = "label",
      font = "menu_font",
      position = {63, 285},
      align = "left",
      normal   = {67, 56, 47},
      hilight  = {255, 255, 255},
      select   = {255, 255, 255},
      disabled = {50, 50, 50},
      shadow = {offset = {0, 0}},
      text = "alliance2",
      sfx = "click",
      Clicked = function(button)
        button.parent:GetImage("alliance_checkon"):SetPosition(44, 285)
        self = button.parent
        self.allianceSelection = 2
      end
    },
    {
      name = "alliance_check1",
      type = "button",
      image = "radio",
      position = {44, 260},
      sfx = "click",
      on_click = function(button)
        button.parent:GetImage("alliance_checkon"):SetPosition(44, 260)
        self = button.parent
        self.allianceSelection = 1
      end
    },
    {
      name = "alliance_check2",
      type = "button",
      image = "radio",
      position = {44, 285},
      sfx = "click",
      on_click = function(button)
        button.parent:GetImage("alliance_checkon"):SetPosition(44, 285)
        self = button.parent
        self.allianceSelection = 2
      end
    },
    {
      name = "alliance_checkon",
      type = "image",
      image = "radio_on",
      position = {44, 260},
    },
    {
      name = "private_text",
      type = "text",
      font = "menu_font",
      position = {44, 320},
      align = "left",
      color   = {67, 56, 47},
      text = "private_game",
    },
    {
      name = "info_button",
      type = "button",
      image = "button_default",
      position = {343, 487},
      text = "information",
      sfx = "click",
      Clicked = function(this)
        this.parent.video:Stop(this.parent.playing)
        this.parent.video:Hide()
        this.parent.historical:Show()
        this:Hide()
        this.parent:GetButton("video_button"):Show()
        this.parent:GetText("briefing_type"):SetLocalText("briefing_text")
      end
    },
    {
      name = "video_button",
      type = "button",
      image = "button_default",
      position = {343, 487},
      text = "video",
      sfx = "click",
      Clicked = function(this)
        this.parent.video:Show()
        this.parent.historical:Hide()
        this:Hide()
        this.parent:GetButton("info_button"):Show()
        this.parent:GetText("briefing_type"):SetLocalText("briefing_video")
      end
      },
    {
      name = "cancel_button",
      type = "button",
      image = "button_default",
      position = {592, 490},
      font = "menu_font",
      text = "cancel",
      sfx = "click",
      Clicked = function(this)
        local self = this.parent
        self.parent:SetPanel(pbemPreviousTab)

        self.parent.gameInfo:Show()

        self.video:Stop()
      end
    },
    {
      name = "launch_button",
      type = "button",
      image = "button_default",
      position = {725, 490},
      font = "menu_font",
      text = "confirm",
      sfx = "click",
      Clicked = function(button)
        local self = button.parent
        self.video:Stop()
        -- (scenario, player alliance, difficulty)
        self.parent.waitPanel:Show()
        pbem:IssueChallenge(self.scenSelection,
                            self.allianceSelection,
                            self.password.text,
                            settings.avatar,
                            function(error)
                              CheckError(error,
                                         function()
                                           pbemPreviousTab.list:Refresh(pbemPreviousTab.filter, "hosted")
                                         end,
                                         function()
                                           button:Show()
                                           button.parent:GetButton("cancel_button"):Show()
                                           self.parent.waitPanel:Hide()
                                         end)
                            end)
        button:Hide()
        button.parent:GetButton("cancel_button"):Hide()
      end
    },
  }
}

------------------------------------------------------------------------


class 'PbemGameInfoPanel' (LuaPanel)

function PbemGameInfoPanel:__init()
  LuaPanel.__init(self)

  self:SetPosition(28, 319)

  --self:SetBackgroundImage("pbem_gameinfo")

  self.playerName = self:AddWidget({
    type = "text",
    position = {68, 13},
    align = "center",
    font = "hud16_font",
    color = {247, 245, 241},
  })

  self.playerAvatar = self:AddWidget({
    type = "image",
    position = {2, 30},
    image = "../commanders/commander_0",
  })


  self.opponentName = self:AddWidget({
    type = "text",
    position = {750, 13},
    align = "center",
    font = "hud16_font",
    color = {247, 245, 241},
    text = "opponent"
  })

  self.opponentAvatar = self:AddWidget({
    type = "image",
    position = {684, 30},
    image = "../commanders/commander_none",
  })


  self:AddWidget({
    type = "text",
    position = {414, 13},
    align = "center",
    font = "hud14_font",
    color = {247, 245, 241},
    text = "scenario_description"
  })

  self.description = self:AddWidget({
    type = "memo",
    position = {188, 40},
    size = {448, 115},
    text = "no_game_selected",
    color = {67, 56, 47},
    border = 0
  })

end

function PbemGameInfoPanel:Refresh(opponent, scenario)
  self.playerName:SetText(pbem.playerName)
  if opponent ~= nil then
    self.opponentName:SetText(opponent)
    settings.opponentName = opponent
  end
  if scenario ~= nil then
    self.description:SetLocalText(scenario .. "info")
  else
    self.description:SetLocalText("no_game_selected")
  end
  self.playerAvatar:SetImage("graphics/commanders", "commander_" .. settings.avatar .. ".png")
  self.opponentAvatar:SetImage("graphics/commanders", "commander_none.png")

  self:Show()
end

function PbemGameInfoPanel:SetOpponentAvatar()
  self.opponentAvatar:SetImage("graphics/commanders", "commander_" .. settings.avatar2 .. ".png")
end


------------------------------------------------------------------------


class 'PBEMGameSetupPanel' (LuaPanel)

function PBEMGameSetupPanel:__init()
  LuaPanel.__init(self, hud.menu_pbemsetup)
  self:Hide()

  -- Select first scenario by default
  self.scenSelection = campaign[1].script
  self.allianceSelection = 1
  self.difficultySelection = 2

  -- Load Scenarios list
  self:AdoptWidget(ScenarioSelectionPanel(40, 100))

  self.video = self:AdoptWidget(VideoPanel(351, 107,  0.75, 0.75))
  self.video:SetVideo(campaign[1])

  self.historical = self:AdoptWidget(HistoricalBriefingPanel(351, 107))

  self:GetButton("video_button"):Hide()

  self.password = self:AddWidget({
    type = "edit",
    position = {44, 345},
    size = {width = 200, height = 20},
    color = {67, 56, 47},
    sfx = "click",
    tooltip = "private_game_info"
  })

end

function PBEMGameSetupPanel:OnShow()

  self.parent.gameInfo:Hide()

  self:GetButton("launch_button"):Show()

  self:GetButton("cancel_button"):Show()

  self.password:SetText("")

end

function PBEMGameSetupPanel:DrawListElement(panelID, id, x, y, labelText, localize)

  local listElement =
  {
    name = "list_label_" .. panelID .. "_" .. id,
    type = "label",
    font = "menu_font",
    position = {x, y},
    align = "left",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "aircraft_carrier",
    sfx = "click",
    Clicked = function(button)
      self.scenSelection = id

      --self:GetMemo("info_memo"):SetLocalText(labelText .. "info")
      self:GetMemo("info_memo"):SetText(campaign[self.scenSelection+1].script .. " " .. self.allianceSelection)
    end
  }

  self:AddWidget(listElement):Show()
  if localize == 1 then
    self:GetText("list_label_" .. panelID .. "_" .. id):SetLocalText(labelText)
  else
    self:GetText("list_label_" .. panelID .. "_" .. id):SetText(labelText)
  end
end

function GetCampaignName(textid)
  local i = 1
  while campaign[i] ~= nil do
    if campaign[i].script == textid then
      return campaign[i].text
    end
    i = i + 1
  end
  return "scenario_not_found"
end

function CheckError(error, noErrorEffect, errorEffect)
  if error == 0 then
    if noErrorEffect ~= nil then
      noErrorEffect()
    end
    return true
  else
    engine.screen.hud:AdoptWidget(ErrorPopup("pbem.pbem_" .. pbem.error))
    if errorEffect ~= nil then
      errorEffect()
    end
    return false
  end
end

------------------------------------------------------------------------


class 'PasswordPanel' (LuaPanel)

function PasswordPanel:__init()
  LuaPanel.__init(self)

  --self:SetPosition((engine.window.width)/2-240, (engine.window.height)/2-88)
  self:SetBackgroundImage("PbemDialog")
  self:SetPosition((874-self.width)/2, (548-self.height)/2)

  self:AddWidget({
    type = "text",
    font = "impact18_font",
    position = {312, 38},
    color   = {67, 56, 47},
    shadow = {offset = {0, 0}},
    text = "private_game_title",
    align = "center"
  })
  self:AddWidget({
    type = "image",
    image = "password_icon",
    position = {19, 29},
  })
  self.text = self:AddWidget({
    type = "memo",
    position = {166, 55},
    font = "times12_font",
    color   = {67, 56, 47},
    size = {300, 50},
    border = 0,
    text = "enter_password",
  })
  self.password = self:AddWidget({
    type = "edit",
    position = {166, 100},
    size = {width = 200, height = 20},
    color = {67, 56, 47},
    sfx = "click",
  })
  self:AddWidget({
    name = "cancel_button",
    type = "button",
    image = "button_default",
    position = {22, 159},
    font = "hud14_font",
    text = "cancel",
    sfx = "click",
    on_click = function(button)
      button.parent:Hide()
    end
  })
  self:AddWidget({
    name = "continue_button",
    type = "button",
    image = "button_default",
    position = {359, 159},
    font = "hud14_font",
    text = "continue",
    sfx = "click",
    on_click = function(button)
      self.parent.waitPanel:Show()
      assert(self.selectedGame ~= nil, "erroref")
      pbem:AcceptChallenge(self.selectedGame,
                           self.password.text,
                           function(error)
                             CheckError(error,
                                        function()
                                          self.parent:HandleAcceptedGame()
                                        end,
                                        function()
                                          self.parent.waitPanel:Hide()
                                        end)
                           end)
      button.parent:Hide()
    end
  })

  self:Hide()
end

function PasswordPanel:Popup(selectedGame)
  self.selectedGame = selectedGame

  self.password:SetText("")

  self:Show()
end


------------------------------------------------------------------------


class 'AcceptedPanel' (LuaPanel)

function AcceptedPanel:__init()
  LuaPanel.__init(self)
  --self:SetPosition((engine.window.width)/2-243, (engine.window.height)/2-104)
  self:SetBackgroundImage("PbemDialog")

  self:SetPosition((874-self.width)/2, (548-self.height)/2)

  self:AddWidget({
    type = "text",
    font = "impact18_font",
    position = {312, 38},
    color   = {67, 56, 47},
    shadow = {offset = {0, 0}},
    text = "game_joined",
    align = "center"
  })
  self:AddWidget({
    type = "image",
    image = "pbemAccepted",
    position = {19, 29},
  })
  self.text = self:AddWidget({
    type = "memo",
    position = {166, 55},
    font = "times12_font",
    color   = {67, 56, 47},
    size = {300, 50},
    border = 0,
  })

  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {22, 159},
    font = "hud14_font",
    text = "close",
    sfx = "click",
    on_click = function(button)
      button.parent:Hide()
    end
  })
  self.playButton = self:AddWidget({
    type = "button",
    image = "button_default",
    position = {359, 159},
    font = "hud14_font",
    text = "play_challenge",
    tooltip = "play_challenge_tip",
    sfx = "click",
    on_click = function(button)
      self.parent.autoRefreshOn = false
      if not frontEnd.refreshing then
        frontEnd.selectedMPGame = self.selectedGame
        engine:SetScreen(screens.loadingMenu)
        engine:CreatePbemGame(self.selectedGame, PbemGameCreateErrorHandler)
        self.parent:CancelAutoRefresh()
      else
        frontEnd.gameLaunched = self.selectedGame
      end
      button.parent:Hide()
    end
  })

  self:Hide()
end

function AcceptedPanel:Popup(selectedGame, hostSide, scenarioName)

  self.selectedGame = selectedGame

  -- get scenario starting side
  local startingFaction = 1
  for _, scenario in pairs(campaign) do
    if scenario.text == scenarioName then
      startingFaction = scenario.startingFaction
      break
    end
  end

  -- determine starting side
  if hostSide == startingFaction then
    self.playButton:Hide()
    self.text:SetLocalText("game_opponent_turn")
  else
    self.playButton:Show()
    self.text:SetLocalText("game_your_turn")
  end

  self:Show()
end

function AcceptedPanel:OnHide()
  frontEnd.accepting = 0
end

------------------------------------------------------------------------


class 'WaitingPanel' (LuaPanel)

function WaitingPanel:__init()
  LuaPanel.__init(self)

  self:SetBackgroundImage("login_bg")

  self:SetPosition((874-self.width)/2, (548-self.height)/2)

  self:AddWidget({
    type = "image",
    position = {62, 54},
    image = "login_box",
  })

  self.descr = self:AddWidget({
    type = "text",
    text = "transferring_data",
    align = "center",
    position = {132, 238},
    color = {226, 224, 223},
  })

  self.video = self:AddWidget({type = "animation"});
  self.video:SetScaling(1, 1)
  self.video:SetPosition(100, 85)

  self:Hide()
end

function WaitingPanel:OnShow()
  frontEnd.disableInput = true
  self.video:PlayFile("ai_gearwheel");
end

function WaitingPanel:OnHide()
  frontEnd.disableInput = false
end

