class 'VideoPanel' (LuaPanel)

function VideoPanel:__init(x, y, scaleX, scaleY)
  LuaPanel.__init(self)
  self.subtitleTasks = {}

  self.subtitleId = 0
  self.video = self:AddWidget({
    type = "video"
  })
  self.video:SetScaling(scaleX, scaleY)
  self.video:SetPosition(x, y)


  self.subtitlesShadowImage = self:AddWidget({
    type = "image",
    image = "subtitles_shadow",
    position = {x, y + 289},
  })

  self.subtitlesShadow = self:AddWidget({
    type = "memo",
    font = "times18_font",
    position = {x + 20 + 1, y + 300 + 1},
    align = "center",
    color   = {0, 0, 0},
    size = {445, 50},
    border = 0,
  })
  self.subtitles = self:AddWidget({
    type = "memo",
    font = "times18_font",
    position = {x + 20, y + 300},
    align = "center",
    color   = {255, 255, 255},
    size = {445, 50},
    border = 0,
  })

  self.playButton = self:AddWidget({
    type = "button",
    image = "video_play",
    position = {x, y},
    sfx = "click",
    on_click = function(button)
      button.parent:Play(self.videoName)
    end
  })

  self.playing = 1

end

function VideoPanel:SetVideo(scenario)

  self:Stop(self.playing)

  self.videoName = scenario.video.name

  --FIXME: need ability to read language setting
  self.timing = scenario.video.timing.english

  self.duration = scenario.video.duration

end

function VideoPanel:Play(videoName)

  if videoName == nil then
    videoName = self.videoName
  end

  self.playing = self.playing + 1

  self.video:Show()

  self.playButton:Hide()

  self.subtitleId = 0

  self.video:PlayFile(videoName)

-- Stop video after playing once
  local timeInMiliseconds = self.duration * 1000
  local serial = self.playing
    engine:ScheduleTask(timeInMiliseconds,
      function()
        self:Stop(serial)
      end
    )

  self:DisplaySubtitle(0, self.playing)

end

function VideoPanel:Stop(serial)

  self:ClearSubtitleTasks()

  self.playButton:Show()

  self.video:Stop()

  self.subtitles:Hide()
  self.subtitlesShadow:Hide()
  self.subtitlesShadowImage:Hide()

  StopAllSounds()

end

function VideoPanel:ScheduleSubtitle(time, slot, serial)
  table.insert(self.subtitleTasks, engine:ScheduleTask(time,
    function()
      self:DisplaySubtitle(slot, serial)
    end))
end

function VideoPanel:ClearSubtitleTasks()
  for _, task in ipairs(self.subtitleTasks) do
    engine:CancellTask(task)
  end
  self.subtitleTasks = {}
end

function VideoPanel:DisplaySubtitle(slot, serial)

  if slot == 0 then
    timeSetNext = self.timing[slot+1]
    local timeInMiliseconds = (timeSetNext[1]*60 + timeSetNext[2]) * 1000
    self:ScheduleSubtitle(timeInMiliseconds, slot+1, serial)
    return
  end

  if serial ~= self.playing then
    return
  end

  if slot == nil then
    self.subtitles:Hide()
    self.subtitlesShadow:Hide()
    self.subtitlesShadowImage:Hide()
    return
  end

  local timeSet = self.timing[slot]

  local text = timeSet[4]
  local audio = timeSet[5]

  if text ~= nil then
    self.subtitles:Show()
    self.subtitles:SetLocalText(text)
    self.subtitlesShadow:Show()
    self.subtitlesShadow:SetLocalText(text)
    self.subtitlesShadowImage:Show()

    -- Check if text has expiration timer
    if timeSet[3] ~= nil and timeSet[3] ~= 0 then
      local timeInMiliseconds = timeSet[3] * 1000
      self:ScheduleSubtitle(timeInMiliseconds, nil, serial)
    end
  end

  -- FIXME: load all at game start so its not reloaded every time video is replayed
  if audio ~= nil then
    local voice = LoadSample("../voice/" .. audio .. ".wav")
    PlaySample(voice)
  end

  -- Schedule next (not for functions that have no text)
  if text ~= nil and self.timing[slot+1] ~= nil then
    timeSetNext = self.timing[slot+1]
    local timeInMiliseconds = ((timeSetNext[1]-timeSet[1])*60 + (timeSetNext[2]-timeSet[2])) * 1000
    self:ScheduleSubtitle(timeInMiliseconds, slot+1, serial)
  end

end
