------------------------------------------------

class 'UnitInfoPanel' (LuaPanel)

function UnitInfoPanel:__init(faction, x, y)
  LuaPanel.__init(self)

  --self.unit = unit
  self.faction = faction

  self:SetPosition(x, y)

  self.tabImage = self:AddWidget({
    type = "image",
    position = {0, 0},
    image = "infotab1"
  })

  self:AddWidget({
    type = "label",
    position = {61, 15},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "statistics",
    sfx = "click",
    on_click = function(button)
      self.panelStats:Show()
      self.panelDescr:Hide()
    end
  })

  self:AddWidget({
    type = "label",
    position = {192, 15},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "information",
    sfx = "click",
    on_click = function(button)
      self.panelStats:Hide()
      self.panelDescr:Show()
    end
  })

  self.panelStats = self:AdoptWidget(UnitStatisticsPanel())
  self.panelDescr = self:AdoptWidget(UnitDescriptionPanel())
end

function UnitInfoPanel:OnShow()
end

function UnitInfoPanel:Refresh(unit, real)
  self.panelStats:Refresh(unit, real)
  self.panelDescr:Refresh(unit)
end

------------------------------------------------

class 'UnitStatisticsPanel' (LuaPanel)

function UnitStatisticsPanel:__init()
  LuaPanel.__init(self, unitStatsTable)

  self.stats = {}
  self.tooltips = {}
  self.statName = {}

  local stat_names =
  {
    "basedefense",
    "highdefense",
    "quality",
    "efficiency",
    "range",
    "groundattack",
    "airattack",
    "navalattack",
    "armourattack",
    "mp",
    "shock",
    "assault",
    "ammo",
    "stratattack",
    "los"
  }

  for i=0, 14 do
    self.statName[i] = stat_names[i+1]
    self.stats[i] = self:AddWidget({
      name = "stat_" .. i .. "_value",
      type = "text",
      font = "unit_font",
      position = {32 + 46 * (i % 5), 14 + 34 * math.floor(i / 5)},
      color   = {31, 21, 6},
      align = "center"
    })
    self.tooltips[i] = self:AddWidget({
      name = "stat_" .. i .. "_tooltip",
      type = "image",
      position = {46 * (i % 5), 34 * math.floor(i / 5)},
      image = "stat_" .. stat_names[i+1],
      tooltip = "stat_" .. stat_names[i+1],
      on_click = function()
        ui.advicePanel:AddAdvice(self.statName[i])
      end
    })
  end

end

function UnitStatisticsPanel:OnShow()
  self.parent.tabImage:SetImage("graphics/hud", "infotab1.png")
end

function UnitStatisticsPanel:Refresh(proto, real)

  local faction
  local panelType
  if proto.prototype == nil then
    unit = {}
    unit.prototype = proto
    level = 1
    panelType = "prodPanel"
    faction = ui.productionFaction
  else
    unit = proto
    level = 1
    panelType = "unitInfo"
    faction = unit.faction
  end

  local commander = GetNearbyCommander(unit)
  local stat

  stat = GetProductionStat(faction, unit, "basedefense")
  if stat ~= nil then
    self.stats[0]:SetNumber(stat + GetCommanderBonus(commander, "basedefense"))
  else
    self.stats[0]:SetText("/")
  end

  stat = GetProductionStat(faction, unit, "highdefense")
  if stat ~= nil then
    self.stats[1]:SetNumber(stat + GetCommanderBonus(commander, "highdefense"))
  else
    self.stats[1]:SetText("/")
  end

  local statsSet = "assault"
  stat = GetProductionStat(faction, unit, "assault")
  if stat ~= nil then
    self.statName[8] = "assault"
    self.stats[11]:SetNumber(stat + GetCommanderBonus(commander, "assault"))
    self.tooltips[11]:SetImage("graphics/hud", "stat_assault.png")
    self.tooltips[11].tooltip = "stat_assault"
  else
    stat = GetProductionStat(faction, unit, "bombard")
    if stat ~= nil then
      statsSet = "bombard"
      self.statName[8] = "bombard"
      self.stats[11]:SetNumber(stat + GetCommanderBonus(commander, "bombard"))
      self.tooltips[11]:SetImage("graphics/hud", "stat_bombard.png")
      self.tooltips[11].tooltip = "stat_bombard"
    else
      self.stats[11]:SetText("/")
    end
  end

  stat = GetProductionStat(faction, unit, "groundattack")
  if stat ~= nil then
    self.stats[5]:SetNumber(stat + GetCommanderBonus(commander, "groundattack"))
  else
    self.stats[5]:SetText("/")
  end
  stat = GetProductionStat(faction, unit, "airattack")
  if stat ~= nil then
    self.stats[6]:SetNumber(stat + GetCommanderBonus(commander, "airattack"))
  else
    self.stats[6]:SetText("/")
  end
  stat = GetProductionStat(faction, unit, "navalattack")
  if stat ~= nil then
    self.stats[7]:SetNumber(stat + GetCommanderBonus(commander, "navalattack"))
  else
    self.stats[7]:SetText("/")
  end
  --panelID:GetText("subattack_text"):SetNumber(GetStatSubAttack(unit))
  if unit.prototype.lua.type ~= "naval" and GetProductionStat(faction, unit, "subattack") == nil then
    if unit.prototype.name == "artillery" then
      stat = GetProductionStat(faction, unit, "counterattack")
      self.statName[8] = "counterattack"
      self.tooltips[8]:SetImage("graphics/hud", "stat_counterattack.png")
      if stat ~= nil then
        self.stats[8]:SetNumber(stat + GetCommanderBonus(commander, "counterattack"))
        self.tooltips[8].tooltip = "stat_counterattack"
      else
        self.stats[8]:SetText("/")
      end
    else
      stat = GetProductionStat(faction, unit, "armourattack")
      self.statName[8] = "armourattack"
      self.tooltips[8]:SetImage("graphics/hud", "stat_armourattack.png")
      if stat ~= nil then
        self.stats[8]:SetNumber(stat + GetCommanderBonus(commander, "armourattack"))
        self.tooltips[8].tooltip = "stat_armourattack"
      else
        self.stats[8]:SetText("/")
      end
    end
  else
    stat = GetProductionStat(faction, unit, "subattack")
    self.statName[8] = "subattack"
    self.tooltips[8]:SetImage("graphics/hud", "stat_subattack.png")
    if stat ~= nil then
      self.stats[8]:SetNumber(stat + GetCommanderBonus(commander, "subattack"))
      self.tooltips[8].tooltip = "stat_subattack"
    else
      self.stats[8]:SetText("/")
    end
  end
  stat = GetProductionStat(faction, unit, "stratattack")
  if stat ~= nil then
    self.stats[13]:SetNumber(stat + GetCommanderBonus(commander, "stratattack"))
    self.tooltips[13]:SetImage("graphics/hud", "stat_stratattack.png")
    self.tooltips[13].tooltip = "stat_stratattack"
  else
    self.stats[13]:SetText("/")
  end

  stat = GetProductionStat(faction, unit, "shock")
  if stat ~= nil then
    self.stats[10]:SetNumber(stat + GetCommanderBonus(commander, "shock"))
  else
    self.stats[10]:SetText("/")
  end

  self.stats[4]:SetNumber(unit.prototype:GetLevelStats(1).range)
  self.stats[9]:SetNumber(unit.prototype:GetLevelStats(1).mp)
  self.stats[14]:SetNumber(unit.prototype:GetLevelStats(1).los)
  self.stats[12]:SetNumber(unit.prototype:GetLevelStats(1).ammunition)
  if panelType == "unitInfo" then
    self.stats[2]:SetNumber(math.floor(unit.luaData.quality/10))
    self.stats[3]:SetNumber(math.floor(unit.luaData.quality/10))
  else
    self.stats[2]:SetNumber(GetMPQuality(faction)/10)
    self.stats[3]:SetNumber(GetMPQuality(faction)/10)
  end

  -- Chassis type
  self.tooltips[9]:SetImage("graphics/hud", "stat_chassis_" .. unit.prototype.lua.chassis .. ".png")

  -- Cost
  self:GetText("pp_cost_text"):SetNumber(unit.prototype.costPP)
  self:GetText("mp_cost_text"):SetNumber(unit.prototype.costMP)
  if self.parent.faction ~= -1 then
    if self.parent.faction.pp < unit.prototype.costPP then
      self:GetText("pp_cost_text"):SetColour(254, 83, 30, 255)
    else
      self:GetText("pp_cost_text"):SetColour(241, 237, 233, 255)
    end
    if self.parent.faction.mp < unit.prototype.costMP then
      self:GetText("mp_cost_text"):SetColour(254, 83, 30, 255)
    else
      self:GetText("mp_cost_text"):SetColour(241, 237, 233, 255)
    end
  end
  self:GetText("pp_upkeep_text"):SetNumber(unit.prototype.upkeepPP)
  if self.parent.faction ~= -1 then
    if GetIncomePP(self.parent.faction) < GetUpkeep(self.parent.faction) + unit.prototype.upkeepPP then
      self:GetText("pp_upkeep_text"):SetColour(254, 83, 30, 255)
    else
      self:GetText("pp_upkeep_text"):SetColour(241, 237, 233, 255)
    end
  end
  self:GetText("turns_text"):SetNumber(unit.prototype.turns)

end

unitStatsTable =
{
  name = "unitstats_panel",
  type = "panel",
  position = {14, 43},
  image = "unit_statistics",

  controls =
  {
    {
      name = "pp_cost",
      type = "image",
      position = {0, 109},
      image = "resource_pp_white",
      tooltip = "costPP"
    },
    {
      name = "pp_cost_text",
      type = "text",
      font = "hud14_font",
      position = {32, 110},
      align = "left",
      color   = {241, 237, 233},
      text = "pp"
    },
    {
      name = "mp_cost",
      type = "image",
      position = {63, 109},
      image = "resource_mp_white",
      tooltip = "costMP"
    },
    {
      name = "mp_cost_text",
      type = "text",
      font = "hud14_font",
      position = {95, 110},
      align = "left",
      color   = {241, 237, 233},
      text = "pp"
    },
    {
      name = "pp_upkeep",
      type = "image",
      position = {123, 109},
      image = "icon_upkeep_white",
      tooltip = "upkeepPP"
    },
    {
      name = "pp_upkeep_text",
      type = "text",
      font = "hud14_font",
      position = {151, 110},
      align = "left",
      color   = {241, 237, 233},
      text = "pp"
    },
    {
      name = "icon_turns",
      type = "image",
      position = {186, 109},
      image = "turn_icon_white",
      tooltip = "turns_to_complete"
    },
    {
      name = "turns_text",
      type = "text",
      font = "hud14_font",
      position = {214, 110},
      align = "left",
      color   = {241, 237, 233},
      text = "0"
    },
  }
}

------------------------------------------------

class 'UnitDescriptionPanel' (LuaPanel)

function UnitDescriptionPanel:__init()
  LuaPanel.__init(self)

  self.content = self:AddWidget({
    type = "memo",
    position = {12, 42},
    size = {220, 120},
    font = "times12_font",
    color   = {224, 220, 218},
    text = "normal",
    border = 0,
    OnScroll = function(button, delta)
      if delta == 1 then
        if button.parent.content.startingLine > 0 then
          button.parent.content.startingLine = button.parent.content.startingLine - 1
          button.parent.pagebar:SetPosition(235, 63 + (80 * button.parent.content.startingLine / self.pageCount) )
        end
      else
        if button.parent.content.startingLine < button.parent.content.numLines - 8 then
          button.parent.content.startingLine = button.parent.content.startingLine + 1
          button.parent.pagebar:SetPosition(235, 63 + (80 * button.parent.content.startingLine / self.pageCount) )
        end
      end
    end
  })

  self:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {230, 43},
    sfx = "click",
    on_click = function(button)
      if button.parent.content.startingLine > 0 then
        button.parent.content.startingLine = button.parent.content.startingLine - 1
        button.parent.pagebar:SetPosition(235, 63 + (80 * button.parent.content.startingLine / self.pageCount) )
      end
    end
  })
  self:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {230, 148},
    sfx = "click",
    on_click = function(button)
      if button.parent.content.startingLine < button.parent.content.numLines - 8 then
        button.parent.content.startingLine = button.parent.content.startingLine + 1
        button.parent.pagebar:SetPosition(235, 63 + (80 * button.parent.content.startingLine / self.pageCount) )
      end
    end
  })

	self.pagebar = self:AddWidget({
		type = "image",
		image = "pagebar_dark",
		position = {235, 63},
	})

  self:Hide()

end

function UnitDescriptionPanel:OnShow()
  self.parent.tabImage:SetImage("graphics/hud", "infotab2.png")
end

function UnitDescriptionPanel:Refresh(unit)
  local name = unit.name
  if name == nil or name == "" then
    name = unit.prototype.name
  end
  self.content:SetLocalText(name .. "_info")
  self.content.maxLines = 8
  self.pageCount = math.max(8, self.content.numLines)
  self.pagebar:SetSource(0, 0, 6, 80 * 8 / self.pageCount)
end

------------------------------------------------



class 'CommanderInfoPanel' (LuaPanel)

function CommanderInfoPanel:__init(x, y)
  LuaPanel.__init(self)

  self:SetPosition(x, y)

  self.tabImage = self:AddWidget({
    type = "image",
    position = {0, 0},
    image = "infotab1"
  })

  self:AddWidget({
    type = "label",
    position = {61, 15},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "statistics",
    sfx = "click",
    on_click = function(button)
      self.panelStats:Show()
      self.panelDescr:Hide()
    end
  })

  self:AddWidget({
    type = "label",
    position = {192, 15},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "information",
    sfx = "click",
    on_click = function(button)
      self.panelStats:Hide()
      self.panelDescr:Show()
    end
  })

  self.panelStats = self:AdoptWidget(CommanderStatisticsPanel())
  self.panelDescr = self:AdoptWidget(CommanderDescriptionPanel())
end

function CommanderInfoPanel:Refresh(commander)
  self.panelStats:Refresh(commander)
  self.panelDescr:Refresh(commander)
end



------------------------------------------------

class 'CommanderStatisticsPanel' (LuaPanel)

function CommanderStatisticsPanel:__init()
  LuaPanel.__init(self)

  self.commanderCommands = self:AddWidget({
    type = "text",
    position = {25, 47},
    color = {241, 237, 233},
    font = "hud12_font",
    text = "unit_types",
    align = "left"
  })
  self.commanderEffects = self:AddWidget({
    type = "text",
    position = {25, 94},
    color = {241, 237, 233},
    font = "hud12_font",
    text = "combat_effects",
    align = "left"
  })

  -- Stat icons
  for i=0, 3 do
    self:AddWidget({
      name = "stat_".. i,
      type = "image",
      image = "stat-empty",
      position = {30 + i*55, 112},
    })
  end


  self:AddWidget({
    name = "stat_number_0",
    type = "text",
    position = {61, 125},
    font = "unit_font",
    align = "center",
    color = {31, 21, 6},
    text = "10"
  })
  self:AddWidget({
    name = "stat_number_1",
    type = "text",
    position = {116, 125},
    font = "unit_font",
    align = "center",
    color = {31, 21, 6},
    text = "10"
  })
  self:AddWidget({
    name = "stat_number_2",
    type = "text",
    position = {171, 125},
    font = "unit_font",
    align = "center",
    color = {31, 21, 6},
    text = "10"
  })
  self:AddWidget({
    name = "stat_number_3",
    type = "text",
    position = {226, 125},
    font = "unit_font",
    align = "center",
    color = {31, 21, 6},
    text = "10"
  })

  -- Stat tooltips
  for i=0, 3 do
    self:AddWidget({
      name = "stat_tooltip_".. i,
      type = "image",
      image = "stat-empty",
      position = {30 + i*55, 112},
    })
  end

  -- Add unit icons
  self.groundUnits = self:AddWidget({
    type = "panel"
  })
  self.groundUnits:AddWidget({
    type = "image",
    image = "../units/garrison_" .. playerAlliance.id .. "_25",
    position = {30, 65},
    tooltip = "garrison"
  })
  self.groundUnits:AddWidget({
    type = "image",
    image = "../units/infantry_" .. playerAlliance.id .. "_25",
    position = {50, 65},
    tooltip = "infantry"
  })
  self.groundUnits:AddWidget({
    type = "image",
    image = "../units/cavalry_" .. playerAlliance.id .. "_25",
    position = {70, 65},
    tooltip = "cavalry"
  })
  self.groundUnits:AddWidget({
    type = "image",
    image = "../units/armour_" .. playerAlliance.id .. "_25",
    position = {90, 65},
    tooltip = "armour"
  })
  self.groundUnits:AddWidget({
    type = "image",
    image = "../units/armouredcar_" .. playerAlliance.id .. "_25",
    position = {110, 65},
    tooltip = "armouredcar"
  })

  self.airUnits = self:AddWidget({
    type = "panel"
  })
  self.airUnits:AddWidget({
    type = "image",
    image = "../units/fighter_" .. playerAlliance.id .. "_25",
    position = {30, 65},
    tooltip = "fighter"
  })
  self.airUnits:AddWidget({
    type = "image",
    image = "../units/bomber_" .. playerAlliance.id .. "_25",
    position = {50, 65},
    tooltip = "bomber"
  })
  self.airUnits:AddWidget({
    type = "image",
    image = "../units/zeppelin_" .. playerAlliance.id .. "_25",
    position = {70, 65},
    tooltip = "zeppelin"
  })

  self.navalUnits = self:AddWidget({
    type = "panel"
  })
  self.navalUnits:AddWidget({
    type = "image",
    image = "../units/lightcruiser_" .. playerAlliance.id .. "_25",
    position = {30, 65},
    tooltip = "lightcruiser"
  })
  self.navalUnits:AddWidget({
    type = "image",
    image = "../units/dreadnought_" .. playerAlliance.id .. "_25",
    position = {50, 65},
    tooltip = "dreadnought"
  })
  self.navalUnits:AddWidget({
    type = "image",
    image = "../units/submarine_" .. playerAlliance.id .. "_25",
    position = {70, 65},
    tooltip = "submarine"
  })


end

function CommanderStatisticsPanel:OnShow()
  self.parent.tabImage:SetImage("graphics/hud", "infotab1.png")
end

function CommanderStatisticsPanel:Refresh(commander)

  if commander == -1 then

    self.groundUnits:Hide()
    self.airUnits:Hide()
    self.navalUnits:Hide()

    self.commanderCommands:Hide()
    self.commanderEffects:Hide()

    for i = 0, 3 do
      self:GetImage("stat_" .. i):Hide()
      self:GetImage("stat_tooltip_" .. i):Hide()
      self:GetText("stat_number_" .. i):Hide()
    end

  else

    self.commanderCommands:Show()
    self.commanderEffects:Show()

    for i = 0, 3 do
      self:GetText("stat_number_" .. i):Show()
    end

    if data.commanders[commander+1].type == "land" then
      self.groundUnits:Show()
      self.airUnits:Hide()
      self.navalUnits:Hide()
    end
    if data.commanders[commander+1].type == "air" then
      self.groundUnits:Hide()
      self.airUnits:Show()
      self.navalUnits:Hide()
    end
    if data.commanders[commander+1].type == "naval" then
      self.groundUnits:Hide()
      self.airUnits:Hide()
      self.navalUnits:Show()
    end

    local stat_names =
    {
      "basedefense",
      "highdefense",
      --"quality",
      "groundattack",
      "airattack",
      "navalattack",
      "armourattack",
      "shock",
      "assault",
      "stratattack",
    }

    slot = 0
    for _, statName in pairs(stat_names) do
      local stat = GetCommanderBonus(data.commanders[commander+1], statName)
      if stat ~= 0 then
        --self:GetImage("stat_" .. slot):SetPosition(30 + slot*55, 112)
        self:GetImage("stat_" .. slot):Show()
        self:GetImage("stat_tooltip_" .. slot):Show()
        self:GetImage("stat_tooltip_" .. slot):SetImage("graphics/hud", "stat_" .. statName .. ".png")
        self:GetImage("stat_tooltip_" .. slot).tooltip = "stat_" .. statName
        self:GetText("stat_number_" .. slot):SetNumber(stat)
        slot = slot + 1
      end
      if slot > 3 then
        break
      end
    end

    -- hide unused stats
    if slot < 4 then
      for i = slot, 3 do
        self:GetImage("stat_" .. i):Hide()
        self:GetImage("stat_tooltip_" .. i):Hide()
        self:GetText("stat_number_" .. i):Hide()
      end
    end
  end

end



------------------------------------------------

class 'CommanderDescriptionPanel' (LuaPanel)

function CommanderDescriptionPanel:__init()
  LuaPanel.__init(self)

  self.content = self:AddWidget({
    type = "memo",
    position = {12, 42},
    size = {220, 120},
    font = "times12_font",
    color   = {224, 220, 218},
    text = "normal",
    border = 0,
    OnScroll = function(button, delta)
      if delta == 1 then
        if button.parent.content.startingLine > 0 then
          button.parent.content.startingLine = button.parent.content.startingLine - 1
          button.parent.pagebar:SetPosition(235, 53 + (80 * button.parent.content.startingLine / self.pageCount) )
        end
      else
        if button.parent.content.startingLine < button.parent.content.numLines - 8 then
          button.parent.content.startingLine = button.parent.content.startingLine + 1
          button.parent.pagebar:SetPosition(235, 53 + (80 * button.parent.content.startingLine / self.pageCount) )
        end
      end
    end
  })

  self:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {230, 43},
    sfx = "click",
    on_click = function(button)
      if button.parent.content.startingLine > 0 then
        button.parent.content.startingLine = button.parent.content.startingLine - 1
        button.parent.pagebar:SetPosition(235, 53 + (80 * button.parent.content.startingLine / self.pageCount) )
      end
    end
  })
  self:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {230, 148},
    sfx = "click",
    on_click = function(button)
      if button.parent.content.startingLine < button.parent.content.numLines - 8 then
        button.parent.content.startingLine = button.parent.content.startingLine + 1
        button.parent.pagebar:SetPosition(235, 53 + (80 * button.parent.content.startingLine / self.pageCount) )
      end
    end
  })

	self.pagebar = self:AddWidget({
		type = "image",
		image = "pagebar_dark",
		position = {235, 63},
	})

  self:Hide()

end

function CommanderDescriptionPanel:OnShow()
  self.parent.tabImage:SetImage("graphics/hud", "infotab2.png")
end

function CommanderDescriptionPanel:Refresh(commander)
  if commander == -1 then
    self.content:SetLocalText("commander_none_info")
  else
    self.content:SetLocalText("commander_" .. commander .. "_info")
  end
  self.content.maxLines = 8
  self.pageCount = math.max(8, self.content.numLines)
  self.pagebar:SetSource(0, 0, 6, 80 * 8 / self.pageCount)
end

------------------------------------------------
