class 'ManagementPanel' (LuaPanel)

function ManagementPanel:__init(alliance)
  LuaPanel.__init(self, management_table)

  self.alliance = alliance

  --self:AddWidget(productionSelect)

  self.factionPanels = {}

  -- Create faction flag buttons
  local slot = 1
  for faction in alliance.factions do
    -- only for factions with capitals
    if (faction.luaData.capitals > 0 or IsNewFaction(faction) == true) then
      -- Select the first faction by default
      if slot == 1 then
        ui.managementFaction = faction
        self.previousFaction = ui.managementFaction
      end
      self:AddFaction(faction, slot)
      slot = slot + 1
    end
  end

  -- Fincances titles
  self:AddWidget({
    type = "text",
    color = {241, 237, 233},
    position = {350, 21},
    text = "finance_title"
  })
  self:AddWidget({
    type = "image",
    position = {397, 50},
    image = "finances_manpower",
    tooltip = "finance_manpower"
  })
  self:AddWidget({
    type = "image",
    position = {461, 50},
    image = "resource_pp",
    tooltip = "finance_current"
  })
  self:AddWidget({
    type = "image",
    position = {525, 50},
    image = "finances_income",
    tooltip = "finance_production"
  })
  self:AddWidget({
    type = "image",
    position = {589, 49},
    image = "finances_upkeep",
    tooltip = "finance_upkeep"
  })
  self:AddWidget({
    type = "image",
    position = {653, 49},
    image = "finances_labs",
    tooltip = "finance_labs"
  })
  self:AddWidget({
    type = "image",
    position = {717, 48},
    image = "finances_balance",
    tooltip = "finance_balance"
  })
  self:AddWidget({
    type = "image",
    position = {781, 50},
    image = "finances_nextturn",
    tooltip = "finance_next"
  })

  self.ammoPanel = self:AdoptWidget(AmmunitionPanel())
  self.railPanel = self:AdoptWidget(RailmovesPanel())
  self.TransportmovesPanel = self:AdoptWidget(TransportmovesPanel())
  self.currentPanel = self.ammoPanel
  self.currentPanel:Show()

  -- Create faction flag buttons
  self.financesRow = {}
  for i = 0, 9 do
    self.financesRow[i] = self:AdoptWidget(FinancesStatsRow(i))
  end

  self.conditions = self:AdoptWidget(VictoryConditionsPanel(playerAlliance))

  self:AddWidget({
    type = "text",
    color = {241, 237, 233},
    position = {350, 221},
    text = "management_options_title"
  })

  -- Ammunition
  self:AddWidget({
    type = "image",
    image = "management_ammo",
    position = {342, 241},
  })
  self.am = self:AddWidget({
    type = "text",
    color = {69, 40, 23},
    position = {413, 246},
    align = "right",
  })
  self:AddWidget({
    type = "image",
    image = "management_button_hotspot",
    position = {342, 241},
    tooltip = "ammunition_tooltip",
    sfx = "click",
    on_click = function()
      self.currentPanel:Hide()
      self.currentPanel = self.ammoPanel
      self.currentPanel:Show()
    end
  })

  -- Railroad
  self:AddWidget({
    type = "image",
    image = "management_rc",
    position = {428, 241},
  })
  self.rc = self:AddWidget({
    type = "text",
    color = {69, 40, 23},
    position = {499, 246},
    align = "right",
    --font = "times20_font",
    text = "0"
  })
  self:AddWidget({
    type = "image",
    image = "management_button_hotspot",
    position = {428, 241},
    tooltip = "railmoves_tooltip",
    sfx = "click",
    on_click = function()
      self.currentPanel:Hide()
      self.currentPanel = self.railPanel
      self.currentPanel:Show()
    end
  })

  -- Transports
  self:AddWidget({
    type = "image",
    image = "management_tc",
    position = {514, 241},
  })
  self.tc = self:AddWidget({
    type = "text",
    color = {69, 40, 23},
    position = {586, 246},
    align = "right",
    --font = "times20_font",
    text = "0"
  })
  self:AddWidget({
    type = "image",
    image = "management_button_hotspot",
    position = {517, 241},
    tooltip = "transportmoves_tooltip",
    sfx = "click",
    on_click = function()
      self.currentPanel:Hide()
      self.currentPanel = self.TransportmovesPanel
      self.currentPanel:Show()
    end
  })

  --self:Refresh()

end

function ManagementPanel:AddFaction(faction, slot)

  --wow what a bad hack...
  if slot == nil then
    slot = 0
    for faction2 in playerAlliance.factions do
      if faction2.luaData.capitals > 0 or faction.id == 26 or faction.id == 27 then
        slot = slot + 1
      end
    end
  end

  self:AddFactionPanel(faction, slot)
  self:AddFactionFlag(faction, slot)
end

function ManagementPanel:AddFactionPanel(faction, slot)
  if self.factionPanels[faction.id] == nil then
    self.factionPanels[faction.id] = self:AdoptWidget(FactionManagementPanel(faction))
    -- Display first faction by default
    if slot == 1 then
      self.factionPanels[faction.id]:Show()
    end
  else
    self:GetButton("flag" .. faction.id .. "_button"):SetPosition(19, 16 + (slot - 1) * 30)
    self:GetButton("flag" .. faction.id .. "_button"):Show()
    --self:GetImage("flag" .. faction.id .. "_disabled"):SetPosition(19, 16 + (slot - 1) * 30)
  end
end

function ManagementPanel:RemoveFaction(faction)
  -- If this faction is currently selected, select another
  if ui.managementFaction.id == faction.id then
    for otherFaction in playerAlliance.factions do
      if CanFactionProduce(otherFaction) then
        ui.managementFaction = otherFaction
        --SelectPanelFaction(otherFaction)
        break
      end
    end
  end

  -- Remove flag button
  self:GetButton("flag" .. faction.id .. "_button"):Hide()

  -- Shift other flags
  local slot = 1
  for otherFaction in playerAlliance.factions do
    if CanFactionProduce(otherFaction) then
      self:GetButton("flag" .. otherFaction.id .. "_button"):SetPosition(19, 16 + (slot - 1) * 30)
      slot = slot + 1
    end
  end
end

function ManagementPanel:AddFactionFlag(faction, slot)

  self:AddWidget({
    name = "flag" .. faction.id .. "_button",
    type = "button",
    image = "../flags/flag_" .. faction.name .. "_32_button",
    position = {19, 16 + (slot - 1) * 30},
    sfx = "click",
    tooltip = faction.name,
    on_click = function(button)
      ui.managementFaction = faction
      SelectPanelFaction(faction)
      self:Refresh()
    end
  })
end

function ManagementPanel:OnShow()
  self:Refresh()
  ui.hotkeysActive = 0
end

function ManagementPanel:Refresh()
  local row = 0
   for faction in self.alliance.factions do
     if IsFactionVisible(faction) then -- and (faction.alliance.id == sai.alliance.id or faction.alliance.id == sai.enemyAlliance.id) then
       self.financesRow[row]:Refresh(faction)
       row = row + 1
     end
     if row > 9 then
       break
     end
   end
--[[  for faction in self.alliance.factions do
    self.financesRow[row]:Refresh(faction)
    row = row + 1
    if row > 9 then
      break
    end
  end--]]
  while row <= 9 do
    self.financesRow[row]:Hide()
    row = row + 1
  end

  -- Resources
  self:GetText("pp_value"):SetNumber(ui.managementFaction.pp)
  self:GetText("mp_value"):SetNumber(ui.managementFaction.mp)
  self:GetText("am_value"):SetNumber(ui.managementFaction.am)

  self.am:SetNumber(ui.managementFaction.luaData.factories)
  self.rc:SetNumber(GetFactionMaxRailmoves(ui.managementFaction))
  self.tc:SetNumber(GetFactionMaxTransportmoves(ui.managementFaction))

  self.conditions:Refresh()
  self.factionPanels[self.previousFaction.id]:Hide()
  self.previousFaction = ui.managementFaction

  self.factionPanels[ui.managementFaction.id]:Show()
  --self.factionPanels[ui.managementFaction.id]:Refresh()

  self.currentPanel:Refresh()

end


------------------------------------------------


class 'FactionManagementPanel' (LuaPanel)

function FactionManagementPanel:__init(faction)
  LuaPanel.__init(self)
  self:Hide()

  self.faction = faction
  self:SetPosition(60, 0)

  self:AddWidget({
    name = "faction_name",
    type = "text",
    font = "hud14_font",
    position = {198, 24},
    align = "center",
    color   = {241, 237, 233},
    text = faction.name .. "_title" 
  })
  -- self:AddWidget({
  --   name = "alliance_name",
  --   type = "text",
  --   font = "hud12_font",
  --   position = {5, 16},
  --   align = "left",
  --   color   = {241, 237, 233},
  --   text = faction.luaData.collaps
  -- })
  self:AddWidget({
    name = "faction_flag",
    type = "image",
    image = "../flags/flag_" .. faction.name .. "_128",
    position = {5, 16},
	
  })

--~   self:AddWidget({
--~     type = "text",
--~     font = "hud14_font",
--~     position = {130, 128},
--~     align = "center",
--~     color   = {241, 237, 233},
--~     text = "build_factory"
--~   })
end

function FactionManagementPanel:OnShow()
  --populate with self.faction data
end


------------------------------------------------



management_table =
{
  name = "management_panel",
  type = "panel",
  position = {(engine.window.width)/2-437, (engine.window.height-200)/2-274},
  image = "management_panel",
  visible = false,

  controls =
  {
    {
      name = "pp_title",
      type = "image",
      position = {204, 39},
      image = "resource_pp_white",
      tooltip = "resourcePP"
    },
    {
      name = "pp_value",
      type = "text",
      font = "hud14_font",
      position = {258, 48},
      align = "center",
      color   = {241, 237, 233},
      tooltip = "currentPP"
    },
    {
      name = "mp_title",
      type = "image",
      position = {204, 64},
      image = "resource_mp_white",
      tooltip = "resourceMP"
    },
    {
      name = "mp_value",
      type = "text",
      font = "hud14_font",
      position = {258, 73},
      align = "center",
      color   = {241, 237, 233},
      tooltip = "currentMP"
    },
    {
      name = "am_title",
      type = "image",
      position = {204, 89},
      image = "resource_am_white",
      tooltip = "resourceAM"
    },
    {
      name = "am_value",
      type = "text",
      font = "hud14_font",
      position = {258, 98},
      align = "center",
      color   = {241, 237, 233},
      tooltip = "currentAM"
    },
--~     {
--~       name = "pp_income",
--~       type = "text",
--~       font = "hud12_font",
--~       position = {298, 48},
--~       align = "center",
--~       color   = {241, 237, 233},
--~       tooltip = "incrementPP"
--~     },
--~     {
--~       name = "mp_income",
--~       type = "text",
--~       font = "hud12_font",
--~       position = {298, 73},
--~       align = "center",
--~       color   = {241, 237, 233},
--~       tooltip = "incrementMP"
--~     },
--~     {
--~       name = "am_income",
--~       type = "text",
--~       font = "hud12_font",
--~       position = {298, 98},
--~       align = "center",
--~       color   = {241, 237, 233},
--~       tooltip = "incrementAM"
--~     },
  }
}


class 'FinancesStatsRow' (LuaPanel)

function FinancesStatsRow:__init(row)
  LuaPanel.__init(self)

  self.row = row

  self:SetPosition(351, 62)

  self:AddWidget({
    name = "finance_flag_" .. row,
    type = "image",
    image = "../flags/flag_germany_16",
    position = {0, 5 + 14 * row},
  })

  for i = 0, 6 do
    self:AddStatValue(row, i)
  end

end

function FinancesStatsRow:AddStatValue(row, step)
  self:AddWidget({
    name = "finance_stat_" .. row .. "_" .. step,
    type = "text",
    color = {69, 40, 23},
    position = {58 + step * 64, 11 + 14 * row},
    text = "0",
    align = "center"
  })
end

function FinancesStatsRow:Refresh(faction)
  self:GetImage("finance_flag_" .. self.row):SetImage("graphics/flags/", "flag_" .. faction.name .. "_16.png")
  local MPoutput = GetMPOutput(faction)
  local currentPP = faction.pp
  local productionPP = GetIncomePP(faction)
  local upkeepPP = GetUpkeep(faction)
  local researchPP = GetResearchUpkeep(faction)
  local balancePP = productionPP - upkeepPP - researchPP
  local totalPP = currentPP + balancePP
  self:GetText("finance_stat_" .. self.row .. "_0"):SetText(MPoutput .. "%")
  self:GetText("finance_stat_" .. self.row .. "_1"):SetNumber(currentPP)
  self:GetText("finance_stat_" .. self.row .. "_2"):SetText("+" .. productionPP)
  self:GetText("finance_stat_" .. self.row .. "_3"):SetText("-" .. upkeepPP)
  self:GetText("finance_stat_" .. self.row .. "_4"):SetText("-" .. researchPP)
  if balancePP >= 0 then
    self:GetText("finance_stat_" .. self.row .. "_5"):SetText("+" .. balancePP)
  else
    self:GetText("finance_stat_" .. self.row .. "_5"):SetNumber(balancePP)
  end
  self:GetText("finance_stat_" .. self.row .. "_6"):SetNumber(totalPP)
  if balancePP > 0 then
    self:GetText("finance_stat_" .. self.row .. "_5"):SetColour(70, 104, 10, 255)
  elseif balancePP < 0 then
    self:GetText("finance_stat_" .. self.row .. "_5"):SetColour(137, 31, 3, 255)
  else
    self:GetText("finance_stat_" .. self.row .. "_5"):SetColour(69, 40, 23, 255)
  end
  if totalPP > currentPP then
    self:GetText("finance_stat_" .. self.row .. "_6"):SetColour(70, 104, 10, 255)
  elseif totalPP < currentPP then
    self:GetText("finance_stat_" .. self.row .. "_6"):SetColour(137, 31, 3, 255)
  else
    self:GetText("finance_stat_" .. self.row .. "_6"):SetColour(69, 40, 23, 255)
  end
  self:Show()
end





------------------------------------------------


class 'VictoryConditionsPanel' (LuaPanel)

function VictoryConditionsPanel:__init(alliance)
  LuaPanel.__init(self)

  self:AddWidget({
    type = "text",
    position = {350, 327},
    color = {241, 237, 233},
    text = "victory_conditions"
  })

  for i = 0, 5 do
    self:AddWidget({
      type = "text",
      position = {385, 370 + i * 25},
      color = {69, 40, 23},
      font = "hud12_font",
      text = "victory_condition_" .. alliance.id .. "_" .. i
    })
  end


  self.star = {}
  self.checkbox = {}
  for i = 0, 5 do
    local stars = 0
    if i == 0 then stars = 3 end
    if i == 1 then stars = 2 end
    if i == 2 then stars = 2 end
    if i == 3 then stars = 2 end
    if i == 4 then stars = 1 end
    if i == 5 then stars = 1 end
    for j = 0, stars - 1 do
      self.star[i*10 + j] = self:AddWidget({
        type = "image",
        position = {810 - j * 20, 362 + i * 25},
        image = "victory_star_inactive"
      })
    end
    self.checkbox[i] = self:AddWidget({
      type = "image",
      position = {360, 365 + i * 25},
      image = "tech_complete"
    })
    self.checkbox[i]:Hide()
  end

end

function VictoryConditionsPanel:Refresh()

  local total = 5
--~   for _, condition in pairs(gameplay.victoryConditions) do
--~     if string.sub(condition.condition, 0, 3) == "VC_" then
--~       total = total + 1
--~     end
--~   end

  local counter = 0
  for i = 1, total do
    if gameplay.victoryConditions[i].condition().id == playerAlliance.id then
      for j = 0, gameplay.victoryConditions[i].stars - 1 do
        self.star[counter * 10 + j]:SetImage("graphics/hud", "victory_star.png")
      end
      self.checkbox[counter]:Show()
    else
      self.checkbox[counter]:Hide()
    end
    counter = counter + 1
  end

end



------------------------------------------------


class 'AmmunitionPanel' (LuaPanel)

function AmmunitionPanel:__init()
  LuaPanel.__init(self)

  self:SetPosition(60, 0)

  self:AddWidget({
    type = "image",
    position = {5, 120},
    image = "management_ammo_large"
  })

  self:AddWidget({
    type = "text",
    font = "times20_font",
    position = {138, 289},
    color   = {67, 56, 47},
    align = "center",
    text = "ammunition_title"
  })

  self.tabImage = self:AddWidget({
    type = "image",
    position = {6, 315},
    image = "infotab1"
  })

  self:AddWidget({
    type = "label",
    position = {67, 330},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "statistics",
    sfx = "click",
    on_click = function(button)
      self.statisticsPanel:Show()
      self.informationPanel:Hide()
      self.tabImage:SetImage("graphics/hud", "infotab1.png")
    end
  })

  self:AddWidget({
    type = "label",
    position = {198, 330},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "information",
    sfx = "click",
    on_click = function(button)
      self.statisticsPanel:Hide()
      self.informationPanel:Show()
      self.tabImage:SetImage("graphics/hud", "infotab2.png")
    end
  })


  self.statisticsPanel = self:AddWidget({
    type = "panel",
    position = {6, 315},
  })

  self.statisticsPanel:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {20, 38},
    align = "left",
    color   = {241, 237, 233},
    text = "current_production"
  })
  self.statisticsPanel:AddWidget({
    name = "ammo_current",
    type = "text",
    position = {50, 60},
    font = "hud12_font",
    align = "left",
    color = {241, 237, 233}
  })
  self.statisticsPanel:AddWidget({
    type = "image",
    position = {20, 58},
    image = "resource_am_white",
    tooltip = "resourceAM"
  })

  self.statisticsPanel:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {20, 85},
    align = "left",
    color   = {241, 237, 233},
    text = "expansion_effect"
  })
  self.statisticsPanel:AddWidget({
    name = "ammo_next",
    type = "text",
    position = {50, 107},
    font = "hud12_font",
    align = "left",
    color = {241, 237, 233},
  })
  self.statisticsPanel:AddWidget({
    type = "image",
    position = {20, 105},
    image = "resource_am_white",
    tooltip = "resourceAM"
  })


  self.statisticsPanel:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {20, 132},
    align = "left",
    color = {241, 237, 233},
    text = "expansion_cost"
  })
  self.statisticsPanel:AddWidget({
    name = "ammo_cost",
    type = "text",
    font = "hud12_font",
    position = {50, 154},
    align = "left",
    color   = {241, 237, 233},
  })
  self.statisticsPanel:AddWidget({
    type = "image",
    position = {20, 152},
    image = "resource_pp_white",
    tooltip = "resourcePP"
  })


  self.informationPanel = self:AddWidget({
    type = "panel",
    position = {6, 315}
  })
  self.informationPanel.content = self.informationPanel:AddWidget({
    type = "memo",
    position = {12, 42},
    size = {220, 120},
    font = "times12_font",
    color   = {224, 220, 218},
    text = "ammunition_info",
    border = 0,
    OnScroll = function(this, delta)
      if delta == 1 then
        if self.informationPanel.content.startingLine > 0 then
          self.informationPanel.content.startingLine = self.informationPanel.content.startingLine - 1
          self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
        end
      else
        if self.informationPanel.content.startingLine < self.informationPanel.content.numLines - 8 then
          self.informationPanel.content.startingLine = self.informationPanel.content.startingLine + 1
          self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
        end
      end
    end
  })
  self.informationPanel:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {230, 43},
    sfx = "click",
    on_click = function(button)
      if self.informationPanel.content.startingLine > 0 then
        self.informationPanel.content.startingLine = self.informationPanel.content.startingLine - 1
        self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
      end
    end
  })
  self.informationPanel:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {230, 148},
    sfx = "click",
    on_click = function(button)
      if self.informationPanel.content.startingLine < self.informationPanel.content.numLines - 8 then
        self.informationPanel.content.startingLine = self.informationPanel.content.startingLine + 1
        self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
      end
    end
  })
  self.informationPanel.pagebar = self.informationPanel:AddWidget({
    type = "image",
    image = "pagebar_dark",
    position = {235, 63},
  })
  self.informationPanel.content.maxLines = 8
  self.informationPanel.pageCount = math.max(8, self.informationPanel.content.numLines)
  self.informationPanel.pagebar:SetSource(0, 0, 6, 80 * 8 / self.informationPanel.pageCount)
  self.informationPanel:Hide()

  self:AddWidget({
    name = "ammo_state",
    type = "text",
    position = {75, 507},
    font = "hud12_font",
    align = "center",
    color = {254,83,30},
    text = "purchase"
  })
  self:AddWidget({
    name = "ammo_expand",
    type = "button",
    image = "button_default",
    position = {17, 492},
    text = "purchase",
    sfx = "click",
    Clicked = function()
      ConstructAmmunitionFactory(ui.managementFaction)
      self.parent:Refresh()
      ui.resourcesPanel:Refresh()
    end
  })
  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {135, 492},
    text = "close",
    sfx = "click",
    Clicked = function()
      hud:GetPanel("management_panel"):Hide()
      ui.hotkeysActive = 1
    end
  })

end

function AmmunitionPanel:OnShow()
  self:Refresh()
end

function AmmunitionPanel:Refresh()

  -- After defeat
  if ui.managementFaction == nil then
    return
  end

  self.statisticsPanel:GetText("ammo_current"):SetNumber(ui.managementFaction.luaData.factories)
  self.statisticsPanel:GetText("ammo_next"):SetText((ui.managementFaction.luaData.factories + 1) .. " (+ 1)")
  self.statisticsPanel:GetText("ammo_cost"):SetNumber(GetAmmunitionFactoryCost(ui.managementFaction))

  if ui.managementFaction.pp < GetAmmunitionFactoryCost(ui.managementFaction) then
    self:GetButton("ammo_expand"):Hide()
    self.statisticsPanel:GetText("ammo_cost"):SetColour(254,83,30,255)
  else
    self:GetButton("ammo_expand"):Show()
    self.statisticsPanel:GetText("ammo_cost"):SetColour(241, 237, 233,255)
  end
end


------------------------------------------------


class 'RailmovesPanel' (LuaPanel)

function RailmovesPanel:__init()
  LuaPanel.__init(self)

  self:SetPosition(60, 0)

  self:AddWidget({
    type = "image",
    position = {5, 120},
    image = "management_rc_large"
  })

  self:AddWidget({
    type = "text",
    font = "times20_font",
    position = {138, 289},
    color   = {67, 56, 47},
    align = "center",
    text = "railmoves_title"
  })

  self.tabImage = self:AddWidget({
    type = "image",
    position = {6, 315},
    image = "infotab1"
  })

  self:AddWidget({
    type = "label",
    position = {67, 330},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "statistics",
    sfx = "click",
    on_click = function(button)
      self.statisticsPanel:Show()
      self.informationPanel:Hide()
      self.tabImage:SetImage("graphics/hud", "infotab1.png")
    end
  })

  self:AddWidget({
    type = "label",
    position = {198, 330},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "information",
    sfx = "click",
    on_click = function(button)
      self.statisticsPanel:Hide()
      self.informationPanel:Show()
      self.tabImage:SetImage("graphics/hud", "infotab2.png")
    end
  })


  self.statisticsPanel = self:AddWidget({
    type = "panel",
    position = {6, 315},
  })

  self.statisticsPanel:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {20, 38},
    align = "left",
    color   = {241, 237, 233},
    text = "current_production"
  })
  self.statisticsPanel:AddWidget({
    name = "rail_current",
    type = "text",
    position = {50, 60},
    font = "hud12_font",
    align = "left",
    color = {241, 237, 233}
  })
  self.statisticsPanel:AddWidget({
    type = "image",
    position = {20, 58},
    image = "resource_rr_white",
    tooltip = "resourceRC"
  })

  self.statisticsPanel:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {20, 85},
    align = "left",
    color   = {241, 237, 233},
    text = "expansion_effect"
  })
  self.statisticsPanel:AddWidget({
    name = "rail_next",
    type = "text",
    position = {50, 107},
    font = "hud12_font",
    align = "left",
    color = {241, 237, 233},
  })
  self.statisticsPanel:AddWidget({
    type = "image",
    position = {20, 105},
    image = "resource_rr_white",
    tooltip = "resourceRC"
  })


  self.statisticsPanel:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {20, 132},
    align = "left",
    color = {241, 237, 233},
    text = "expansion_cost"
  })
  self.statisticsPanel:AddWidget({
    name = "rail_cost",
    type = "text",
    font = "hud12_font",
    position = {50, 154},
    align = "left",
    color   = {241, 237, 233},
  })
  self.statisticsPanel:AddWidget({
    type = "image",
    position = {20, 152},
    image = "resource_pp_white",
    tooltip = "resourcePP"
  })


  self.informationPanel = self:AddWidget({
    type = "panel",
    position = {6, 315}
  })
  self.informationPanel.content = self.informationPanel:AddWidget({
    type = "memo",
    position = {12, 42},
    size = {220, 120},
    font = "times12_font",
    color   = {224, 220, 218},
    text = "railmoves_info",
    border = 0,
    OnScroll = function(this, delta)
      if delta == 1 then
        if self.informationPanel.content.startingLine > 0 then
          self.informationPanel.content.startingLine = self.informationPanel.content.startingLine - 1
          self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
        end
      else
        if self.informationPanel.content.startingLine < self.informationPanel.content.numLines - 8 then
          self.informationPanel.content.startingLine = self.informationPanel.content.startingLine + 1
          self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
        end
      end
    end
  })
  self.informationPanel:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {230, 43},
    sfx = "click",
    on_click = function(button)
      if self.informationPanel.content.startingLine > 0 then
        self.informationPanel.content.startingLine = self.informationPanel.content.startingLine - 1
        self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
      end
    end
  })
  self.informationPanel:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {230, 148},
    sfx = "click",
    on_click = function(button)
      if self.informationPanel.content.startingLine < self.informationPanel.content.numLines - 8 then
        self.informationPanel.content.startingLine = self.informationPanel.content.startingLine + 1
        self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
      end
    end
  })
  self.informationPanel.pagebar = self.informationPanel:AddWidget({
    type = "image",
    image = "pagebar_dark",
    position = {235, 63},
  })
  self.informationPanel.content.maxLines = 8
  self.informationPanel.pageCount = math.max(8, self.informationPanel.content.numLines)
  self.informationPanel.pagebar:SetSource(0, 0, 6, 80 * 8 / self.informationPanel.pageCount)
  self.informationPanel:Hide()

  self:AddWidget({
    name = "ammo_state",
    type = "text",
    position = {75, 507},
    font = "hud12_font",
    align = "center",
    color = {254,83,30},
    text = "purchase"
  })
  self:AddWidget({
    name = "rail_expand",
    type = "button",
    image = "button_default",
    position = {17, 492},
    text = "purchase",
    sfx = "click",
    Clicked = function()
      if PurchaseFactionRailmoves(ui.managementFaction) then
        self.parent:Refresh()
        ui.resourcesPanel:Refresh()
      end
    end
  })
  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {135, 492},
    text = "close",
    sfx = "click",
    Clicked = function()
      hud:GetPanel("management_panel"):Hide()
    end
  })

  self:Hide()

end

function RailmovesPanel:OnShow()
  self:Refresh()
end

function RailmovesPanel:Refresh()
  local cost = GetRailmovesIncrementCost(GetFactionMaxRailmoves(ui.managementFaction))
  local current = GetFactionMaxRailmoves(ui.managementFaction)

  self.statisticsPanel:GetText("rail_current"):SetNumber(current)
  self.statisticsPanel:GetText("rail_next"):SetText((current + 1) .." (+ 1)")
  self.statisticsPanel:GetText("rail_cost"):SetNumber(cost)

  if ui.managementFaction.pp < cost then
    self:GetButton("rail_expand"):Hide()
    self.statisticsPanel:GetText("rail_cost"):SetColour(254,83,30,255)
  else
    self:GetButton("rail_expand"):Show()
    self.statisticsPanel:GetText("rail_cost"):SetColour(241, 237, 233,255)
  end
end

------------------------------------------------


class 'TransportmovesPanel' (LuaPanel)

function TransportmovesPanel:__init()
  LuaPanel.__init(self)

  self:SetPosition(60, 0)

  self:AddWidget({
    type = "image",
    position = {5, 120},
    image = "management_tc_large"
  })

  self:AddWidget({
    type = "text",
    font = "times20_font",
    position = {138, 289},
    color   = {67, 56, 47},
    align = "center",
    text = "transport_title"
  })

  self.tabImage = self:AddWidget({
    type = "image",
    position = {6, 315},
    image = "infotab1"
  })

  self:AddWidget({
    type = "label",
    position = {67, 330},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "statistics",
    sfx = "click",
    on_click = function(button)
      self.statisticsPanel:Show()
      self.informationPanel:Hide()
      self.tabImage:SetImage("graphics/hud", "infotab1.png")
    end
  })

  self:AddWidget({
    type = "label",
    position = {198, 330},
    align = "center",
    font = "hud12_font",
    normal   = {224, 220, 218},
    hilight  = {255, 255, 255},
    select   = {255, 255, 255},
    disabled = {224, 220, 218},
    text = "information",
    sfx = "click",
    on_click = function(button)
      self.statisticsPanel:Hide()
      self.informationPanel:Show()
      self.tabImage:SetImage("graphics/hud", "infotab2.png")
    end
  })


  self.statisticsPanel = self:AddWidget({
    type = "panel",
    position = {6, 315},
  })

  self.statisticsPanel:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {20, 38},
    align = "left",
    color   = {241, 237, 233},
    text = "current_production"
  })
  self.statisticsPanel:AddWidget({
    name = "transport_current",
    type = "text",
    position = {50, 60},
    font = "hud12_font",
    align = "left",
    color = {241, 237, 233}
  })
  self.statisticsPanel:AddWidget({
    type = "image",
    position = {20, 58},
    image = "resource_tc_white",
    tooltip = "resourceTC"
  })

  self.statisticsPanel:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {20, 85},
    align = "left",
    color   = {241, 237, 233},
    text = "expansion_effect"
  })
  self.statisticsPanel:AddWidget({
    name = "transport_next",
    type = "text",
    position = {50, 107},
    font = "hud12_font",
    align = "left",
    color = {241, 237, 233},
  })
  self.statisticsPanel:AddWidget({
    type = "image",
    position = {20, 105},
    image = "resource_tc_white",
    tooltip = "resourceTC"
  })


  self.statisticsPanel:AddWidget({
    type = "text",
    font = "hud12_font",
    position = {20, 132},
    align = "left",
    color = {241, 237, 233},
    text = "expansion_cost"
  })
  self.statisticsPanel:AddWidget({
    name = "transport_cost",
    type = "text",
    font = "hud12_font",
    position = {50, 154},
    align = "left",
    color   = {241, 237, 233},
  })
  self.statisticsPanel:AddWidget({
    type = "image",
    position = {20, 152},
    image = "resource_pp_white",
    tooltip = "resourcePP"
  })


  self.informationPanel = self:AddWidget({
    type = "panel",
    position = {6, 315}
  })
  self.informationPanel.content = self.informationPanel:AddWidget({
    type = "memo",
    position = {12, 42},
    size = {220, 120},
    font = "times12_font",
    color   = {224, 220, 218},
    text = "transport_info",
    border = 0,
    OnScroll = function(this, delta)
      if delta == 1 then
        if self.informationPanel.content.startingLine > 0 then
          self.informationPanel.content.startingLine = self.informationPanel.content.startingLine - 1
          self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
        end
      else
        if self.informationPanel.content.startingLine < self.informationPanel.content.numLines - 8 then
          self.informationPanel.content.startingLine = self.informationPanel.content.startingLine + 1
          self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
        end
      end
    end
  })
  self.informationPanel:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {230, 43},
    sfx = "click",
    on_click = function(button)
      if self.informationPanel.content.startingLine > 0 then
        self.informationPanel.content.startingLine = self.informationPanel.content.startingLine - 1
        self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
      end
    end
  })
  self.informationPanel:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {230, 148},
    sfx = "click",
    on_click = function(button)
      if self.informationPanel.content.startingLine < self.informationPanel.content.numLines - 8 then
        self.informationPanel.content.startingLine = self.informationPanel.content.startingLine + 1
        self.informationPanel.pagebar:SetPosition(235, 63 + (80 * self.informationPanel.content.startingLine / self.informationPanel.pageCount) )
      end
    end
  })
  self.informationPanel.pagebar = self.informationPanel:AddWidget({
    type = "image",
    image = "pagebar_dark",
    position = {235, 63},
  })
  self.informationPanel.content.maxLines = 8
  self.informationPanel.pageCount = math.max(8, self.informationPanel.content.numLines)
  self.informationPanel.pagebar:SetSource(0, 0, 6, 80 * 8 / self.informationPanel.pageCount)
  self.informationPanel:Hide()

  self:AddWidget({
    name = "transport_state",
    type = "text",
    position = {75, 507},
    font = "hud12_font",
    align = "center",
    color = {254,83,30},
    text = "purchase"
  })
  self:AddWidget({
    name = "transport_expand",
    type = "button",
    image = "button_default",
    position = {17, 492},
    text = "purchase",
    sfx = "click",
    Clicked = function()
      if PurchaseFactionTransportmoves(ui.managementFaction) then
        self.parent:Refresh()
        ui.resourcesPanel:Refresh()
      end
    end
  })
  self:AddWidget({
    type = "button",
    image = "button_default",
    position = {135, 492},
    text = "close",
    sfx = "click",
    Clicked = function()
      hud:GetPanel("management_panel"):Hide()
    end
  })

  self:Hide()

end

function TransportmovesPanel:OnShow()
  self:Refresh()
  ui.hotkeysActive = 0
end

function TransportmovesPanel:Refresh()
  local cost = GetTransportmovesIncrementCost(GetFactionMaxTransportmoves(ui.managementFaction))
  local current = GetFactionMaxTransportmoves(ui.managementFaction)

  self.statisticsPanel:GetText("transport_current"):SetNumber(current)
  self.statisticsPanel:GetText("transport_next"):SetText((current + 1) .." (+ 1)")
  self.statisticsPanel:GetText("transport_cost"):SetNumber(cost)

  if ui.managementFaction.pp < cost then
    self:GetButton("transport_expand"):Hide()
    self.statisticsPanel:GetText("transport_cost"):SetColour(254,83,30,255)
  else
    self:GetButton("transport_expand"):Show()
    self.statisticsPanel:GetText("transport_cost"):SetColour(241, 237, 233,255)
  end
end

------------------------------------------------
