class 'ManagementTab' (LuaPanel)

function ManagementTab:__init()
  LuaPanel.__init(self, management_tab)

  ui.statsType = 0

  self.factionStats = {}

  self.displayLimit = 12

  local step = 0
  for faction in game.factions do
    --if IsFactionVisible(faction) then
      self.factionStats[faction.id] = self:AdoptWidget(ManagementFactionStats(faction))
      step = step + 1
    --end
  end

  self.kills_button = self:AddWidget({
    type = "button",
    image = "button_kills",
    position = {25, 39},
    sfx = "click",
		tooltip = "display_casualties",
    Clicked = function(button)
      ui.statsType = 1
      self.kills_button:Hide()
      self.casualties_button:Show()
      self:Refresh()
    end
  })
  self.casualties_button = self:AddWidget({
    type = "button",
    image = "button_casualties",
    position = {25, 39},
    sfx = "click",
		tooltip = "display_kills",
    Clicked = function(button)
      ui.statsType = 0
      self.kills_button:Show()
      self.casualties_button:Hide()
      self:Refresh()
    end
  })
  self.casualties_button:Hide()

  self:AddWidget({
    type = "image",
    position = {25, 65},
    tooltip = "class_ground",
    image = "class_ground",
  })
  self:AddWidget({
    type = "image",
    position = {25, 87},
    tooltip = "class_artillery",
    image = "class_artillery",
  })
  self:AddWidget({
    type = "image",
    position = {25, 109},
    tooltip = "class_air",
    image = "class_air",
  })
  self:AddWidget({
    type = "image",
    position = {25, 131},
    tooltip = "class_naval",
    image = "class_naval",
  })
  self:AddWidget({
    type = "image",
    position = {25, 153},
    tooltip = "class_armour",
    image = "class_armour",
  })

end

function ManagementTab:OnShow()
  self:Refresh()

  hud:GetPanel("management_panel"):Show()
end

function ManagementTab:OnHide()
  hud:GetPanel("management_panel"):Hide()
end

function ManagementTab:Refresh()
  local count = 0
  local refreshStats = function(faction)
    if IsFactionVisible(faction) then
      self.factionStats[faction.id]:Refresh(count)
      count = count + 1
    end
  end
  for faction in playerAlliance.factions do
      refreshStats(faction)
  end
  for faction in ui.enemyAlliance.factions do
      refreshStats(faction)
  end
  for faction in ui.neutralAlliance.factions do
    if IsFactionVisible(faction) then
      refreshStats(faction)
    end
  end
end

-------------------------------------------------------

class 'ManagementFactionStats' (LuaPanel)

function ManagementFactionStats:__init(faction)
  LuaPanel.__init(self)

  self.faction = faction

    self:AddWidget({
      name = "flag" .. faction.id,
      type = "image",
      image = "../flags/flag_" .. faction.name .. "_32",
      position = {0, 0},
      tooltip = faction.name
    })
    self:AddWidget({
      name = "ground_" .. faction.id,
      type = "text",
      align = "center",
      position = {16, 38},
      color   = {69, 40, 23}
    })
    self:AddWidget({
      name = "artillery_" .. faction.id,
      type = "text",
      align = "center",
      position = {16, 60},
      color   = {69, 40, 23}
    })
    self:AddWidget({
      name = "air_" .. faction.id,
      type = "text",
      align = "center",
      position = {16, 82},
      color   = {69, 40, 23}
    })
    self:AddWidget({
      name = "naval_" .. faction.id,
      type = "text",
      align = "center",
      position = {16, 104},
      color   = {69, 40, 23}
    })
    self:AddWidget({
      name = "armour_" .. faction.id,
      type = "text",
      align = "center",
      position = {16, 126},
      color   = {69, 40, 23}
    })

  self:Hide()
end

function ManagementFactionStats:Refresh(count)
  self:SetPosition(60 + count * 35, 40)

  if ui.statsType == 1 then
    self:GetText("ground_" .. self.faction.id):SetNumber(self.faction.luaData.statsCasualties["ground"]/10)
    self:GetText("artillery_" .. self.faction.id):SetNumber(self.faction.luaData.statsCasualties["artillery"]/10)
    self:GetText("air_" .. self.faction.id):SetNumber(self.faction.luaData.statsCasualties["air"]/10)
    self:GetText("naval_" .. self.faction.id):SetNumber(self.faction.luaData.statsCasualties["naval"]/10)
    self:GetText("armour_" .. self.faction.id):SetNumber(self.faction.luaData.statsCasualties["armour"]/10)
  else
    self:GetText("ground_" .. self.faction.id):SetNumber(self.faction.luaData.statsKills["ground"]/10)
    self:GetText("artillery_" .. self.faction.id):SetNumber(self.faction.luaData.statsKills["artillery"]/10)
    self:GetText("air_" .. self.faction.id):SetNumber(self.faction.luaData.statsKills["air"]/10)
    self:GetText("naval_" .. self.faction.id):SetNumber(self.faction.luaData.statsKills["naval"]/10)
    self:GetText("armour_" .. self.faction.id):SetNumber(self.faction.luaData.statsKills["armour"]/10)
  end

  if count < self.parent.displayLimit then
    self:Show()
  else
    self:Hide()
  end
end

function ManagementFactionStats:StatsKills()

  self:AddWidget({
    type = "button",
    image = "button_25x26",
    position = {25, 40},
    sfx = "click",
    tooltip = "display_casualties",
    Clicked = function(button)
      self:Hide()
    end
  })
  self:AddWidget({
    type = "image",
    position = {25, 65},
    image = "class_ground",
    tooltip = "kills_ground"
  })
  self:AddWidget({
    type = "image",
    position = {25, 95},
    image = "class_artillery",
    tooltip = "kills_artillery"
  })
  self:AddWidget({
    type = "image",
    position = {25, 125},
    image = "class_air",
    tooltip = "kills_air"
  })
  self:AddWidget({
    type = "image",
    position = {25, 155},
    image = "class_naval",
    tooltip = "kills_naval"
  })

end

function ManagementFactionStats:StatsCasualties()


end

function ManagementFactionStats:Stats(set)

  self:AddWidget({
    type = "button",
    image = "button_25x26",
    position = {25, 40},
    sfx = "click",
    tooltip = "display_casualties",
    Clicked = function(button)
      self:Hide()
    end
  })
  self:AddWidget({
    type = "image",
    position = {25, 65},
    image = "class_ground",
    tooltip = "kills_ground"
  })
  self:AddWidget({
    type = "image",
    position = {25, 90},
    image = "class_artillery",
    tooltip = "kills_artillery"
  })
  self:AddWidget({
    type = "image",
    position = {25, 115},
    image = "class_air",
    tooltip = "kills_air"
  })
  self:AddWidget({
    type = "image",
    position = {25, 140},
    image = "class_naval",
    tooltip = "kills_naval"
  })
  self:AddWidget({
    type = "image",
    position = {25, 135},
    image = "class_armour",
    tooltip = "kills_armour"
  })

end

management_tab =
{
  name = "management_tab",
  type = "panel",
  image = "tabs_statistics",

  controls =
  {
    {
      name = "queue_image",
      type = "image",
      position = {12, 28},
      image = "tab_management"
    },
  }
}
