class 'ProductionTab' (LuaPanel)

function ProductionTab:__init()
  LuaPanel.__init(self)

  self:SetBackgroundImage("tabs_production")

  self.scroll = 0

  self.scrollUp = self:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {485, 32},
    sfx = "click",
    on_click = function(button)
      if self.scroll > 0 then
        self.scroll = self.scroll - 1
        if self.scroll == 0 then
          button:Hide()
        end
        self.scrollDown:Show()

        self:Refresh()
      end
    end
  })
  self.scrollUp:Hide()

  self.scrollDown = self:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {485, 166},
    sfx = "click",
    on_click = function(button)
      if self.scroll < math.floor((self.queueCounter - 1) / 6) then
        self.scroll = self.scroll + 1
        if self.scroll == math.floor((self.queueCounter - 1) / 6) then
          button:Hide()
        end
        self.scrollUp:Show()

        self:Refresh()
      end
    end
  })

  -- Production Queue
  self.slots = {}
  for i=0,11 do
    self.slots[i] = self:AdoptWidget(ProductionQueueElement(i))
  end
end

function ProductionTab:OnShow()
  hud:GetPanel("production_panel"):Show()

  self:Refresh(0)
end

function ProductionTab:OnHide()
  hud:GetPanel("production_panel"):Hide()
end


function ProductionTab:Refresh(row)
  local slot = 0
  local queuePos = 0
  local unitImage, itemPanel
  local maxSlots = 12

  -- Call can reset queue scroll row
  if row ~= nil and row ~= self.scroll then
    self.scroll = row
  end

  -- Commanders
  for faction in playerAlliance.factions do
    if faction.luaData.commanders ~= nil then
      for id, commander in pairs(faction.luaData.commanders) do
        if commander.unlocked ~= 0 and faction.luaData.commanders[id].alive == 1 and commander.unit == nil then
          if queuePos >= self.scroll*6 and slot < maxSlots then

            itemPanel = self.slots[slot]

            itemPanel:Show()

            if commander.turns == 0 then
              itemPanel.turnIcon:Hide()
              itemPanel.turnText:SetColour(241, 237, 233, 255)
              itemPanel.turnText:SetLocalText("ready")
              itemPanel.unitIcon.tooltip = "clickdeploy"
            else
              itemPanel.turnIcon:Show()
              itemPanel.turnText:SetColour(241, 237, 233, 255)
              itemPanel.turnText:SetNumber(commander.turns)
              itemPanel.unitIcon.tooltip = "commander_not_ready"
            end

            itemPanel.flag:SetImage("graphics/flags", "flag_" .. faction.name .. "_16.png" )
            itemPanel.unitIcon:SetImage("graphics/commanders", "commander_" .. id .. "_queue.png")

            itemPanel.info.queue = id
            itemPanel.info.faction = faction
            itemPanel.info.type = "commander"
            itemPanel.info.turns = commander.turns

            slot = slot + 1
          end
          queuePos = queuePos + 1
        end
      end
    end
  end

  --go through queue
  --start changing queue images from queue item row*6
  if settings.sortQueue == 1 then
    slot, queuePos = self:QueueAddUnits(slot, queuePos, "ready")
    slot, queuePos = self:QueueAddUnits(slot, queuePos, "building")
  else
    slot, queuePos = self:QueueAddUnits(slot, queuePos)
  end

  -- erase remaining items
  if slot < 12 then
    for i = slot, 11 do
      self.slots[i]:Hide()
    end
  end

  self.queueCounter = queuePos

--~   if self.scroll > 0 then
--~     self.scrollDown:Hide()
--~   else
--~     self.scrollDown:Show()
--~   end
  if self.queueCounter <= self.scroll * 6 + 12 then
    self.scrollDown:Hide()
  else
    self.scrollDown:Show()
  end

end


function ProductionTab:QueueAddUnits(slot, queuePos, sort)
  local unitImage, itemPanel
  local maxSlots = 12

  for faction in playerAlliance.factions do
  --if faction.alliance.id == sai.alliance.id or faction.alliance.id == sai.enemyAlliance.id then
    i = 0
    for unit in faction.production_que do
      local valid = false
      if unit.turns == 0 and sort ~= "building" then
        valid = true
      end
      if unit.turns ~= 0 and sort ~= "ready" then
        valid = true
      end
      if valid then
        if queuePos >= self.scroll*6 and slot < maxSlots then

          itemPanel = self.slots[slot]

          itemPanel:Show()

          if unit.turns == 0 then
            itemPanel.turnIcon:Hide()
            itemPanel.turnText:SetColour(10, 5, 0,255)
            itemPanel.turnText:SetLocalText("ready")
            itemPanel.unitIcon.tooltip = "clickdeploy"
          else
            itemPanel.turnIcon:Show()
            itemPanel.turnText:SetColour(10, 5, 0,255)
            itemPanel.turnText:SetNumber(unit.turns)
            itemPanel.unitIcon.tooltip = "clickcancel"
          end
          itemPanel.flag:SetImage("graphics/flags", "flag_" .. faction.name .. "_16.png" )

          if engine:FileExists("graphics/units", unit.prototype.name .. "_" .. faction.name .. "_" .. unit.level .. "_queue.png") then
            unitImage = unit.prototype.name .. "_" .. faction.name .. "_" .. unit.level .. "_queue.png"
          else
            unitImage = unit.prototype.name .. "_default_" .. unit.level .. "_queue.png"
          end
          itemPanel.unitIcon:SetImage("graphics/units", unitImage)

          itemPanel.info.queue = i
          itemPanel.info.faction = faction
          itemPanel.info.type = "unit"
          itemPanel.info.unit = unit
          itemPanel.info.turns = unit.turns

          slot = slot + 1
        end
        queuePos = queuePos + 1
      end
      i = i + 1
    end
  --end
  end

  return slot, queuePos
end


-------------------------------------------------------------------

class 'ProductionQueueElement' (LuaPanel)

function ProductionQueueElement:__init(slot)
  LuaPanel.__init(self)

  self:SetPosition(10 + (slot % 6) * 79, 32 + math.floor(slot / 6) * 77)

  self:SetBackgroundImage("queue")

  self.info = {}

  self.unitIcon = self:AddWidget({
    name = "queueunit_image",
    type = "image",
    position = {5, 1},
    sfx = "click",
    --tooltip = "clickdeploy",
    on_click = function()
      if self.info.turns == 0 then
        -- If not a commander
        if self.info.type == "unit" then
          ui.UIMode = DEPLOY
          ui.cursormode:Show()

          ui.prodqueue.faction = self.info.faction
          ui.prodqueue.unit = self.info.unit
          ui.prodqueue.slot = self.info.queue
          --ui.prodqueue.unit = ui.production_unit[this.parent.name]
          hud:GetWidget("production_panel"):Hide()
          HighlightDeployment(self.info.unit, self.info.faction)

          AdviceDeploy(self.info.faction, self.info.unit)
        else
          ui.UIMode = ATTACH
          ui.cursormode:Show()

          ui.prodqueue.faction = self.info.faction
          ui.prodqueue.slot = self.info.queue

          DisplayAttachZone(self.info.queue)

          hud:GetWidget("production_panel"):Hide()
        end
        self:SetBackgroundImage("queue_select")
        if ui.selectedQueueSlot ~= nil then
          ui.selectedQueueSlot:SetBackgroundImage("queue")
        end
        ui.selectedQueueSlot = self
      elseif self.info.type == "unit" then

        function CancelConfirmed()
          CancelUnit(self.info.faction, self.info.unit, self.info.queue)
          ui.resourcesPanel:Refresh()
          self.parent:Refresh()
        end
        PopupDialog("confirm_unit_cancel", CancelConfirmed)

      end
    end
  })

  self.flag = self:AddWidget({
    name = "queueflag_image",
    type = "image",
    position = {8, 56},
  })

  self.turnText = self:AddWidget({
    name = "queueturns_text",
    type = "text",
    font = "hud12_font",
    position = {52, 63},
    align = "center",
    color   = {10, 5, 0},
  })
  self.turnIcon = self:AddWidget({
    name = "queueturns_icon",
    type = "image",
    position = {60, 56},
    image = "turn_icon"
  })

end
