class 'ResearchTab' (LuaPanel)

function ResearchTab:__init()
  LuaPanel.__init(self)

  self.scroll = 0

  self.alliance = playerAlliance

  self:AddWidget({
    type = "image",
    image = "tabs_research",
    position = {0, 0},
  })

  self.queueItem = {}

  for i = 0, 11 do
    self.queueItem[i] = self:AdoptWidget(TechQueueItem(i))
  end

  self.scrollUp = self:AddWidget({
    type = "button",
    image = "scroll_up",
    position = {485, 32},
    sfx = "click",
    on_click = function(button)
      if self.scroll > 0 then
        self.scroll = self.scroll - 1
        if self.scroll == 0 then
          button:Hide()
        end
        self.scrollDown:Show()

        self:Refresh()
      end
    end
  })
  self.scrollUp:Hide()

  self.scrollDown = self:AddWidget({
    type = "button",
    image = "scroll_down",
    position = {485, 166},
    sfx = "click",
    on_click = function(button)
      if self.scroll < math.floor((self.queueCounter - 1) / 4) then
        self.scroll = self.scroll + 1
        if self.scroll == math.floor((self.queueCounter - 1) / 4) then
          button:Hide()
        end
        self.scrollUp:Show()

        self:Refresh()
      end
    end
  })

  self:CountQueueTechs()

end

function ResearchTab:OnShow()
  hud:GetPanel("research_panel"):Show()
  self:Refresh()
end

function ResearchTab:OnHide()
  hud:GetPanel("research_panel"):Hide()
end

function ResearchTab:Refresh(row)
  local techCategories = {"ground", "artillery", "air", "naval", "armour"}
  local slot = 0
  local queuePos = 0
  local maxSlots = 12

  -- Call can reset queue scroll row
  if row ~= nil and row ~= self.scroll then
    self.scroll = row
  end

  for faction in self.alliance.factions do

    -- Break at limit
    if slot >= maxSlots then
      break
    end


    if CanFactionProduce(faction) then

      -- Display techs nearing completion
      local techsTable = GetAvailableTechs(faction)
      for k, techName in pairs(techsTable) do
        if GetTechTime(faction, techName) ~= -1 then
        if queuePos >= self.scroll*6 then
          self.queueItem[slot]:Update(faction, techName)
          slot = slot + 1
          if slot >= maxSlots then
          break
          end
        end
        queuePos = queuePos + 1
        end
      end

      -- Break at limit
      if slot >= maxSlots then
        break
      end

    end

  end

  while slot < maxSlots do
    self.queueItem[slot]:Hide()
    slot = slot + 1
  end
end

function ResearchTab:CountQueueTechs()

  self.queueCounter = 0
  self.scroll = 0

  for faction in self.alliance.factions do

    local techsTable = GetAvailableTechs(faction)
    for k, techName in pairs(techsTable) do
      if GetTechTime(faction, techName) ~= -1 then
        self.queueCounter = self.queueCounter + 1
      end
    end

  end

  self.scrollUp:Hide()
  if self.queueCounter <= 12 then
    self.scrollDown:Hide()
  else
    self.scrollDown:Show()
  end

end

function ResearchTab:ResearchOverviewFlag(faction, step)

  local factionName = faction.name

  overviewFlag =
  {
    name = "overviewFlag" .. faction.id,
    type = "panel",
    position = {0, 53 + step * 16},
    --image = "tab_blocking",

    controls =
    {
      {
        name = "flag" .. faction.id .. "_image",
        type = "image",
        image = "../flags/flag_" .. factionName .. "_16",
        position = {28, 0},
        tooltip = factionName
      },
      {
        name = "overview_expenses_value",
        type = "text",
        font = "hud10_font",
        position = {85, 5},
        color   = {42, 34, 22},
        shadow = {offset = {0, 0}},
        align = "center",
        text = "zero",
      },
      {
        name = "overview_tech1_text",
        type = "text",
        font = "hud10_font",
        position = {188, 5},
        color   = {42, 34, 22},
        shadow = {offset = {0, 0}},
        align = "center",
        text = "tech_0",
      },
      {
        name = "overview_time1_text",
        type = "text",
        font = "hud10_font",
        position = {263, 5},
        color   = {42, 34, 22},
        shadow = {offset = {0, 0}},
        align = "center",
        text = "zero",
      },
    }
  }

  self:AddWidget(overviewFlag):Show()
end

research_tab =
{
  name = "research_tab",
  type = "panel",
  rect = {0, 0, 500, 500},
  image = "tabs_research",


  controls =
  {
    {
      name = "overview_image",
      type = "image",
      position = {0, 26},
      image = "tab_research",
      group = "research"
    },
    {
      name = "expenses_text",
      type = "text",
      font = "hud12_font",
      position = {85, 44},
      color   = {42, 34, 22},
      shadow = {offset = {0, 0}},
      align = "center",
      text = "expenses",
    },
    {
      name = "technologies_text",
      type = "text",
      font = "hud12_font",
      position = {169, 44},
      color   = {42, 34, 22},
      shadow = {offset = {0, 0}},
      align = "center",
      text = "technologies",
    },
  }
}


---------------------------------------------------------


class 'TechQueueItem' (LuaPanel)

function TechQueueItem:__init(slot)
  LuaPanel.__init(self)

  self:SetPosition(10 + (slot % 6) * 79, 32 + math.floor(slot / 6) * 77)

  self:SetBackgroundImage("queue")

  self.techName = -1

  self.boosted = 0

  self:AddWidget({
    name = "tech_icon",
    type = "image",
    position = {7, 5},
    sfx = "click",
    on_click = function(button)
      ui.researchPanel:SelectFaction(self.faction)
      ui.researchPanel:Show()
      if self.techName ~= -1 then
        for techClass, _ in pairs(data.techLines) do
        for i, techLine in pairs(data.techLines[techClass]) do
          for _, tech in pairs(techLine) do
            if tech == self.techName then
              ui.researchPanel:UpdateTechStats(self.techName, ui.researchPanel.Techtree[techClass].techLines[i], ui.researchFaction)
              break
            end
          end
        end
        end
      end
    end
  })

  self:AddWidget({
    name = "tech_turns",
    type = "text",
    font = "hud12_font",
    position = {52, 63},
    color   = {10, 5, 0},
    align = "center",
    text = "10",
  })

  self:AddWidget({
    name = "tech_flag",
    type = "image",
    position = {8, 56},
  })

  self:AddWidget({
    name = "tech_turns_icon",
    type = "image",
    position = {60, 56},
    image = "turn_icon"
  })

  self:Hide()
end

function TechQueueItem:Update(faction, techName)

  self.techName = techName
  self.faction = faction

  if techName ~= -1 then

    local time = GetTechTime(faction, techName)
    if time ~= -1 then
      self:GetText("tech_turns"):SetNumber(time)
    else
      self:GetText("tech_turns"):SetText("?")
    end
    self:GetText("tech_turns"):Show()
    self:GetImage("tech_turns_icon"):Show()

    self:GetImage("tech_flag"):SetImage("graphics/flags", "flag_" .. faction.name .. "_16.png" )

    self:GetImage("tech_icon"):SetImage("graphics/techs", techName .. "_queue.png" )
    self:GetImage("tech_icon").tooltip = "tech_" .. techName

    local techClass = data.newTechnologies[techName]
    if faction.luaData.currentTech[techClass] == techName and self.boosted == 0 then
      self:SetBackgroundImage("queue_boosted")
      self.boosted = 1
    elseif faction.luaData.currentTech[techClass] ~= techName and self.boosted == 1 then
      self:SetBackgroundImage("queue")
      self.boosted = 0
    end

  else

    self:GetText("tech_turns"):Hide()
    self:GetImage("tech_turns_icon"):Hide()

    self:GetImage("tech_flag"):SetImage("graphics/flags", "flag_" .. faction.name .. "_16.png" )

    self:GetImage("tech_icon"):SetImage("graphics/hud", "no_tech.png" )
    self:GetImage("tech_icon").tooltip = "no_tech"

  end

  self:Show()
end
