class 'SelectionTab' (LuaPanel)

function SelectionTab:__init()
  LuaPanel.__init(self, selection_tab)

  -- Boost icons
  self:CreateBoostIcon(0)
  self:CreateBoostIcon(1)
  self:CreateBoostIcon(2)
  self:CreateBoostIcon(3)
  self:CreateBoostIcon(4)
  self:CreateBoostIcon(5)
  self:CreateBoostIcon(6)

  self:CreateBoostIcon("gas")
  self:CreateBoostIcon("gasmask")
  self:CreateBoostIcon("port")
  self:CreateBoostIcon("amphibious")
  self:CreateBoostIcon("trample")
  self:CreateBoostIcon("quickretreat")
  self:CreateBoostIcon("torpedoA")
  self:CreateBoostIcon("torpedoD")
  self:CreateBoostIcon("firstStrike")
  self:CreateBoostIcon("mud")


  self.prognosisPanel = self:GetPanel("prognosis_panel")
  self.prognosisPanel:Hide()
end

function SelectionTab:CreateBoostIcon(id)

  local BoostImage =
  {
    name = "boost_" .. id .. "_image",
    type = "image",
    position = {0, 0},
    image = "boost_" .. id,
    tooltip = "boost_" .. id
  }

  --self:GetPanel("stats_panel"):AddWidget(BoostImage):Hide()
  self:GetPanel("prognosis_panel"):AddWidget(BoostImage):Hide()

end

function SelectionTab:DisplayPrognosis(attacker, defender)

  hud:GetPanel("prognosis_panel"):Show()
  hud:GetPanel("stats_panel"):Hide()
  hud:GetPanel("terrain_panel"):Hide()

  local damage = CalculateDamage(attacker, defender, true)

  local attackerHex = attacker.hex

  if attacker.cargo ~= nil then
    for cargo in attacker.cargo.units do
      attacker = cargo
      break
    end
  end

  local attackerEfficiency, defenderEfficiency = CalculateEfficiency(attacker,defender,damage[1],damage[2])

  local panelID = self:GetPanel("prognosis_panel")
  if settings.showKills == 1 then
    panelID:GetText("kills_text"):SetNumber(damage[2])
    panelID:GetText("casualties_text"):SetNumber(damage[1])
  else
    panelID:GetText("kills_text"):SetNumber(-damage[1])
    panelID:GetText("casualties_text"):SetNumber(-damage[2])
  end
  panelID:GetText("damage4_text"):SetNumber((defenderEfficiency - defender.efficiency) / 10)
  panelID:GetText("damage3_text"):SetNumber((attackerEfficiency - attacker.efficiency) / 10)

  local unitImage
  if engine:FileExists("graphics/units", defender.prototype.name .. "_" .. defender.faction.name .. "_" .. defender.level .. "_hud.png") then
    unitImage = defender.prototype.name .. "_" .. defender.faction.name .. "_" .. defender.level .. "_hud.png"
  else
    unitImage = defender.prototype.name .. "_default_" .. defender.level .. "_hud.png"
  end
  hud:GetImage("u2_image"):SetImage("graphics/units", unitImage)
  hud:GetImage("u2-level_image"):SetImage("graphics/hud", "level" .. GetUnitTechCount(defender) .. ".png")
  hud:GetImage("u2-nation_image"):SetImage("graphics/flags", "emblem_" .. defender.faction.name .. "100.png")
  --hud:GetImage("ter-owner_image"):SetImage("graphics/flags", "emblem_" .. defender.hex.faction.name .. "100.png")
  hud:GetImage("u2-class_image"):SetImage("graphics/units", defender.prototype.name .. "_" .. defender.faction.alliance.id .. "_25.png")

  hud:GetText("u2-type_text"):SetLocalText(defender.prototype.name)

  -- Get correct stats
  local aAttack = ""
  local aDefense = ""
  local dAttack = ""
  local dDefense = ""
  local aBombard = GetStat(attacker, "bombard")
  local aAssault = GetStat(attacker, "assault")

	local aCommander = GetNearbyCommander(attacker)
	local dCommander = GetNearbyCommander(defender)

  local defenderPrototype
  if defender.prototype.lua.defenseType ~= nil then
    defenderPrototype = GetPrototypeByName(defender.prototype.lua.defenseType)
  else
    defenderPrototype = defender.prototype.lua
  end

  if attacker.prototype.lua.type == defenderPrototype.type and attacker.prototype.lua.class ~= "artillery" then
    aDefense = "basedefense"
    dDefense = "basedefense"
  else
    aDefense = "highdefense"
    dDefense = "highdefense"
  end

  if defenderPrototype.type == "land" then
    aAttack = "groundattack"
  elseif defenderPrototype.type == "air" then
    aAttack = "airattack"
  elseif defenderPrototype.name == "submarine" then
    aAttack = "subattack"
  else
    aAttack = "navalattack"
  end

  if attacker.prototype.lua.class ~= "artillery" then
    if attacker.type == Unit.LAND then
      dAttack = "groundattack"
    elseif attacker.type == Unit.AIR then
      dAttack = "airattack"
    elseif attacker.prototype.name == "submarine" then
      dAttack = "subattack"
    else
      dAttack = "navalattack"
    end
  end

	local aAttackValue = GetStat(attacker, aAttack)
  if aAttackValue ~= nil then
    aAttackValue = aAttackValue + GetCommanderBonus(aCommander, aAttack)
  else
    aAttackValue = 0
  end
	local dAttackValue = GetStat(defender, dAttack, defenderPrototype)
  if dAttackValue ~= nil then
    dAttackValue = dAttackValue + GetCommanderBonus(dCommander, dAttack)
  else
    dAttackValue = 0
  end

	local aDefenseValue = GetStat(attacker, aDefense) + GetCommanderBonus(aCommander, aDefense)
	local dDefenseValue = GetStat(defender, dDefense, defenderPrototype) + GetCommanderBonus(dCommander, dDefense)

  if HasRiver(defender.hex, defender.hex:GetDirection(attackerHex)) == true then
    dDefenseValue = dDefenseValue + gameplay.RiverBonus
  end

	hud:GetImage("attack1_image"):SetImage("graphics/hud", "stat-" .. aAttack .. ".png")
	if dAttack ~= "" then
		hud:GetImage("attack2_image"):Show()
		hud:GetText("attack2_text"):Show()
		hud:GetImage("attack2_image"):SetImage("graphics/hud", "stat-" .. dAttack .. ".png")
	else
		hud:GetImage("attack2_image"):Hide()
		hud:GetText("attack2_text"):Hide()
		dAttackValue = 0
	end

	hud:GetImage("defense1_image"):SetImage("graphics/hud", "stat-" .. aDefense .. ".png")
	hud:GetImage("defense2_image"):SetImage("graphics/hud", "stat-" .. dDefense .. ".png")

	if defender.prototype.name == "armour" then
    local bonus = GetStat(attacker, "armourattack")
    if bonus ~= nil then
      aAttackValue = aAttackValue + bonus
      hud:GetImage("attack1_image"):SetImage("graphics/hud", "stat-armourattack.png")
    end
	end
	if attacker.prototype.name == "armour" then
    local bonus = GetStat(defender, "armourattack", defenderPrototype)
    if bonus ~= nil then
      dAttackValue = dAttackValue + bonus
      hud:GetImage("attack2_image"):SetImage("graphics/hud", "stat-armourattack.png")
    end
	end

  hud:GetText("attack1_text"):SetNumber(aAttackValue)
  hud:GetText("defense1_text"):SetNumber(aDefenseValue)
  hud:GetText("attack2_text"):SetNumber(dAttackValue)
  hud:GetText("defense2_text"):SetNumber(dDefenseValue)

  if defender.type == Unit.LAND or attacker.type == Unit.LAND then
    hud:GetText("fortification_text"):SetNumber(GetStatEntrenchment(defender))
    hud:GetText("fortification_text"):Show()
    hud:GetImage("fortification_image"):Show()
    if aBombard ~= nil then
      hud:GetText("bonus_text"):Show()
      hud:GetImage("bonus_image"):Show()
      hud:GetImage("bonus_image"):SetImage("graphics/hud", "stat-bombard.png")
      hud:GetText("bonus_text"):SetNumber(aBombard + GetCommanderBonus(aCommander, "bombard"))
    end
    if aAssault ~= nil then
      hud:GetText("bonus_text"):Show()
      hud:GetImage("bonus_image"):Show()
      hud:GetImage("bonus_image"):SetImage("graphics/hud", "stat-assault.png")
      hud:GetText("bonus_text"):SetNumber(aAssault + GetCommanderBonus(aCommander, "assault"))
    end
  else
    hud:GetText("fortification_text"):Hide()
    hud:GetImage("fortification_image"):Hide()
    hud:GetText("bonus_text"):Hide()
    hud:GetImage("bonus_image"):Hide()
  end

   -- Bonuses
   local boost = 0

  -- Clear boost icons
  for i=0, 6 do
    panelID:GetImage("boost_" .. i .. "_image"):Hide()
  end

  -- Commander
  if aCommander ~= nil then
    panelID:GetImage("boost_0_image"):Show()
    panelID:GetImage("boost_0_image"):SetPosition(11,126)
    boost = boost + 1
  end

  -- Gas
  local gasattack = false
  if attacker.prototype.lua.class == "artillery" and HasUnitTech(attacker, "chlorine_gas") then
    panelID:GetImage("boost_gas_image"):Show()
    panelID:GetImage("boost_gas_image"):SetPosition(11+boost*26,126)
    boost = boost + 1
    gasattack = true
  else
    panelID:GetImage("boost_gas_image"):Hide()
  end

  -- Amphibious attack
  if attacker.hex == nil then
    panelID:GetImage("boost_amphibious_image"):Show()
    panelID:GetImage("boost_amphibious_image"):SetPosition(11+boost*26,126)
    boost = boost + 1
  else
    panelID:GetImage("boost_amphibious_image"):Hide()
  end

  -- Trample attack
  if attacker.prototype.lua.pursuitBonus ~= nil then
    panelID:GetImage("boost_trample_image"):Show()
    panelID:GetImage("boost_trample_image"):SetPosition(11+boost*26,126)
    boost = boost + 1
  else
    panelID:GetImage("boost_trample_image"):Hide()
  end

  -- Torpedo Boat Support
  if attacker.type == Unit.NAVAL and defender.type == Unit.NAVAL and HasHomeSupport(playerAlliance, defender.hex) and attacker.prototype.name ~= "submarine" then
    panelID:GetImage("boost_torpedoA_image"):Show()
    panelID:GetImage("boost_torpedoA_image"):SetPosition(11+boost*26,126)
    boost = boost + 1
  else
    panelID:GetImage("boost_torpedoA_image"):Hide()
  end

  -- FirstStrike bonus
  if attacker.prototype.lua.firstStrike == 1 then
    panelID:GetImage("boost_firstStrike_image"):Show()
    panelID:GetImage("boost_firstStrike_image"):SetPosition(11+boost*26,126)
    boost = boost + 1
  else
    panelID:GetImage("boost_firstStrike_image"):Hide()
  end
  
  ---Mud---
  local impassable = game:GetFactionById(24)

  if impassable.luaData.mud == game.turn and attacker.type == Unit.LAND and attacker.luaData.mud ~= nil and attacker.luaData.mud == game.turn and attacker.prototype.name ~= "artillery" then
	panelID:GetImage("boost_mud_image"):Show()
    panelID:GetImage("boost_mud_image"):SetPosition(11+boost*26,126)
    boost = boost + 1
	else     
	 panelID:GetImage("boost_mud_image"):Hide()

  
  end
  

  -- VS

  boost = 0
  -- Commander
  if dCommander ~= nil then
   panelID:GetImage("boost_5_image"):Show()
   panelID:GetImage("boost_5_image"):SetPosition(169,126)
   boost = boost + 1
  end

  -- River
  if HasRiver(defender.hex, defender.hex:GetDirection(attackerHex)) == true then
    panelID:GetImage("boost_4_image"):Show()
    panelID:GetImage("boost_4_image"):SetPosition(169-boost*26,126)
    boost = boost + 1
  end

  -- Port
  if attacker.type == Unit.NAVAL and defender.type == Unit.NAVAL and HasPortSupport(defender.faction.alliance, defender.hex) then
    panelID:GetImage("boost_port_image"):Show()
    panelID:GetImage("boost_port_image"):SetPosition(169-boost*26,126)
    boost = boost + 1
    panelID:GetImage("boost_torpedoD_image"):Hide()
  else
    panelID:GetImage("boost_port_image"):Hide()

    -- Torpedo Boat Support
    if attacker.type == Unit.NAVAL and defender.type == Unit.NAVAL and HasHomeSupport(defender.faction.alliance, defender.hex) and attacker.prototype.name ~= "submarine" then
      panelID:GetImage("boost_torpedoD_image"):Show()
      panelID:GetImage("boost_torpedoD_image"):SetPosition(169-boost*26,126)
      boost = boost + 1
    else
      panelID:GetImage("boost_torpedoD_image"):Hide()
    end
  end

  -- Gasmask
  if gasattack and HasUnitTech(defender, "cotton_pad_mask") then
    panelID:GetImage("boost_gasmask_image"):Show()
    panelID:GetImage("boost_gasmask_image"):SetPosition(169-boost*26,126)
    boost = boost + 1
  else
    panelID:GetImage("boost_gasmask_image"):Hide()
  end

  -- Quick retreat
  if defender.prototype.lua.quickRetreat ~= nil then
    panelID:GetImage("boost_quickretreat_image"):Show()
    panelID:GetImage("boost_quickretreat_image"):SetPosition(11+boost*26,126)
    boost = boost + 1
  else
    panelID:GetImage("boost_quickretreat_image"):Hide()
  end

end

function SelectionTab:DisplayStats(unit)

  -- If no unit specified, show current stats
  if unit ~= nil then
    RefreshUnitStats(unit, "stats_panel", 1)
  else
    hud:GetPanel("terrain_panel"):Show()
  end
  hud:GetPanel("stats_panel"):Show()
  hud:GetPanel("prognosis_panel"):Hide()

end


selection_tab =
{
  name = "selection_tab",
  type = "panel",
  position = {0, 0},
  image = "tabs_selection",


  controls =
  {
    {
      name = "stats_image",
      image = "selection_overview",
      position = {14, 28},
    },

    -- SELECTED UNIT IMAGE GROUP: Always visible
--~     {
--~       type = "image",
--~       name = "unit_terrain",
--~       position = {21, 252},
--~     },

    {
      name = "HUDunit_image",
      type = "image",
      position = {19, 52},
    },
    {
      name = "type_text",
      font = "unit_font",
      position = {84, 37},
      color   = {235, 232, 229},
      align = "center",
      tooltip = "rename_tooltip",
      sfx = "click",
      on_click = function(button)
        unit = GetSelectedUnit()
        if unit ~= nil and unit.faction.alliance.id == playerAlliance.id then
          --PopupRename(unit)
          ui.renamePanel:Show()
        end

        -- Reset cursor
        ResetCursorMode()
      end
    },
    {
      name = "class_image",
      position = {24, 57},
      image = "../units/infantry_2_25",
      tooltip = "unit_class",
    },
    {
      name = "level_image",
      position = {41, 57},
      image = "level1",
      tooltip = "unit_level",
    },
    {
      name = "detach_button",
      type = "button",
      position = {110, 56},
      image = "x",
      sfx = "click",
      tooltip = "detach_commander",
      on_click = function(button)
        local unit = GetSelectedUnit()
        if unit.supply == 2 then
          function DetachConfirm()
            DetachCommanderFromUnit(unit, unit.commander)
            ui.SelectHex(unit.hex)
          end
          PopupDialog("detachConfirm", DetachConfirm)
        else
          PopupMessage("detachFailSupply")
        end

        -- Reset cursor
        ResetCursorMode()
      end
    },
    {
      name = "info_button",
      type = "button",
      position = {125, 56},
      image = "info-icon",
      sfx = "click",
      tooltip = "unit_info",
      on_click = function(button)
        local unit = GetSelectedUnit()
        if unit ~= nil then
          ui.UnitInfoPopupPanel:Refresh(unit)
        end

        -- Reset cursor
        ResetCursorMode()
      end
    },
    {
      name = "nation_image",
      position = {24, 162-4},
      image = "../flags/emblem_germany100",
      tooltip = "unit_owner",
    },
    {
      name = "experience_image",
      position = {24, 146},
    },

    --stats_panel =
    {
      name = "stats_panel",
      position = {155, 28},
      visible = false,

      controls =
      {

        {
          name = "background",
          type = "image",
          position = {11, 21},
          --image = "selection_stats_default"
        },
        {
          name = "unitstats_text",
          type = "text",
          font = "unit_font",
          position = {103, 9},
          color   = {244, 238, 231},
          text = "unit_stats_text",
          align = "center"
        },

        {
          name = "basedefense_text",
          type = "text",
          font = "unit_font",
          position = {42, 35},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "basedefense_image",
          type = "image",
          position = {12, 22},
          image = "stat-box",
          tooltip = "stat_basedefense",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("basedefense")
          end
        },
        {
          name = "groundattack_text",
          type = "text",
          font = "unit_font",
          position = {42, 69},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "groundattack_image",
          type = "image",
          position = {11, 56},
          image = "stat-box",
          tooltip = "stat_groundattack",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("groundattack")
          end
        },
        {
          name = "shock_text",
          type = "text",
          font = "unit_font",
          position = {42, 103},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "shock_image",
          type = "image",
          position = {11, 90},
          image = "stat-box",
          tooltip = "stat_shock",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("shock")
          end
        },
        {
          name = "range_text",
          type = "text",
          font = "unit_font",
          position = {42, 137},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "range_image",
          type = "image",
          position = {11, 124},
          image = "stat-box",
          tooltip = "stat_range",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("range")
          end
        },
        {
          name = "highdefense_text",
          type = "text",
          font = "unit_font",
          position = {88, 35},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "highdefense_image",
          type = "image",
          position = {57, 22},
          image = "stat-box",
          tooltip = "stat_highdefense",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("highdefense")
          end
        },
        {
          name = "airattack_text",
          type = "text",
          font = "unit_font",
          position = {88, 69},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "airattack_image",
          type = "image",
          position = {57, 56},
          image = "stat-box",
          tooltip = "stat_airattack",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("airattack")
          end
        },
        {
          name = "bombard_text",
          type = "text",
          font = "unit_font",
          position = {88, 103},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "bombard_image",
          type = "image",
          position = {57, 90},
          image = "stat-box",
          tooltip = "stat_bombard",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("bombard")
          end
        },
        {
          name = "assault_image",
          type = "image",
          position = {57, 90},
          image = "stat-box",
          tooltip = "stat_assault",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("assault")
          end
        },
        {
          name = "mp_text",
          type = "text",
          font = "unit_font",
          position = {88, 137},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "mp_image",
          type = "image",
          position = {57, 123},
          image = "stat-box",
          tooltip = "stat_mp",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("mp")
          end
        },
        	        {
          name = "endurance_image",
          type = "image",
          position = {149, 54},
          image = "stat-box",
          tooltip = "stat_endurance1",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("endurance")
          end
        },
		
		{
          name = "entrenchment_text",
          type = "text",
          font = "unit_font",
          position = {134, 35},
          color   = {31, 21, 6},
          align = "center"
        },
        
		
		{
          name = "entrenchment_image",
          type = "image",
          position = {103, 22},
          image = "stat-box",
          tooltip = "stat_entrenchment",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("entrenchment")
          end
        },
	
		
        {
          name = "navalattack_text",
          type = "text",
          font = "unit_font",
          position = {134, 69},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "navalattack_image",
          type = "image",
          position = {103, 56},
          image = "stat-box",
          tooltip = "stat_navalattack",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("navalattack")
          end
        },
        {
          name = "ammo_text",
          type = "text",
          font = "unit_font",
          position = {134, 103},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "ammo_image",
          type = "image",
          position = {103, 90},
          image = "stat-box",
          tooltip = "stat_ammo",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("ammo")
          end
        },
        {
          name = "los_text",
          type = "text",
          font = "unit_font",
          position = {134, 137},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "los_image",
          type = "image",
          position = {103, 124},
          image = "stat-box",
          tooltip = "stat_los",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("los")
          end
        },
        {
          name = "efficiency_text",
          type = "text",
          font = "unit_font",
          position = {180, 35},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "efficiency_image",
          type = "image",
          position = {149, 22},
          image = "stat-box",
          tooltip = "stat_efficiency",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("efficiency")
          end
        },
        {
          name = "armourattack_text",
          type = "text",
          font = "unit_font",
          position = {180, 69},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "armourattack_image",
          type = "image",
          position = {149, 54},
          image = "stat_armourattack",
          tooltip = "stat_armourattack",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("armourattack")
          end
        },
        {
          name = "counterattack_image",
          type = "image",
          position = {149, 54},
          image = "stat_counterattack",
          tooltip = "stat_counterattack",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("counterattack")
          end
        },
        {
          name = "subattack_image",
          type = "image",
          position = {149, 54},
          image = "stat_subattack",
          tooltip = "stat_subattack",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("subattack")
          end
        },
        {
          name = "stratattack_text",
          type = "text",
          font = "unit_font",
          position = {180, 103},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "stratattack_image",
          type = "image",
          position = {149, 90},
          image = "stat-box",
          tooltip = "stat_stratattack",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("stratattack")
          end
        },
        {
          name = "quality_text",
          type = "text",
          font = "unit_font",
          position = {180, 137},
          color   = {31, 21, 6},
          align = "center"
        },
        {
          name = "quality_image",
          type = "image",
          position = {149, 124},
          image = "stat-box",
          tooltip = "stat_quality",
          sfx = "click",
          on_click = function()
            ui.advicePanel:AddAdviceForced("quality")
          end
        },
      }
    },

    -- SELECTED HEX TERRAIN GROUP - visible in unit stats mode
    {
      name = "terrain_panel",
      position = {362, 0},

      controls =
      {
        {
          name = "terrain_image",
          position = {3, 52},
        },
        {
          name = "terrain_text",
          font = "unit_font",
          position = {67, 37},
          color   = {235, 232, 229},
          text = "terrain0",
          align = "center"
        },
        {
          name = "ter-info_image",
          type = "button",
          position = {109, 56},
          image = "info-icon",
          sfx = "click",
          tooltip = "terrain_info",
          on_click = function(button)
            local hex = GetSelectedHex()
            if hex ~= nil then
              ui.TerrainInfoPopupPanel:Refresh(hex)
            end
          end
        },
        {
          name = "ter-owner_image",
          position = {8, 162-4},
          image = "../flags/emblem_germany100",
          --tooltip = "terrain_owner",
        },
        {
          name = "ter-nationality_image",
          position = {114-4, 162-4},
          image = "../flags/emblem_germany100",
          --tooltip = "terrain_nationality",
        }
      }
    },


    prognosis_panel =
    {
      name = "prognosis_panel",
      position = {155, 28},
      visible = false,
      controls =
      {

        {
          name = "background",
          type = "image",
          position = {60, 20},
          image = "prognosis_bg"
        },

        {
          name = "prognosis_text",
          font = "unit_font",
          position = {103, 9},
          color   = {244, 238, 231},
          text = "prognosis_text",
          align = "center"
        },

        {
          name = "u2_image",
          position = {211, 22},
        },
        {
          name = "u2-type_text",
          font = "unit_font",
          position = {274, 9},
          color   = {235, 232, 229},
          align = "center"
        },
        {
          name = "u2-class_image",
          position = {215, 29},
          image = "../units/infantry_2_25",
        },
        {
          name = "u2-level_image",
          position = {232, 29},
          image = "level1",
        },
        {
          name = "u2-nation_image",
          position = {215, 130},
          image = "../flags/emblem_germany100",
        },

        {
          name = "attack1_image",
          position = {11, 21},
          image = "prog-att",
        },
        {
          name = "attack1_text",
          font = "unit_font",
          position = {42, 35},
          color   = {43, 27, 9},
          align = "center",
        },
        {
          name = "defense1_image",
          position = {11, 55},
          image = "prog-def",
        },
        {
          name = "defense1_text",
          font = "unit_font",
          position = {42, 69},
          color   = {70, 50, 33},
          align = "center",
        },
        {
          name = "bonus_image",
          position = {11, 89},
          image = "stat-shock",
        },
        {
          name = "bonus_text",
          font = "unit_font",
          position = {42, 103},
          color   = {70, 50, 33},
          align = "center",
        },
        {
          name = "attack2_image",
          position = {149, 21},
          image = "prog-att",
        },
        {
          name = "attack2_text",
          font = "unit_font",
          position = {180, 35},
          color   = {70, 50, 33},
          align = "center",
        },
        {
          name = "defense2_image",
          position = {149, 55},
          image = "prog-def",
        },
        {
          name = "defense2_text",
          font = "unit_font",
          position = {180, 69},
          color   = {70, 50, 33},
          align = "center",
        },
        {
          name = "fortification_image",
          position = {149, 89},
          image = "stat-entrench",
        },
        {
          name = "fortification_text",
          font = "unit_font",
          position = {180, 103},
          color   = {70, 50, 33},
          align = "center",
        },

        {
          name = "kills_text",
          font = "prognosis_font",
          position = {81, 52},
          color   = {70, 50, 33},
          align = "center",
        },
        {
          name = "casualties_text",
          font = "prognosis_font",
          position = {126, 52},
          color   = {70, 50, 33},
          align = "center",
        },

        {
          name = "damage3_text",
          font = "hud16_font",
          position = {81, 92},
          color   = {70, 50, 33},
          align = "center",
        },
        {
          name = "damage4_text",
          font = "hud16_font",
          position = {126, 92},
          color   = {70, 50, 33},
          align = "center",
        }
      }
    }
  }
}


------------------------------------------------------------------

class 'ImprovementsTab' (LuaPanel)

function ImprovementsTab:__init()
  LuaPanel.__init(self)

  self:SetPosition(256, 30)
  self:SetBackgroundImage("tab_improvements")

  self.unitInfo = self:AddWidget({ type = "panel", position = {0, 0}})

  self.unitInfo:AddWidget({
    name = "HUDunit_image",
    type = "image",
    position = {19, 30},
  })
  self.unitInfo:AddWidget({
    name = "type_text",
    font = "unit_font",
    position = {84, 15},
    color   = {235, 232, 229},
    align = "center",
  })
  self.unitInfo:AddWidget({
    name = "class_image",
    position = {24, 35},
  })
  self.unitInfo:AddWidget({
    name = "level_image",
    position = {41, 35},
  })
  self.unitInfo:AddWidget({
    name = "nation_image",
    position = {24, 136},
  })

  self.terrain = self:AddWidget({ type = "panel", position = {362, 0}})

  self.terrain:AddWidget({
    name = "terrain_image",
    position = {3, 30},
  })
  self.terrain:AddWidget({
    name = "terrain_text",
    font = "unit_font",
    position = {67, 15},
    color   = {235, 232, 229},
    align = "center"
  })
  self.terrain:AddWidget({
    name = "ter-owner_image",
    position = {8, 136},
  })
  self.terrain:AddWidget({
    name = "ter-nationality_image",
    position = {110, 136},
  })

  self.stats = self:AddWidget({type = "panel", position = {154,6}})

  self.stats:AddWidget({
    name = "stats_display",
    type = "image",
    position = {12, 21},
  })

  self.stats:AddWidget({
    name = "unitstats_text",
    type = "text",
    font = "unit_font",
    position = {101, 9},
    color   = {244, 238, 231},
    text = "improvements_text",
    align = "center"
  })

  self.stats.basedefense = self.stats:AddWidget({
    name = "basedefense_text",
    type = "text",
    font = "unit_font",
    position = {42, 35},
    color   = {31, 21, 6},
    align = "center"
  })
  self.stats.groundattack = self.stats:AddWidget({
    name = "groundattack_text",
    type = "text",
    font = "unit_font",
    position = {42, 69},
    color   = {31, 21, 6},
    align = "center"
  })
  self.stats.shock = self.stats:AddWidget({
    name = "shock_text",
    type = "text",
    font = "unit_font",
    position = {42, 103},
    color   = {31, 21, 6},
    align = "center"
  })
  self.stats:AddWidget({
    name = "range_text",
    type = "text",
    font = "unit_font",
    position = {42, 137},
    color   = {31, 21, 6},
    align = "center",
    text = "-"
  })
  self.stats.highdefense = self.stats:AddWidget({
    name = "highdefense_text",
    type = "text",
    font = "unit_font",
    position = {88, 35},
    color   = {31, 21, 6},
    align = "center"
  })
  self.stats.airattack = self.stats:AddWidget({
    name = "airattack_text",
    type = "text",
    font = "unit_font",
    position = {88, 69},
    color   = {31, 21, 6},
    align = "center"
  })
  self.stats.bombard = self.stats:AddWidget({
    name = "bombard_text",
    type = "text",
    font = "unit_font",
    position = {88, 103},
    color   = {31, 21, 6},
    align = "center"
  })
  self.stats.assault = self.stats.bombard
  self.stats:AddWidget({
    name = "mp_text",
    type = "text",
    font = "unit_font",
    position = {88, 137},
    color   = {31, 21, 6},
    align = "center",
    text = "-"
  })
  self.stats:AddWidget({
    name = "entrenchment_text",
    type = "text",
    font = "unit_font",
    position = {134, 35},
    color   = {31, 21, 6},
    align = "center",
    text = "-"
  })
  self.stats.navalattack = self.stats:AddWidget({
    name = "navalattack_text",
    type = "text",
    font = "unit_font",
    position = {134, 69},
    color   = {31, 21, 6},
    align = "center"
  })
  self.stats:AddWidget({
    name = "ammo_text",
    type = "text",
    font = "unit_font",
    position = {134, 103},
    color   = {31, 21, 6},
    align = "center",
    text = "-"
  })
  self.stats:AddWidget({
    name = "los_text",
    type = "text",
    font = "unit_font",
    position = {134, 137},
    color   = {31, 21, 6},
    align = "center",
    text = "-"
  })
  self.stats:AddWidget({
    name = "efficiency_text",
    type = "text",
    font = "unit_font",
    position = {180, 35},
    color   = {31, 21, 6},
    align = "center",
    text = "-"
  })
  self.stats.armourattack = self.stats:AddWidget({
    name = "armourattack_text",
    type = "text",
    font = "unit_font",
    position = {180, 69},
    color   = {31, 21, 6},
    align = "center"
  })
  self.stats.subattack = self.stats.armourattack
  self.stats.counterattack = self.stats.armourattack
  self.stats.stratattack = self.stats:AddWidget({
    name = "stratattack_text",
    type = "text",
    font = "unit_font",
    position = {180, 103},
    color   = {31, 21, 6},
    align = "center"
  })
  self.stats:AddWidget({
    name = "quality_text",
    type = "text",
    font = "unit_font",
    position = {180, 137},
    color   = {31, 21, 6},
    align = "center",
    text = "-"
  })
  self.stats:AddWidget({
    name = "armourattack_image",
    type = "image",
    position = {149, 54},
  })
  self.stats:AddWidget({
    name = "mp_image",
    type = "image",
    position = {57, 123},
  })

  self:Hide()

end

function ImprovementsTab:Display(unit, improvements)

  local hex = unit.hex

  local stats =
  {
    "basedefense",
    "highdefense",
    "groundattack",
    "airattack",
    "navalattack",
    "armourattack",
    "subattack",
    "counterattack",
    "stratattack",
    "shock",
    "bombard",
    "assault"
  }

  for _, statName in pairs(stats) do
    if improvements[statName] ~= nil then
      if improvements[statName] > 0 then
        self.stats[statName]:SetText("+" .. improvements[statName])
        self.stats[statName]:SetColour(70, 104, 10, 255)
      else
        self.stats[statName]:SetNumber(improvements[statName])
        self.stats[statName]:SetColour(137, 31, 3, 255)
      end
    else
      self.stats[statName]:SetText("-")
      self.stats[statName]:SetColour(69, 40, 23, 255)
    end
  end

  -- Unit faction
  self.unitInfo:GetImage("nation_image"):SetImage("graphics/flags", "emblem_" .. unit.faction.name .. "100.png")

  local unitImage
  if engine:FileExists("graphics/units", unit.prototype.name .. "_" .. unit.faction.name .. "_" .. unit.level .. "_hud.png") then
    unitImage = unit.prototype.name .. "_" .. unit.faction.name .. "_" .. unit.level .. "_hud.png"
  else
    unitImage = unit.prototype.name .. "_default_" .. unit.level .. "_hud.png"
  end
  self.unitInfo:GetImage("HUDunit_image"):SetImage("graphics/units", unitImage)

  if unit.params:Get("name") ~= nil and unit.faction.alliance.id == playerAlliance.id then
    self.unitInfo:GetText("type_text"):SetText(unit.params:Get("name"))
  else
    self.unitInfo:GetText("type_text"):SetLocalText(unit.prototype.name)
  end
  self.unitInfo:GetImage("class_image"):SetImage("graphics/units", unit.prototype.name .. "_" .. unit.faction.alliance.id .. "_25.png")
  self.unitInfo:GetImage("level_image"):SetImage("graphics/hud", "level" .. GetUnitTechCount(unit) .. ".png")


  if hex.construction == nil then
    self.terrain:GetImage("terrain_image"):SetImage("graphics/terrain", "terrain_" .. hex.terrain.name .. ".png")
    self.terrain:GetText("terrain_text"):SetLocalText(hex.terrain.name)
  else
    if hex.construction.type == Construction.TYPE_PORT then
      self.terrain:GetImage("terrain_image"):SetImage("graphics/terrain", "terrain_port.png")
      self.terrain:GetText("terrain_text"):SetLocalText("port")
    end
    if hex.construction.type == Construction.TYPE_FORTRESS then
      self.terrain:GetImage("terrain_image"):SetImage("graphics/terrain", "terrain_fortress.png")
      self.terrain:GetText("terrain_text"):SetLocalText("fortress")
    end
    if hex.construction.type == Construction.TYPE_CITY then
      self.terrain:GetImage("terrain_image"):SetImage("graphics/terrain", "terrain_city.png")
      self.terrain:GetText("terrain_text"):SetLocalText("city")
    end
    if hex.construction.type == Construction.TYPE_CAPITAL then
      hud:GetImage("terrain_image"):SetImage("graphics/terrain", "terrain_capital.png")
      hud:GetText("terrain_text"):SetLocalText("capital")
    end
  end

  if hex.faction ~= nil then
    self.terrain:GetImage("ter-owner_image"):Show()
    self.terrain:GetImage("ter-owner_image"):SetImage("graphics/flags", "emblem_" .. hex.faction.name .. "100.png")
    self.terrain:GetImage("ter-owner_image").tooltip = "terrain_owner_" .. hex.faction.name

    self.terrain:GetImage("ter-nationality_image"):Show()
    self.terrain:GetImage("ter-nationality_image"):SetImage("graphics/flags", "emblem_" .. hex.originalFaction.name .. "100.png")
    self.terrain:GetImage("ter-nationality_image").tooltip = "terrain_nationality_" .. hex.originalFaction.name
  else
    self.terrain:GetImage("ter-owner_image"):Hide()
    self.terrain:GetImage("ter-nationality_image"):Hide()
  end

  -- Variable icons
 

 if unit.type == Unit.LAND and unit.prototype.name ~= "artillery" then
    self.stats:GetImage("stats_display"):SetImage("graphics/hud", "selection_stats_assault.png")
  elseif unit.type ~= Unit.NAVAL then
    self.stats:GetImage("stats_display"):SetImage("graphics/hud", "selection_stats_bombard.png")
  end
  if unit.prototype.name == "artillery" then
    self.stats:GetImage("armourattack_image"):SetImage("graphics/hud", "stat_counterattack.png")
  elseif unit.prototype.name == "zeppelin" then
    self.stats:GetImage("armourattack_image"):SetImage("graphics/hud", "stat_subattack.png")
  else
    self.stats:GetImage("armourattack_image"):SetImage("graphics/hud", "stat_armourattack.png")
  end
 
  self.stats:GetImage("mp_image"):SetImage("graphics/hud", "stat_chassis_" .. unit.prototype.lua.chassis .. ".png")

  self:Show()

end

