class 'TurnReportPanel' (LuaPanel)

function TurnReportPanel:__init()
  LuaPanel.__init(self, reportpanel)

  for alliance in game.alliances do
    alliance.luaData.report = {}
    alliance.luaData.reportImage = {}
    alliance.luaData.reportCount = 1
  end

--~ 	local techId = 0
--~ 	while data.newTechnologies[techId+1] ~= nil do
--~ 		for faction in playerAlliance.factions do
--~ 			if faction.luaData.researchComplete[techId] > 0 then
--~ 				printLog("defined " .. techId+100*faction.id)
--~ 				ui.completedTechs[techId+100*faction.id] = game.turn
--~ 			end
--~ 		end
--~ 		techId = techId + 1
--~ 	end

  -- Add dynamic content to panel
  self.content = self:AdoptWidget(TurnReportContent())
end

function TurnReportPanel:OnShow()

  ui.disableControls = false

  self.content:Refresh()

  -- Auto Save
--~   if game.type == Game.TYPE_SINGLE then
--~     if settings.autosave == 1 then
--~       playerAlliance.luaData.camera = {}
--~       playerAlliance.luaData.camera["x"] = GetPlayer().camera.x
--~       playerAlliance.luaData.camera["y"] = GetPlayer().camera.y
--~       engine:SaveGame("AutoSave")
--~     end
--~   end

  ShowManpowerWarnings()
  ShowCommanderWoundedMessages()
end

function ShowManpowerWarnings()
  for faction in playerAlliance.factions do
    if faction.luaData.showManpowerWarning == 1 then
      faction.luaData.showManpowerWarning = 0
      PopupManpowerDrop(faction)
    end
  end
end

function ShowCommanderWoundedMessages()
  if playerAlliance.luaData.popupCommanderWounded == 1 then
    playerAlliance.luaData.popupCommanderWounded = 0
    PopupMessage("commander_wounded")
  end

  if playerAlliance.luaData.popupCommanderCaptured == 1 then
    playerAlliance.luaData.popupCommanderCaptured = 0
    PopupMessage("commander_captured")
  end
end

reportpanel =
{
  name = "report_panel",
  type = "panel",
  position = {(engine.window.width)/2-228, (engine.window.height-200)/2-270},
  image = "turn_report",
  visible = false,

  controls =
  {
    {
      name = "tab_title",
      type = "text",
      font = "unit_font",
      position = {72, 23},
      align = "center",
      color   = {238, 229, 220},
      text = "turn_report"
    },
    {
      name = "turn_title",
      type = "text",
      position = {116, 66},
      align = "left",
      color   = {238, 229, 220},
      text = "turn"
    },
    {
      name = "event_title",
      type = "text",
      position = {45, 125},
      align = "left",
      color   = {238, 229, 220},
      text = "events"
    },
    {
      name = "techs_title",
      type = "text",
      position = {45, 287},
      align = "left",
      color   = {238, 229, 220},
      text = "techs_completed"
    },
    {
      name = "units_title",
      type = "text",
      position = {45, 387},
      align = "left",
      color   = {238, 229, 220},
      text = "units_completed"
    },
    {
      name = "save",
      type = "button",
      image = "button_default",
      position = {30, 488},
      font = "menu_font",
      text = "save",
      sfx = "click",
      Clicked = function(this)
        ui.optionsPanel:Display()
      end
    },
    {
      name = "save_label",
      type = "text",
      position = {182, 496},
      align = "left",
      color   = {238, 229, 220},
      text = "autosave"
    },
    {
      name = "continue",
      type = "button",
      image = "button_default",
      position = {324, 488},
      font = "menu_font",
      text = "continue",
      sfx = "click",
      Clicked = function(this)
        this.parent:Hide()

        local prodLostPopup = false
        if game.turn > 1 then
          for faction in playerAlliance.factions do
            if faction.luaData.productionLost == game.turn then
              PopupProductionLost(faction)
              prodLostPopup = true
            end
          end
        end

        if not prodLostPopup then
          for faction in playerAlliance.factions do
            if faction.luaData.commanders ~= nil then
              for i, commander in pairs(faction.luaData.commanders) do
                if commander.unlocked == game.turn and game.turn ~= 1 then
                  PopupCommander(faction, i)
                end
              end
            end
          end

          -- Advice messages
          AdviceTurnStart()
        end
      end
    },
    {
      name = "autosave_checkbox_off",
      type = "button",
      image = "bigcheck_off",
      position = {153, 490},
      sfx = "click",
      on_click = function(button)
        button.parent:GetButton("autosave_checkbox_on"):Show()
        button:Hide()
        settings.autosave = 1
        SetUserInt("autosave", settings.autosave)
      end
    },
    {
      name = "autosave_checkbox_on",
      type = "button",
      image = "bigcheck_on",
      position = {153, 490},
      sfx = "click",
      visible = false,
      on_click = function(button)
        button.parent:GetButton("autosave_checkbox_off"):Show()
        button:Hide()
        settings.autosave = 0
        SetUserInt("autosave", settings.autosave)

        --if settings.autosave == 1 then
          playerAlliance.luaData.camera = {}
          playerAlliance.luaData.camera["x"] = GetPlayer().camera.x
          playerAlliance.luaData.camera["y"] = GetPlayer().camera.y
          SaveGame("AutoSave")
        --end
      end
    },
  }
}

----------------------------------------------------------

class 'TurnReportContent' (LuaPanel)

function TurnReportContent:__init()
  LuaPanel.__init(self)
end

function TurnReportContent:OnShow()
end

function TurnReportContent:Refresh()

  -- Clear
  self:RemoveWidgets()

  local day, month, year = GetTurnDate()

  self:AddWidget({
    name = "turn_number",
    type = "text",
    position = {166, 66},
    align = "left",
    color   = {238, 229, 220},
    text = game.turn .. " / " .. scenario.turnLimit
  })

  self:AddWidget({
    name = "date_day",
    type = "text",
    position = {116, 86},
    color   = {238, 229, 220},
    align = "left",
    text = "" .. day
  })
  self:AddWidget({
    name = "date_month",
    type = "text",
    position = {135, 86},
    color   = {238, 229, 220},
    align = "left",
    text = "month" .. month .. "_full"
  })
  self:AddWidget({
    name = "date_year",
    type = "text",
    position = {220, 86},
    color   = {238, 229, 220},
    align = "left",
    text = "" .. year
  })

  -- Events list
  local i = 1
  local maxEvents = 6
  while i < playerAlliance.luaData.reportCount and i <= maxEvents do
    local eventString = playerAlliance.luaData.report[i]
    self:AddWidget({
      type = "text",
      text = eventString,
      color = {67, 56, 47},
      position = {63, 157 + (i-1) * 20},
    })
    self:AddWidget({
      type = "image",
      image = playerAlliance.luaData.reportImage[i],
      position = {42, 158 + (i-1) * 20},
    })
    i = i + 1
  end
  if playerAlliance.luaData.reportCount == 1 then
    self:AddWidget({
      type = "text",
      text = "no_events",
      color = {67, 56, 47},
      position = {63, 157},
    })
  end
  playerAlliance.luaData.reportCount = 1

  -- Completed units list
  local count = 0
  local maxSlots = 5
  for faction in playerAlliance.factions do
    for unit in faction.production_que do
      if unit.turns == 0 then
        self:AdoptWidget(CompletedUnitBox(faction, unit, count))
        count = count + 1
        if count == maxSlots then
          break
        end
      end
    end
    if count == maxSlots then
      break
    end
  end
  if count == 0 then
    self:AddWidget({
      type = "text",
      text = "no_developments",
      color = {67, 56, 47},
      position = {63, 417},
    })
  end

  -- Techs list
  local techId = 0
  count = 0
  for faction in playerAlliance.factions do
    for techName, _ in pairs(data.newTechnologies) do
      if game.turn > 1 and faction.luaData.researchComplete[techName] == game.turn then
        self:AdoptWidget(CompletedTechBox(faction, techName, count))
        count = count + 1
        if count == maxSlots then
          break
        end
      end
    end
    if count == maxSlots then
      break
    end
  end
  if count == 0 then
    self:AddWidget({
      type = "text",
      text = "no_developments",
      color = {67, 56, 47},
      position = {63, 317},
    })
  end

end


----------------------------------------------------------

function AddToReportQueue(alliance, text, image)
  alliance.luaData.report[alliance.luaData.reportCount] = text
  alliance.luaData.reportImage[alliance.luaData.reportCount] = image
  alliance.luaData.reportCount = alliance.luaData.reportCount + 1
end

----------------------------------------------------------

class 'CompletedUnitBox' (LuaPanel)

function CompletedUnitBox:__init(faction, unit, step)
  LuaPanel.__init(self)

  self.faction = faction
  self.unit = unit

  --self:AddWidget({type = "image", image = "queue"})

  self:SetPosition(30 + 80*step, 406)

  local unitImage
  if engine:FileExists("graphics/units", unit.prototype.name .. "_" .. faction.name .. "_" .. unit.level .. "_queue.png") then
    unitImage = unit.prototype.name .. "_" .. faction.name .. "_" .. unit.level .. "_queue"
  else
    unitImage = unit.prototype.name .. "_default_" .. unit.level .. "_queue"
  end

  self:AddWidget({
    type = "image",
    position = {8, 5},
    image = "../units/" .. unitImage,
    tooltip = unit.prototype.name
  })

  self:AddWidget({
    type = "image",
    position = {7, 57},
    image = "../flags/flag_" .. faction.name .. "_16",
    tooltip = faction.name
  })

end

function CompletedUnitBox:OnShow()

end

----------------------------------------------------------

class 'CompletedTechBox' (LuaPanel)

function CompletedTechBox:__init(faction, techName, step)
  LuaPanel.__init(self)

  self.faction = faction
  self.techName = techName

  --self:AddWidget({type = "image", image = "queue"})

  self:SetPosition(30 + 80*step, 306)

  self:AddWidget({
    type = "image",
    position = {7, 57},
    image = "../flags/flag_" .. faction.name .. "_16",
    tooltip = faction.name
  })

  techImage = techName .. "_queue"
  if not engine:FileExists("graphics/techs", techImage .. ".png") then
    techImage = "camouflage_queue"
  end

  self:AddWidget({
    name = "tech_icon",
    type = "image",
    position = {7, 5},
    image = "../techs/" .. techImage,
    tooltip = "tech_" .. techName
  })

end

function CompletedTechBox:OnShow()

end
